import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Plugin {
    id: container

    // Required! Function to check if all required settings are set
    function isFinished() {
        // Check if the username is entered
        if ( username.text.length == 0 ) {
            showBanner(qsTrId("qtn_web2sms_genericaccountsettings_banner_enter_username"));
            return false;
        }

        // Check if the password is entered
        if ( password.text.length == 0 ) {
            showBanner(qsTrId("qtn_web2sms_genericaccountsettings_banner_enter_password"));
            return false;
        }

        // Ensure the user did choose a provider
        if (providerDialog.selectedIndex == -1) {
            //% "Select a provider"
            showBanner(qsTrId("qtn_betamax_no_provider"));
            return false;
        }

        if ( sender.text.length == 0 ) {
            //% "Enter sender phone number"
            showBanner(qsTrId("qtn_betamax_enter_sender"));
            return false;
        }

        // Get the selected provider
        var item = providerList.get(providerDialog.selectedIndex);

        // Update the display name
        account.displayName = username.text + " | " + item.name;

        // Update the plugin credentials
        var credentials = {
            "PROVIDER_ID":item.providerId,
            "SENDER":sender.text
        };

        account.pluginCredentials = credentials;

        return true;
    }

    function didSelectProvider() {
        // Get the selected item
        var item = providerList.get(providerDialog.selectedIndex);

        // Update the button label
        tumblerButton.text = item.name;
    }

    Component.onCompleted: {
        // Make the default selection
        var credentials = account.pluginCredentials;
        if ( typeof(credentials) != "object" )
            return;

        var providerId = credentials.PROVIDER_ID;
        var senderNumber = credentials.SENDER;

        // Update the sender number
        if ( typeof(senderNumber) == "string" ) {
            sender.text = senderNumber;
        }

        console.log("Initialized betamax with", providerId);

        // Don't continue if this is a new account
        if (typeof(providerId) != "string")
            return;

        for(var i=0; i < providerList.count; ++i) {
            var item = providerList.get(i);

            if ( item.providerId == providerId ) {
                console.log("did preselect row", i);

                // Update the selection
                providerDialog.selectedIndex = i;
                didSelectProvider();
                return;
            }
        }
    }

    Label {
        //% "Provider"
        text: qsTrId("qtn_web2sms_betamax_provider_name")
        color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
    }

    TumblerButton {
        id: tumblerButton

        width: parent.width
        //% "Choose Betamax provider"
        text: qsTrId("qtn_betamax_choose_providers")
        onClicked: providerDialog.open()

        ListModel {
            id: providerList

            ListElement {providerId:"voipgain"; name:"VoipGain.com"}
            ListElement {providerId:"voipcheap"; name:"VoipCheap.com"}
            ListElement {providerId:"smsdiscount"; name:"SmsDiscount.com"}
            ListElement {providerId:"lowratevoip"; name:"LowrateVoip.com"}
            ListElement {providerId:"poivy"; name:"Poivy.com"}
            ListElement {providerId:"12voip"; name:"12Voip.com" }
            ListElement {providerId:"actionvoip"; name:"ActionVoip.com" }
            ListElement {providerId:"budgetsip"; name:"BudgetSip.com" }
            ListElement {providerId:"calleasy"; name:"CallEasy.com" }
            ListElement {providerId:"dialnow"; name:"DialNow.com" }
            ListElement {providerId:"freecall"; name:"FreeCall.com" }
            ListElement {providerId:"internetcalls"; name:"InternetCalls.com" }
            ListElement {providerId:"intervoip"; name:"InterVoip.com" }
            ListElement {providerId:"jumblo"; name:"Jumblo.com" }
            ListElement {providerId:"justvoip"; name:"JustVoip.com" }
            ListElement {providerId:"netappel"; name:"Netappel.fr" }
            ListElement {providerId:"nonoh"; name:"Nonoh.net" }
            ListElement {providerId:"rynga"; name:"Rynga.com" }
            ListElement {providerId:"sipdiscount"; name:"SipDiscount.com" }
            ListElement {providerId:"smartvoip"; name:"SmartVoip.com" }
            ListElement {providerId:"smslisto"; name:"SmsListo.com" }
            ListElement {providerId:"telbo"; name:"Telbo.com" }
            ListElement {providerId:"sparvoip"; name:"SparVoip.com" }
            ListElement {providerId:"voipbuster"; name:"VoipBuster.com" }
            ListElement {providerId:"voipbusterpro"; name:"VoipBusterPro.com" }
            ListElement {providerId:"voipcheap"; name:"VoipCheap.co.uk" }
            ListElement {providerId:"voipian"; name:"Voipian.com" }
            ListElement {providerId:"voipraider"; name:"VoipRaider.com" }
            ListElement {providerId:"voipstunt"; name:"VoipStunt.com" }
            ListElement {providerId:"voipwise"; name:"VoipWise.com" }
            ListElement {providerId:"voipzoom"; name:"VoipZoom.com" }
            ListElement {providerId:"webcalldirect"; name:"WebCallDirect.com" }
        }

        SelectionDialog {
            id: providerDialog

            //% "Betamax providers"
            titleText: qsTrId("qtn_betamax_tumbler_title")
            model: providerList
            onAccepted: didSelectProvider()
        }
    }

    Label {
        text: qsTrId("qtn_web2sms_genericaccountsettings_label_username")
        color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
    }

    TextField {
        id: username
        width: container.width
        text: account.username

        Binding {
            target: account
            property: "username"
            value: username.text
        }
    }

    Label {
        text: qsTrId("qtn_web2sms_genericaccountsettings_label_password")
        color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
    }

    TextField {
        id: password
        width: container.width
        echoMode: TextInput.Password
        text: account.password

        Binding {
            target: account
            property: "password"
            value: password.text
        }
    }

    Label {
        //% "Sender phone number"
        text: qsTrId("qtn_betamax_label_sender")
        color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
    }

    TextField {
        id: sender
        width: container.width
        //% "+49xxxxxxx"
        placeholderText: qsTrId("qtn_betamax_placeholder_sender")
    }
}
