// Local includes
#include "pluginxmlfile.h"

// Global includes
#include <QSharedData>
#include <QDomDocument>
#include <QFile>
#include <QDir>
#include <QFileInfo>

class PluginXmlFileData : public QSharedData
{
public:
    PluginXmlFileData()
    {
    }

    PluginXmlFileData(const PluginXmlFileData& rhs)
    : QSharedData(rhs)
    {
        errorString = rhs.errorString;
    }

    QString errorString;

    // XML Data
    QString providerId;
    QString pluginPath;
};

PluginXmlFile::PluginXmlFile()
: data(new PluginXmlFileData)
{
}

PluginXmlFile::PluginXmlFile(const QString &absolutePath)
: data(new PluginXmlFileData)
{
    loadFromFile(absolutePath);
}

PluginXmlFile::PluginXmlFile(const PluginXmlFile &rhs)
: data(rhs.data)
{
}

PluginXmlFile &PluginXmlFile::operator=(const PluginXmlFile &rhs)
{
    if (this != &rhs)
        data.operator=(rhs.data);
    return *this;
}

PluginXmlFile::~PluginXmlFile()
{
}

bool PluginXmlFile::loadFromFile(const QString &absolutePath)
{
    // Open the input file
    QFile file(absolutePath);
    if ( file.open(QIODevice::ReadOnly) == false )
        return false;

    // Try to parse the xml file
    QDomDocument document;
    if ( document.setContent(&file, &data->errorString) == false )
        return false;

    QDomElement docElement = document.documentElement();
    if ( docElement.hasAttribute("id") == false ) {
        data->errorString = "document element has no 'id' attribute";
        return false;
    }

    // Read the provider id
    data->providerId = docElement.attribute("id");

    QDomNodeList pluginElements = document.elementsByTagName("plugin");
    if ( pluginElements.size() != 1 ) {
        data->errorString = "only one 'plugin' element is allowed, none or multiple found";
        return false;
    }

    QDomElement pluginElement = pluginElements.at(0).toElement();
    QString pluginName = pluginElement.text();

    // Get the directory
    QDir directory = QFileInfo(absolutePath).dir();
    data->pluginPath = directory.absoluteFilePath( pluginName );

    return true;
}

QString PluginXmlFile::errorString() const
{
    return data->errorString;
}

QString PluginXmlFile::providerId() const
{
    return data->providerId;
}

QString PluginXmlFile::pluginPath() const
{
    return data->pluginPath;
}
