#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# arcor_germany.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
# with help of Nico Lehmann <kingpin-@gmx.de>
#

__version__ = '1.0.1'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import re
import mechanize


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = 'https://www.arcor.de/ums/ums_neu_sms.jsp'

        # set up our browser
        br = mechanize.Browser()

        # login
        try:
            br.open(URL)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to open %(U)s" % {"U": URL}
            }, "BALANCE": None}
            return MSG

        try:
            br.select_form(name='login')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to select login form"
            }, "BALANCE": None}
            return MSG

        try:
            br.form['user_name'] = LOGIN
            br.form['password'] = PASSWORD
        except:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to insert LOGIN/PW into form"
                }, "BALANCE": None}
            return MSG

        try:
            br.submit()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to submit login credentials"
                }, "BALANCE": None}
            return MSG

        # verify our login worked and return error if not
        try:
            br.find_link(text_regex="Logout")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Login failed, probably wrong login/password"
                }, "BALANCE": None}
            return MSG

        # Skip the login promo.
        # Since we have no permanent cokies I guess we have to do it every time
        try:
            br.select_form(name='loginpromoform')
        except mechanize.FormNotFoundError:
            pass
        try:
            br.submit(name='weiter', label='Weiter')
        except Exception, e:
            pass

        try:
            br.select_form(name='smsform')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
            "MESSAGE": "Error selecting SMS form"
            }, "BALANCE": None}
            return MSG

        ToStr = NUMBERS[0]["number"].encode('utf8')

        try:
            br.form['empfaengerAn'] = ToStr
            br.form['nachricht'] = SMSTEXT.encode(br.encoding())
        except:
            MSG = {"STATUS": {"TYPE": "Error",
            "MESSAGE": "Failed to insert RECIPIENT/MESSAGE into form"
            }, "BALANCE": None}
            return MSG

        try:
            br.submit(name='senden', label='Senden')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to submit the form"
                }, "BALANCE": None}
            return MSG

        # check to see if our send worked
        response = br.response().read()
        if re.search(r'gesendet!', response):
            MSG = {"STATUS": {"TYPE": "Success",
            "MESSAGE": "SMS send"
            }, "BALANCE": None}
            return MSG
        else:
            MSG = {"STATUS": {"TYPE": "Error",
            "MESSAGE": "SMS not send"
            }, "BALANCE": None}
            return MSG

def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "arcor_germany"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
