#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# betamax.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.4'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import re
from urllib import urlencode, quote_plus

# http://code.google.com/p/websmsdroid/issues/detail?id=263
betamax_encode = {
    u'£': u'%01', u'$': u'%02', u'¥': u'%03', u'è': u'%04',
    u'é': u'%05', u'ù': u'%06', u'ì': u'%07', u'ò': u'%08',
    u'Ç': u'%09', u'Ø': u'%0B', u'ø': u'%0C', u'Å': u'%0E',
    u'å': u'%0F', u'_': u'%11', u'Æ': u'%1C', u'æ': u'%1D',
    u'ß': u'%1E', u'É': u'%1F', u' ': u'%20', u'!': u'%21',
    u'"': u'%22', u'#': u'%23', u'%': u'%25', u'&': u'%26',
    u"'": u'%27', u'(': u'%28', u')': u'%29', u'*': u'%2A',
    u'+': u'%2B', u',': u'%2C', u'-': u'%2D', u'.': u'%2E',
    u'/': u'%2F', u'0': u'%30', u'1': u'%31', u'2': u'%32',
    u'3': u'%33', u'4': u'%34', u'5': u'%35', u'6': u'%36',
    u'7': u'%37', u'8': u'%38', u'9': u'%39', u':': u'%3A',
    u';': u'%3B', u'<': u'%3C', u'=': u'%3D', u'>': u'%3E',
    u'?': u'%3F', u'¡': u'%40', u'A': u'%41', u'B': u'%42',
    u'C': u'%43', u'D': u'%44', u'E': u'%45', u'F': u'%46',
    u'G': u'%47', u'H': u'%48', u'I': u'%49', u'J': u'%4A',
    u'K': u'%4B', u'L': u'%4C', u'M': u'%4D', u'N': u'%4E',
    u'O': u'%4F', u'P': u'%50', u'Q': u'%51', u'R': u'%52',
    u'S': u'%53', u'T': u'%54', u'U': u'%55', u'V': u'%56',
    u'W': u'%57', u'X': u'%58', u'Y': u'%59', u'Z': u'%5A',
    u'Ä': u'%5B', u'Ö': u'%5C', u'Ñ': u'%5D', u'Ü': u'%5E',
    u'§': u'%5F', u'¿': u'%60', u'a': u'%61', u'b': u'%62',
    u'c': u'%63', u'd': u'%64', u'e': u'%65', u'f': u'%66',
    u'g': u'%67', u'h': u'%68', u'i': u'%69', u'j': u'%6A',
    u'k': u'%6B', u'l': u'%6C', u'm': u'%6D', u'n': u'%6E',
    u'o': u'%6F', u'p': u'%70', u'q': u'%71', u'r': u'%72',
    u's': u'%73', u't': u'%74', u'u': u'%75', u'v': u'%76',
    u'w': u'%77', u'x': u'%78', u'y': u'%79', u'z': u'%7A',
    u'ä': u'%7B', u'ö': u'%7C', u'ñ': u'%7D', u'ü': u'%7E',
    u'à': u'%7F', u'\n': u'%0A', u'\r': u''
}


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        try:
            P = PLUGINDICT["PROVIDER_ID"]
        except Exception:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "No provider_id supplied!"
                }, "BALANCE": None}
            return MSG
        else:
            print "Betamax provider: %(P)s" % {"P": P}

        try:
            SENDER = PLUGINDICT["SENDER"]
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "No sender supplied!"
                }, "BALANCE": None}
            return MSG

        PD = {"voipcheap": ["https://www.voipcheap.com/", "success"],
                "voipgain": ["https://www.voipgain.com/", "success"],
                "smsdiscount": ["https://www.smsdiscount.com/", "success"],
                "lowratevoip": ["https://www.lowratevoip.com/", "success"],
                "poivy": ["https://www.poivy.com/", "success"],
                "12voip": ["https://www.12voip.com/", "success"],
                "actionvoip": ["https://www.actionvoip.com/", "success"],
                "budgetsip": ["https://www.budgetsip.com/", "success"],
                "calleasy": ["https://www.calleasy.com/", "success"],
                "dialnow": ["https://www.dialnow.com/", "success"],
                "freecall": ["https://www.freecall.com/", "success"],
                "internetcalls": ["https://www.internetcalls.com/", "success"],
                "intervoip": ["https://www.intervoip.com/", "success"],
                "jumblo": ["https://www.jumblo.com/", "success"],
                "justvoip": ["https://www.justvoip.com/", "success"],
                "netappel": ["https://www.netappel.fr/", "success"],
                "nonoh": ["https://www.nonoh.net/", "success"],
                "rynga": ["https://www.rynga.com/", "success"],
                "sipdiscount": ["https://www.sipdiscount.com/", "success"],
                "smartvoip": ["https://www.smartvoip.com/", "success"],
                "smslisto": ["https://www.smslisto.com/", "success"],
                "sparvoip": ["https://www.sparvoip.com/", "success"],
                "telbo": ["https://www.telbo.com/", "success"],
                "voipbuster": ["https://www.voipbuster.com/", "success"],
                "voipbusterpro": ["https://www.voipbusterpro.com/", "success"],
                "voipcheap": ["https://www.voipcheap.co.uk/", "success"],
                "voipian": ["https://www.voipian.com/", "success"],
                "voipraider": ["https://www.voipraider.com/", "success"],
                "voipstunt": ["https://www.voipstunt.com/", "success"],
                "voipwise": ["https://www.voipwise.com/", "success"],
                "voipzoom": ["https://www.voipzoom.com/", "success"],
                "webcalldirect": ["https://www.webcalldirect.com/", "success"],
                }
        URL = PD[P][0] + "myaccount/sendsms.php"

        succes_id = PD[P][1]
        re_suc = re.compile("%s" % (succes_id), flags=re.IGNORECASE)

        ToStr = NUMBERS[0]["number"]

        if not isinstance(SMSTEXT, unicode):
            SMSTEXT = unicode(SMSTEXT, "utf-8")

        TXT = "".join([betamax_encode.get(c, u'%3F%3F') for c in SMSTEXT])

        POST = {"username": LOGIN,
                "password": PASSWORD,
                "from": SENDER,
                "to": ToStr,
                }
        POST = urlencode(POST)
        POST = POST + "&text=" + TXT

        try:
            F = urllib2.urlopen(URL, POST)
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            return MSG
        else:
            response = F.read()
            try:
                ver = re.match(r".*?<version>(.*?)</version>.*",
                                response, re.DOTALL
                                ).group(1)
            except Exception, e:
                ver = "Unknown"

            try:
                res = re.match(r".*?<result>(.*?)</result>.*",
                                response, re.DOTALL
                                ).group(1)
            except Exception, e:
                res = "Unknown"

            try:
                ress = re.match(r".*?<resultstring>(.*?)</resultstring>.*",
                                response, re.DOTALL
                                ).group(1)
            except Exception, e:
                ress = "Unknown"

            try:
                des = re.match(r".*?<description>(.*?)</description>.*",
                                response, re.DOTALL
                                ).group(1)
            except Exception, e:
                des = "Unknown"

            try:
                eca = re.match(r".*?<endcause>(.*?)</endcause>.*",
                                response, re.DOTALL
                                ).group(1)
            except Exception, e:
                eca = "Unknown"

            if res == "0" and ress == "failure" and des == "error":
                TYPE = "Success"
            elif res == "1" and ress == "success":
                TYPE = "Success"
            else:
                TYPE = "Error"

            MSG = {"STATUS": {"TYPE": TYPE,
                "MESSAGE": "Provider: %(P)s\n"\
                "Version: %(ver)s\n"\
                "Result: %(res)s\n"\
                "Resultstring: %(ress)s\n"\
                "Description: %(des)s\n"\
                "Endcause: %(eca)s" % {"P": P,
                        "ver": ver,
                        "res": res,
                        "ress": ress,
                        "des": des,
                        "eca": eca,
                    }
                }, "BALANCE": None}

            return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "betamax"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
