#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# ecall_business.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#
# API documentation:
# http://hilfe.ecall.ch/schnittstellen_http.html
#

__version__ = '1.0.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import re
import cookielib
from urllib import urlencode, quote_plus
from lxml import etree


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "https://www.ecall.ch/"

        try:
            Function = PLUGINDICT["Function"]
        except KeyError:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Function not set!"
                }, "BALANCE": None}
            return MSG

        POST_URL = URL + "ecallurl/ecallurl.ASP?Function=" + Function

        POST = {"WCI": "Interface",
                "AccountName": LOGIN,
                "AccountPassword": PASSWORD,
                "Message": quote_plus(SMSTEXT),
                }

        try:
            POST["JobID"] = PLUGINDICT["JobID"]
        except KeyError:
            pass
        try:
            POST["CallBack"] = PLUGINDICT["CallBack"]
        except KeyError:
            pass
        try:
            POST["Answer"] = PLUGINDICT["Answer"]
        except KeyError:
            pass
        try:
            POST["Notification"] = PLUGINDICT["Notification"]
        except KeyError:
            pass
        try:
            POST["AlwaysNotification"] = PLUGINDICT["AlwaysNotification"]
        except KeyError:
            pass
        try:
            POST["SendDate"] = PLUGINDICT["SendDate"]
        except KeyError:
            pass
        try:
            POST["MsgType"] = PLUGINDICT["MsgType"]
        except KeyError:
            pass
        try:
            POST["NoLog"] = PLUGINDICT["NoLog"]
        except KeyError:
            pass

        ToStr = []
        for item in NUMBERS:
                ToStr.append(item["number"].encode('utf8'))

        POST["Address"] = ";".join(ToStr)

        try:
            F = urllib2.urlopen(POST_URL + "&" + urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            return MSG
        else:
            response = F.read()
            try:
                result = re.match(r"ResultPage\tResultCode:(?P<code>.*?)\tResultText:(?P<text>.*)", response).groupdict()
            except AttributeError, e:
                result = {"code": "X", "text": "Unknown error"}
            if result["code"] == 0:
                MSG = {"STATUS": {"TYPE": "Success",
                    "MESSAGE": "SMS send"
                    }, "BALANCE": None}
                return MSG
            else:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "%(text)s (%(code)s)" % result
                    }, "BALANCE": None}
                return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "ecall_business"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
