#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# exetel_australia.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
from urllib import urlencode


class provider():
    def CheckBalance(self, LOGIN, PASSWORD):
        URL = "https://smsgw.exetel.com.au/sendsms/api_sms_credit.php"
        POST = {"username": LOGIN,
                "password": PASSWORD}

        try:
            F = urllib2.urlopen(URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Balance: " + str(e.code)
                }, "BALANCE": None}
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Balance: " + str(e.reason)
                }, "BALANCE": None}
        else:
            response = F.read()
            status, credits, exetelnote = response.split("|")
            if not credits:
                credits = None
            MSG = {"STATUS": {"TYPE": int(status),
                "MESSAGE": exetelnote,
                }, "BALANCE": credits}
        return MSG

    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "https://smsgw.exetel.com.au/sendsms/api_sms.php"

        try:
            SENDER = PLUGINDICT["SENDER"]
        except KeyError:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Sender Name/Number not set!"
                }, "BALANCE": None}
            return MSG

        ToStr = []
        for item in NUMBERS:
                ToStr.append(item["number"].encode('utf8').lstrip("+"))

        MSG = self.CheckBalance(LOGIN, PASSWORD)
        BALANCE = MSG["BALANCE"]
        STATUS = MSG["STATUS"]["TYPE"]
        MESSAGE = MSG["STATUS"]["MESSAGE"]
        if STATUS == 0:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": MESSAGE
                }, "BALANCE": BALANCE}
            return MSG

        POST = {"username": LOGIN,
                "password": PASSWORD,
                "sender": SENDER.lstrip("+"),
                "message": SMSTEXT.encode("utf8"),
                "mobilenumber": ",".join(ToStr),
                "messagetype": "Text"
                }

        if len(NUMBERS) > 5000:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Maximum of 5000 recipients exceede"
                }, "BALANCE": BALANCE}
            return MSG

        try:
            F = urllib2.urlopen(URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": BALANCE}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": BALANCE}
            return MSG
        else:
            response = F.read()
            resultlist = response[:-4].split("<br>")
            failed = []
            for item in resultlist:
                r = item.split("|")
                status = int(r[0])
                fn = r[1]
                note = r[4]
                if status != 1:
                    failed.append("%(fn)s: %(n)s" % {"fn": fn, "n": note})
            if failed:
                s = ", ".join(failed)
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": s
                    }, "BALANCE": BALANCE}
                return MSG
            else:
                MSG = {"STATUS": {"TYPE": "Success",
                    "MESSAGE": "OK"
                    }, "BALANCE": BALANCE}
                return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "exetel_australia"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
