#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# fetion_cn.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
# with help of Larry <Scenery.July@live.com>
#

__version__ = '1.0.2'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import re
import urllib2
from lxml import etree
from urllib import urlencode


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "http://lab.3haku.net/f/do"
        BALANCE = None

        if not isinstance(SMSTEXT, unicode):
            print "SMSTEXT is not a unicode string"
            SMSTEXT = unicode(SMSTEXT, "utf-8")

        ToStr = []
        for item in NUMBERS:
            num = item["number"].encode('utf8')
            if not re.match(r"^\+86[0-9]*$", num):
                m = "The number %(num)s is not from China!\n"\
                "Fetion does not allow foreign numbers" % {"num": num}
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": m
                    }, "BALANCE": BALANCE}
                return MSG
            elif not re.match(r"^\+[0-9]*$", num):
                m = "The number %(num)s is not following the international "\
                    " format +86xxxx" % {"num": num}
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": m
                    }, "BALANCE": BALANCE}
                return MSG
            else:
                ToStr.append(num.replace("+86", ""))

        To = ",".join(ToStr)

        try:
            SMS = SMSTEXT.encode("gbk")
        except UnicodeEncodeError, e:
            print e
            print "Failed to convert your message to GBK trying UNICODE"
            SMS = SMSTEXT.encode("utf8")
        except UnicodeEncodeError, e:
            print e
            print "Failed to convert your message to UTF-8 giving up"
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to convert your message to UNICODE"
                }, "BALANCE": BALANCE}
            return MSG

        POST = {"phone": LOGIN.strip("+"),
                "pwd": PASSWORD,
                "to": To,
                "msg": SMS,
                }

        # NOTE: Not unicode aware, will fail on special characters!
        try:
            UPOST = urlencode(POST)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "URLencode failed",
                }, "BALANCE": BALANCE}
            print e
            return MSG

        PURL = "%(URL)s?%(UPOST)s" % {"URL": URL, "UPOST": UPOST}

        try:
            r = urllib2.urlopen(PURL)
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "HTTPError: " + str(e.code)
                }, "BALANCE": BALANCE}
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "URLError: " + str(e.reason)
                }, "BALANCE": BALANCE}
        else:
            response = r.read()
            result = etree.XML(response)
            print(etree.tostring(result, pretty_print=True))
            RP = "<action>(?P<action>.*?)</action>.*?"\
                "<statu>(?P<statu>.*?)</statu>.*?"\
                "<phone>(?P<num>.*?)</phone>"

            status = re.findall(r'%(rp)s' % {"rp": RP}, response, re.DOTALL)

            if not status:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Unable to find status info in response:\n"\
                        "%(resp)s" % {"resp": response}
                    }, "BALANCE": BALANCE}
                return MSG

            if status[0][1] != 'success':
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "%(login)s %(statu)s" % {
                        "login": status[0][0],
                        "statu": status[0][1],
                        }
                    }, "BALANCE": BALANCE}
            else:
                del status[0]
                status_string = " ".join([j for i in status for j in i])
                status_dict = {}
                for item in status:
                    num = "+86" + "".join([n for n in item[2] if n.isdigit()])
                    status_dict[num] = item[1]
                if "success" in status_string:
                    TYPE = "Success"
                else:
                    TYPE = "Error"
                MSG = {"STATUS": {"TYPE": TYPE,
                    "MESSAGE": status_dict
                    }, "BALANCE": BALANCE}

        return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "fetion_cn"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
