#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# fishtext.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
# with help of Matthias Deutscher <Matthias.Deutscher@web.de>
#

__version__ = '1.0.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import re
import sys
import cookielib
from urllib import urlencode, quote_plus
from HTMLParser import HTMLParser
from lxml import etree

parser = HTMLParser()


class provider():
    def getBalance(self, URL):
        try:
            F = urllib2.urlopen(URL)
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            BALANCE = None
        else:
            response = F.read()
            cur_val = parser.unescape(response)
            CURRENCY = cur_val[0:1]
            BALANCE = cur_val[1:].strip()
            CURBAL = CURRENCY + BALANCE
        return BALANCE, CURRENCY, CURBAL

    def getMSGID(self, URL):
        try:
            F = urllib2.urlopen(URL)
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            BALANCE = None
        else:
            response = F.read()
            s = '<textarea class="messagelargeinput" name="(.*?)" id="message"'
            mid = re.search(r'%(s)s' % {"s": s}, response)
            try:
                MSGID = mid.group(1)
            except AttributeError:
                MSGID = False
        return MSGID

    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "https://www.fishtext.com/cgi-bin/mobi/"
        LOGIN_URL = URL + "account"
        SEND_URL = "https://www.fishtext.com/SendSMS/SendSMS"
        SEND_URLC = URL + "sendMessage.cgi"
        SENT_URL = URL + "sentMessages.cgi"
        BALANCE_URL = URL + "getBalance.cgi"
        INBOX_URL = URL + "inbox.cgi"

        # Install a cookie handler and add custom headers
        # Any call to "urlopen" will from now on handle Cookies
        # with cookielib.CookieJar()
        cj = cookielib.CookieJar()
        opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
        header = [('User-agent',
                    'Opera/9.80',
                    )]
        opener.addheaders = header
        urllib2.install_opener(opener)

        POST = {"action": "login",
                "mobile": LOGIN,
                "password": PASSWORD,
                "_sp_errorJS": 0,
                "sp_tooltip_init": 1,
                }

        try:
            F = urllib2.urlopen(LOGIN_URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            return MSG
        else:
            response = F.read()
            if not re.match(r".*?\>Logout\<.*", response, re.DOTALL):
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Login Failed"
                    }, "BALANCE": None}
                return MSG
            BALANCE, CURRENCY, CURBAL = self.getBalance(BALANCE_URL)

        MSGID = self.getMSGID(SEND_URLC)
        if not MSGID:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Login Failed"
                }, "BALANCE": CURBAL}
            return MSG

        ToStr = []
        for item in NUMBERS:
                ToStr.append(item["number"].encode('utf8').strip("+"))

        try:
            SMS = SMSTEXT.encode("ISO-8859-15")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e)
                }, "BALANCE": CURBAL}
            return MSG

        POST = {"action": "Send",
                "SA": 0,
                "DR": 1,
                "ST": 1,
                "%(MSGID)s" % {"MSGID": MSGID}: SMS,
                }
        POST["RN"] = ",".join(ToStr)

        try:
            F = urllib2.urlopen(SEND_URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": CURBAL}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": CURBAL}
            return MSG
        else:
            response = F.read()
            server_msg = re.findall("<p>(.*?)<", response, re.DOTALL)
            SRV_MSG = parser.unescape(" ".join(server_msg))
            if re.search(r"successfully", response):
                BALANCE, CURRENCY, CURBAL = self.getBalance(BALANCE_URL)
                MSG = {"STATUS": {"TYPE": "Success",
                    "MESSAGE": SRV_MSG
                    }, "BALANCE": CURBAL}
                return MSG
            else:
                BALANCE, CURRENCY, CURBAL = self.getBalance(BALANCE_URL)
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": SRV_MSG
                    }, "BALANCE": CURBAL}
                return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "fishtext"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
