#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# mutlusms.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#
# Based on API v0.2
# API Docs:
# http://www.mutlusms.com/yazilimgelistiriciler.html
# http://www.mutlusms.com/js/MutluSmsHttpApi.pdf
#

__version__ = '1.0.1'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import re
from urllib import urlencode, quote_plus


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = 'http://www.mutlusms.com/api/gonder.do'

        try:
            SENDER = PLUGINDICT["SENDER"]
        except Exception:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Sender is missing"
                }, "BALANCE": None}
            return MSG

        try:
            FLASH = PLUGINDICT["FLASH"]
        except Exception:
            pass

        ToStr = NUMBERS[0]["number"]

        if not isinstance(SMSTEXT, unicode):
            SMSTEXT = unicode(SMSTEXT, "utf-8")

        POST = {"id": LOGIN,
                "sifre": PASSWORD,
                "gonderen": SENDER,
                "mesaj": quote_plus(SMSTEXT),
                "telefon": ToStr,
                "flash": FLASH,
        }

        ERROR_MSG = {
            "11": "Message is too long",
            "12": "Message has unusable characters: ^ { } [ ] ~ \ |",
            "13": "Wrong receiver phone number",
            "14": "Wrong sender name",
            "15": "Unsupported sender number",
            "16": "Unsupported sender name",
            "17": "Flash sms cant have special characters",
            "18": "Wrong parameter",
            "19": "You cant send more than 100 person once.",
            "101": "Wrong authority info",
            "102": "Wrong username or password",
            "103": "Not enough credits",
            "301": "This registry number couldnt found!",
            "999": "Error about mutlusms (Contact to MutluSMS)",
            }

        try:
            F = urllib2.urlopen(URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": None}
            return MSG
        else:
            response = F.read()
            re_suc = re.compile('OK')
            if re_suc.search(response):
                MSG = {"STATUS": {"TYPE": "Success",
                "MESSAGE": "Sent. Message ID: %s" %(response.strip("OK:"))
                }, "BALANCE": None}
                return MSG
            else:
                STATUS = ERROR_MSG[response.strip("HATA:")]
                MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed: %s" %(STATUS)
                }, "BALANCE": None}
                return MSG

def plugin_instance():
    return provider()

#
# Uncomment the following and run the plugin directly for testing purpose:
#
#tLOGIN = "Login"
#tPASSWORD = "Password"
#tSMSTEXT = "This is a test"
#tNUMBERS = [{"number": "+1111111", "name": "AAAAA"},
#           {"number": "+22222", "name": "BBBBB"}
#        ]
#tPLUGINDICT = {"SENDER": 12312121,
#              "FLASH": 0
#              }
#
#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
