#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# o2_germany.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.2'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import sys
import re
import mechanize
import time
from urllib import quote_plus, unquote_plus

class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        BASEURL = 'https://www.o2online.de/asp/login/comcenter-login'
        QUERY1 = '?scheme=http&port=80&server=email.o2online.de'
        QUERY2 = '&url=%2Fssomanager.osp%3FAPIID%3DAUTH-WEBSSO'
        LOGINURL = BASEURL + QUERY1 + QUERY2

        try:
            SENDER = PLUGINDICT["SENDER"]
        except Exception, e:
            SENDER = False
        try:
            FLASH = PLUGINDICT["FLASH"]
        except Exception:
            FLASH = False
        try:
            ANONYM = PLUGINDICT["ANONYM"]
        except Exception:
            ANONYM = False

        # RegExp to check if SMS was send successfully:
        re_suc = re.compile('erfolgreich', re.IGNORECASE)
        # RegEx to check if en error occured
        re_fail = re.compile(
            '(Fehler beim Login)|(Error logging in)',
            re.IGNORECASE
            )
        # RegExp to find remaining FREE SMS:
        re_free = re.compile('Frei-SMS: [0-9]+')

        # Instanziate a Browser
        try:
            br = mechanize.Browser()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
            "MESSAGE": "Failed to initiate the Browser"
            }, "BALANCE": None}
            return MSG

        #### REQUIRED!!!!
        br.set_handle_robots(False)

        # User-Agent
        UA = 'Opera/9.80 (X11; Linux x86_64; U; en) Version/11.51 Gentoo'
        br.addheaders = [('User-agent', UA)]

        #------------------------------------
        # Open the O2 website
        #------------------------------------
        try:
            br.open(LOGINURL)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to open the O2 website"
                }, "BALANCE": None}
            return MSG

        #------------------------------------
        # Select the Login form and feed it with the login credentials
        #------------------------------------
        try:
            br.select_form(nr=0)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Faild to select the login Form"
                }, "BALANCE": None}
            return MSG

        br.form['loginName:loginName'] = LOGIN
        br.form['password:password'] = PASSWORD

        try:
            br.submit()
            response = br.response().read()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": 'Failed to load "Mein O2"'
                }, "BALANCE": None}
            return MSG

        if re_fail.search(str(response)):
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Probably wrong login parameters"
                }, "BALANCE": None}
            return MSG

        #------------------------------------
        # Check for a redirect
        #------------------------------------
        if re.search(r"weitergeleitet", response):
            try:
                br.select_form(nr=0)
                #br.submit(nr=0)
                br.submit(name='_eventId_login', label='Weiter')
            except Exception, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Automatic redirection failed"
                    }, "BALANCE": None}
                return MSG

        #------------------------------------
        # Open the SMS-Center
        #------------------------------------
        BASEURL = "https://email.o2online.de/smscenter_new.osp"
        QUERY = "?Autocompletion=1&MsgContentID=-1"
        SMSCENTER = BASEURL + QUERY
        try:
            br.open(SMSCENTER)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to open the SMSCenter"
                }, "BALANCE": None}
            return MSG
        response = br.response().read()

        #------------------------------------
        # Select the SMS-Form
        #------------------------------------
        try:
            br.select_form(name='frmSMS')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to select the SMS form"
                }, "BALANCE": None}
            return MSG

        # Find how much FREE SMS we have left.
        FREESMS = int(re_free.findall(response)[0].split()[1])

        #------------------------------------
        # Fill the forms
        #------------------------------------
        br.form.set_all_readonly(False)
        ToStr = []
        for item in NUMBERS:
            try:
                NA = item["name"].encode("iso-8859-15")
            except KeyError:
                ToStr.append(item["number"].encode("iso-8859-15"))
            else:
                ToStr.append("%(NAME)s <%(NUMBER)s>" %
                    {"NAME": item["name"].encode("iso-8859-15"),
                    "NUMBER": item["number"].encode("iso-8859-15"),
                    }
                    )

        br.form['SMSTo'] = ", ".join(ToStr)

        # Encode the text to something mechanize can cope with
        try:
            SMS = SMSTEXT.encode("iso-8859-15")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e)
                }, "BALANCE": FREESMS}
            return MSG

        br.form['SMSText'] = SMS
        # FlashSMS?
        if FLASH:
            br.form['FlagFlash'] = '1'
        # Anonymous SMS?
        if ANONYM:
            br.form['FlagDefSender'] = '1'
            br.form['FlagAnonymous'] = '1'
        # Replaced sender number?
        if SENDER:
            br.form['FlagDefSender'] = '1'
            br.form['SMSFrom'] = str(SENDER)

        #------------------------------------
        # Finally send the SMS
        #------------------------------------
        try:
            br.submit()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Submission faild"
                }, "BALANCE": FREESMS}
        else:
            response = br.response().read()
            if re_suc.search(str(response)):

                # Reopen the SMS-Center to get the remaining free SMS
                try:
                    br.open(SMSCENTER)
                except Exception, e:
                    SMS_LEFT = None
                else:
                    response = br.response().read()
                    SMS_LEFT = int(re_free.findall(response)[0].split()[1])

                MSG = {"STATUS": {"TYPE": "Success",
                    "MESSAGE": "SMS erfolgreich versendet."
                    }, "BALANCE": SMS_LEFT}
            else:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "SMS wurde nicht versendet."
                    }, "BALANCE": FREESMS}
        return MSG

def plugin_instance():
    return provider()

#
# Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "o2_germany"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
