#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# saunalahti_finland.py, Copyright Jarkko Lehtoranta
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Jarkko Lehtoranta <devel@jlranta.com>
# based on code written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.2'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import re
import mechanize


class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = 'https://oma.saunalahti.fi/settings/smsSend'
        # set up our browser
        br = mechanize.Browser()
        br.addheaders = [('User-Agent',
                'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.3)'
                )]
        #cj = cookielib.LWPCookieJar()
        #br.set_cookiejar(cj)
        br.set_handle_robots(False)

        # login to provider page
        try:
            br.open(URL)
        except  Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to open %s" % (URL)
                }, "BALANCE": None}
            return MSG

        try:
            br.select_form(name="login_form")
        except  Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to select login form"
                }, "BALANCE": None}
            return MSG

        try:
            br.form['username'] = LOGIN.encode('ISO-8859-1')
            br.form['password'] = PASSWORD.encode('ISO-8859-1')
        except  Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to insert login form values"
                }, "BALANCE": None}
            return MSG

        try:
            br.submit()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed submit login data"
                }, "BALANCE": None}
            return MSG
        # verify login
        response = unicode(br.response().read(), 'ISO-8859-1')
        if (re.search(u"Kirjautuminen epäonnistui:", response)
                or not re.search(u"Lähetettyjä viestejä:", response)):
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Error logging in"
                }, "BALANCE": None}
            return MSG

        # Find number of SMS messages left

        index = response.find(u"Kuukausittaisia viestejä jäljellä:")
        line = response[index:]
        sms_per_month = line[line.find(": ") + 2:line.find("<")]

        index = response.find(u"Kertakäyttöisiä viestejä jäljellä:")
        line = response[index:]
        sms_one_time = line[line.find(": ") + 2:line.find("<")]

        try:
            sms_balance = int(sms_per_month) + int(sms_one_time)
        except ValueError:
            sms_balance = None

        if sms_balance == 0:
            MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Sorry, no more messages left"
                    }, "BALANCE": sms_balance}
            return MSG

        # send SMS
        numberarray = []
        for item in NUMBERS:
            try:
                if not item["number"] == "":
                    numberarray.append(item["number"].replace('+', '00'))
            except KeyError:
                pass
        try:
            br.select_form(name="myform")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to select myform"
                }, "BALANCE": sms_balance}
            return MSG
        try:
            br.form['recipients'] = ",".join(numberarray)
            br.form['text'] = SMSTEXT.encode('ISO-8859-15')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to fill recipients or text form"
                }, "BALANCE": sms_balance}
            return MSG

        try:
            br.submit()
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Error submitting SMS form"
                }, "BALANCE": sms_balance}
            return MSG

        # verify that sending worked
        response = unicode(br.response().read(), 'ISO-8859-1')
        sent = re.findall(u"Tässä kuussa lähettämäsi viestit:.*</table>",
            response,
            re.S)

        try:
            sentlist = re.findall(r"td class=.*", sent[0])
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Error compiling sent list"
                }, "BALANCE": sms_balance}
            return MSG

        numberarray.reverse()
        failarray = []
        okarray = []
        n = 0
        for number in numberarray:
            if not re.search(r'OK', sentlist[-2 - 3 * n]):
                failarray.append(number)
            else:
                okarray.append(number)
            n = n + 1
        failednumbers = ", ".join(failarray)
        oknumbers = ", ".join(okarray)

        # Find number of SMS messages left after sending
        index = response.find(u"Kuukausittaisia viestejä jäljellä:")
        line = response[index:]
        sms_per_month = line[line.find(": ") + 2:line.find("<")]

        index = response.find(u"Kertakäyttöisiä viestejä jäljellä:")
        line = response[index:]
        sms_one_time = line[line.find(": ") + 2:line.find("<")]

        try:
            sms_balance_after = int(sms_per_month) + int(sms_one_time)
        except ValueError:
            sms_balance_after = None

        if (not sms_balance_after or not sms_balance or
            sms_balance_after < sms_balance and failednumbers == ""):
            MSG = {"STATUS": {"TYPE": "Success",
                "MESSAGE": "Delivered"
                }, "BALANCE": sms_balance_after}
            return MSG

        if failednumbers != "" and oknumbers == "":
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed for: %(f)s." %
                    {"f": str(failednumbers)}
            }, "BALANCE": sms_balance_after}
            return MSG

        if failednumbers != "" and oknumbers != "":
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed for: %(f)s. Delivered to: %(s)s." %
                    {"f": str(failednumbers), "s": str(oknumbers)}
                }, "BALANCE": sms_balance_after}
            return MSG

        MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Error sending SMS."
                }, "BALANCE": sms_balance_after}
        return MSG


def plugin_instance():
    return provider()

#
# Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "saunalahti_finland"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
