#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# sipgate.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.2'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import xmlrpclib

STATUS_CODES = {
    200: 'Method success',
    400: 'Method not supported',
    401: 'Request denied (no reason specified)',
    402: 'Internal error',
    403: 'Invalid arguments',
    404: 'Resources exceeded (this MUST not be used to indicate parameters in error)',
    405: 'Invalid parameter name',
    406: 'Invalid parameter type',
    407: 'Invalid parameter value',
    408: 'Attempt to set a non-writable parameter',
    409: 'Notification request rejected.',
    410: 'Parameter exceeds maximum size.',
    411: 'Missing parameter.',
    412: 'Too many requests.',
    500: 'Date out of range.',
    501: 'Uri does not belong to user.',
    502: 'Unknown type of service.',
    503: 'Selected payment method failed.',
    504: 'Selected currency not supported.',
    505: 'Amount exceeds limit.',
    506: 'Malformed SIP URI.',
    507: 'URI not in list.',
    508: 'Format is not valid E.164.',
    509: 'Unknown status.',
    510: 'Unknown ID.',
    511: 'Invalid timevalue.',
    512: 'Referenced session not found.',
    513: 'Only single default per TOS allowed.',
    514: 'Malformed VCARD format.',
    515: 'Malformed PID format.',
    516: 'Presence information not available.',
    517: 'Invalid label name.',
    518: 'Label not assigned.',
    519: 'Label doesn’t exist.',
    520: 'Parameter includes invalid characters.',
    521: 'Bad password. (Rejected due to security concerns.)',
    522: 'Malformed timezone format.',
    523: 'Delay exceeds limit.',
    524: 'Requested VPN type not available.',
    525: 'Requested TOS not available.',
    526: 'Unified messaging not available.',
    527: 'URI not available for registration.',
}

class provider():
    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = 'https://%(l)s:%(p)s@samurai.sipgate.net/RPC2'
        CLIENT = 'web2sms'
        VENDOR = 'web2sms'

        nr = NUMBERS[0]["number"].encode('utf8')
        # Remove everything except numbers
        ToStr = "".join(i for i in nr if i.isdigit())

        # Create an SIPGATE server object
        try:
            sipgServer = xmlrpclib.ServerProxy(URL % {
                    'l': LOGIN,
                    'p': PASSWORD
                        }
                    )
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "Failed to create the server object"
                }, "BALANCE": None}
            return MSG

        # Identify ourselfs to SIPGATE
        try:
            sipgCID = sipgServer.samurai.ClientIdentify({
                    'ClientName': CLIENT,
                    'ClientVersion': __version__,
                    'ClientVendor': VENDOR
                        }
                    )
        except xmlrpclib.ProtocolError, e:
            errcode = e.errcode
            errmsg = e.errmsg
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "%(c)s - %(m)s" % {"c": errcode, "m": errmsg}
                }, "BALANCE": None}
            return MSG

        # Send SMS
        try:
            sipgINIT = sipgServer.samurai.SessionInitiate({
                    'RemoteUri': "sip:%(to)s@sipgate.de" % {"to": ToStr},
                    'TOS': 'text',
                    'Content': SMSTEXT
                        }
                    )
        except xmlrpclib.ProtocolError, e:
            errcode = e.errcode
            errmsg = e.errmsg
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "%(c)s - %(m)s" % {"c": errcode, "m": errmsg}
                }, "BALANCE": None}
            return MSG
        else:
            # Statuscode 200 == Success!
            MSG = {"STATUS": {"TYPE": "Success",
                "MESSAGE": "SMS send"
                }, "BALANCE": None}
            return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "sipgate"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
