#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# sms77.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#
# API documentation:
# http://www.sms77.de/products.html?art=interfaces
#

__version__ = '1.0.2'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import requests
import urllib2
from urllib import urlencode

error_dict = {
    100: "SMS wurde erfolgreich verschickt",
    101: "Versand an mindestens einen Empfänger fehlgeschlagen",
    201: "Ländercode für diesen SMS-Typ nicht gültig. Bitte als Basic SMS" \
        "verschicken.",
    202: "Empfängernummer ungültig",
    300: "Bitte Benutzer/Passwort angeben",
    301: "Variable to nicht gesetzt",
    304: "Variable type nicht gesetzt",
    305: "Variable text nicht gesetzt",
    306: "Absendernummer ungültig. Diese muss vom Format 0049..." \
        " sein und eine gültige Handynummer darstellen.",
    307: "Variable url nicht gesetzt",
    400: "type ungültig. Siehe erlaubte Werte oben.",
    401: "Variable text ist zu lang",
    402: "Reloadsperre – diese SMS wurde bereits innerhalb der letzten" \
        "90 Sekunden verschickt",
    500: "Zu wenig Guthaben vorhanden.",
    600: "Carrier Zustellung misslungen",
    700: "Unbekannter Fehler",
    801: "Logodatei nicht angegeben",
    802: "Logodatei existiert nicht",
    803: "Klingelton nicht angegeben",
    900: "Benutzer/Passwort-Kombination falsch",
    902: "http API für diesen Account deaktiviert",
    903: "Server IP ist falsch",
}


class provider():
    def getBalance(self, LOGIN, PASSWORD):
        BalanceURL = "http://gateway.sms77.de/balance.php"
        POST = {"u": LOGIN, "p": PASSWORD}
        r = requests.post(BalanceURL, data=POST)
        c = r.content
        if not r.status_code == requests.codes.ok or c == "900":
            BALANCE = None
        else:
            BALANCE = float(c)
        return BALANCE

    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "https://gateway.sms77.de/"

        if not isinstance(SMSTEXT, unicode):
            SMSTEXT = unicode(SMSTEXT, "utf-8")

        BALANCE = self.getBalance(LOGIN, PASSWORD)

        try:
            To = NUMBERS[0]["number"].replace("+", "00")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": e
                }, "BALANCE": BALANCE}

        POST = {"u": LOGIN,
                "p": PASSWORD,
                "to": To,
                "text": SMSTEXT.encode("utf-8"),
                }

        try:
            smstype = PLUGINDICT["type"]
        except Exception, e:
            smstype = "basicplus"

        POST["type"] = smstype

        try:
            SENDER = PLUGINDICT["from"].lstrip("+")
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "No sender specified"
                }, "BALANCE": BALANCE}

        if smstype != "basicplus":
            POST["from"] = SENDER

        try:
            POST["debug"] = PLUGINDICT["debug"]
        except Exception:
            pass

        try:
            POST["delay"] = PLUGINDICT["delay"]
        except Exception:
            pass

        try:
            r = requests.post(URL, data=POST)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": e
                }, "BALANCE": BALANCE}
        else:
            if not r.ok:
                try:
                    reason = r.headers['status']
                except Exception, e:
                    reason = "Error occured while talking to the server"
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": reason
                    }, "BALANCE": BALANCE}
            else:
                response = int(r.content)
                if response == 100:
                    BALANCE = self.getBalance(LOGIN, PASSWORD)
                    MSG = {"STATUS": {"TYPE": "Success",
                        "MESSAGE": error_dict[response]
                        }, "BALANCE": BALANCE}
                else:
                    BALANCE = self.getBalance(LOGIN, PASSWORD)
                    MSG = {"STATUS": {"TYPE": "Error",
                        "MESSAGE": error_dict[response]
                        }, "BALANCE": BALANCE}

        return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "sms77"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
