#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# valuesms.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
#

__version__ = '1.0.1'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import re
from urllib import urlencode, quote_plus
from lxml import etree


class provider():
    def get_balance(self, URL, LOGIN, PASSWORD):
        URL = "https://www.valuesms.com/cred.php"
        POST = {"u": LOGIN, "p": PASSWORD}

        try:
            F = urllib2.urlopen(URL, urlencode(POST))
        except urllib2.HTTPError, e:
            return None
        except urllib2.URLError, e:
            return None
        else:
            BALANCE = F.read()
            if BALANCE.isdigit():
                return BALANCE
            else:
                return None

    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = "https://www.valuesms.com"
        SEND_URL = URL + "/msg.php"
        STATUS_URL = URL + "/msgtrk.php"

        STATUS_DICT = {"0": "Delivery pending",
                        "1": "Successfully delivered",
                        "2": "Delivery failed"
                        }

        if not isinstance(SMSTEXT, unicode):
            SMSTEXT = unicode(SMSTEXT, "utf-8")

        BALANCE = self.get_balance(self, LOGIN, PASSWORD)

        try:
            SMSTEXT = SMSTEXT.encode('utf-8')
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": e,
                }, "BALANCE": BALANCE}
            return MSG

        try:
            SMSTEXT = quote_plus(SMSTEXT)
        except Exception, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": e,
                }, "BALANCE": BALANCE}
            return MSG

        POST = {"u": LOGIN,
                "p": PASSWORD,
                "m": SMSTEXT,
                }

        try:
            POST["o"] = PLUGINDICT["MSG_ORIGIN"]
        except Exception:
            pass
        try:
            POST["r"] = PLUGINDICT["REL_TIME"]
        except Exception:
            pass
        try:
            # Minimum scheduling time is 1 hour
            POST["s"] = PLUGINDICT["ABS_TIME"]
        except Exception:
            pass
        try:
            POST["rr"] = PLUGINDICT["RECEIVE_REPLIES"]
        except Exception:
            pass
        try:
            POST["rs"] = PLUGINDICT["ROUTE"]
        except Exception:
            POST["rs"] = 0

        ToStr = []
        for item in NUMBERS:
                ToStr.append(item["number"].encode('utf8'))

        POST["d"] = ",".join(ToStr)

        try:
            F = urllib2.urlopen(SEND_URL, urlencode(POST))
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.code)
                }, "BALANCE": BALANCE}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": str(e.reason)
                }, "BALANCE": BALANCE}
            return MSG
        else:
            response = F.read()
            stat, reason = response.split()
            if reason.isdigit():
                jobID = reason
            else:
                " ".join(reason)
            if stat == "ACK":
                BALANCE = self.get_balance(self, LOGIN, PASSWORD)
                MSG = {"STATUS": {"TYPE": "Success",
                    "MESSAGE": "SMS send"
                    }, "BALANCE": BALANCE}
                return MSG
            elif stat == "NAK":
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": reason
                    }, "BALANCE": BALANCE}
                return MSG
            else:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": response
                    }, "BALANCE": BALANCE}
                return MSG


def plugin_instance():
    return provider()

#
#Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "valuesms"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
#tSMSTEXT = config.get(ID, 'SMSTEXT')
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
