#!/bin/env python
# -*- coding: utf-8 -*-
#
# -----------------------------------------------------------------------------
# vodafone_ireland.py, Copyright Bjoern Olausson
# -----------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view the license visit
# http://www.gnu.org/licenses/gpl-3.0.html
# or write to
# Free Software Foundation, Inc.
# 51 Franklin St, Fifth Floor
# Boston, MA
# 02110-1301 USA
# -----------------------------------------------------------------------------
# -----------------------------------------------------------------------------
#
# Provider written by Bjoern Olausson <maemo@olausson.de>
# with help of Colm Smyth <colm.smyth@gmail.com>
#

__version__ = '1.0.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import urllib2
import cookielib
import re
import time
from urllib import urlencode, quote_plus


class provider():
    def getBalanceToken(self, WURL):
        try:
            F = urllib2.urlopen(WURL)
        except urllib2.HTTPError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "WURL: " + str(e.code)
                }, "BALANCE": None}
            return MSG
        except urllib2.URLError, e:
            MSG = {"STATUS": {"TYPE": "Error",
                "MESSAGE": "WURL: " + str(e.reason)
                }, "BALANCE": None}
            return MSG
        else:
            response = F.read()
            string = 'org.apache.struts.taglib.html.TOKEN.+?value="(.+?)">'
            regex = re.search(r'%(R)s' % {"R": string}, response)
            try:
                TOKEN = regex.group(1)
            except Exception, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Failed to find TOKEN"
                    }, "BALANCE": None}
                return MSG
            regex = "<strong>([0-9]*?)</strong> web texts left this month"
            B = re.search(r"%(R)s" % {"R": regex}, response)
            try:
                BALANCE = B.group(1)
            except Exception, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Failed to get SMS balance"
                    }, "BALANCE": None}
                BALANCE = None
                return MSG

        return BALANCE, TOKEN

    def sendSMS(self, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        URL = 'https://www.vodafone.ie/'
        LURL = URL + "myv/services/login/Login.shtml"
        WURL = URL + "myv/messaging/webtext/index.jsp"
        PURL = URL + "myv/messaging/webtext/Process.shtml"

        DATE = str(PLUGINDICT["DATE"])
        if not DATE.isdigit():
            DATE = "false"
        TIME = str(PLUGINDICT["TIME"])
        if not TIME.isdigit():
            TIME = "false"

        ToStr = []
        for item in NUMBERS:
            try:
                NA = item["name"].encode('utf8')
            except KeyError:
                ToStr.append(item["number"].encode('utf8'))
            else:
                ToStr.append("%(NUMBER)s" % {
                    "NUMBER": item["number"].encode('utf8')
                    }
                )

            # Fill number of elements to a total of 5 and
            # bail out if we find more then 5 numbers
            nNumbers = len(ToStr)
            if nNumbers < 5:
                fill = 5 - nNumbers
                ToStr.extend([""] * fill)
            elif nNumbers > 5:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "Max recipients exceeded"
                    }, "BALANCE": None}
                return MSG

            # Install a cookie handler and add custom headers
            # Any call to "urlopen" will from now on handle Cookies
            # with cookielib.CookieJar()
            cj = cookielib.CookieJar()
            opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
            header = [('User-agent',
                        'Opera/9.80',
                        )]
            opener.addheaders = header
            urllib2.install_opener(opener)

            POST = {"username": LOGIN,
                    "password": PASSWORD
                    }

            try:
                F = urllib2.urlopen(LURL, urlencode(POST))
            except urllib2.HTTPError, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "LURL: " + str(e.code)
                    }, "BALANCE": None}
                return MSG
            except urllib2.URLError, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "LURL: " + str(e.reason)
                    }, "BALANCE": None}
                return MSG
            else:
                response = F.read()
                if not re.search(r"Sign out", response):
                    MSG = {"STATUS": {"TYPE": "Error",
                        "MESSAGE": "Login Failed"
                        }, "BALANCE": None}
                    return MSG

            BALANCE, TOKEN = self.getBalanceToken(WURL)

            # Create the recipient POST data
            To = {}
            index = 0
            for nr in ToStr:
                KEY = "recipients[%(I)s]" % {"I": index}
                To[KEY] = nr
                index += 1

            num =  470 - len(SMSTEXT)

            POST = {"message": SMSTEXT,
                    "futuredate": DATE,
                    "futuretime": TIME,
                    "org.apache.struts.taglib.html.TOKEN": TOKEN,
                    }

            POST.update(To)

            # We need to slow down a bit, otherwise the Vodafone Server throws
            # error "ME-WT-04"
            time.sleep(2)
            try:
                F = urllib2.urlopen(PURL, urlencode(POST))
            except urllib2.HTTPError, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "PURL: " + str(e.code)
                    }, "BALANCE": BALANCE}
                return MSG
            except urllib2.URLError, e:
                MSG = {"STATUS": {"TYPE": "Error",
                    "MESSAGE": "PURL: " + str(e.reason)
                    }, "BALANCE": BALANCE}
                return MSG
            else:
                response = F.read()
                if re.search(r"Message sent", response):
                    BALANCE, TOKEN = self.getBalanceToken(WURL)
                    MSG = {"STATUS": {"TYPE": "Success",
                        "MESSAGE": "Message sent"
                        }, "BALANCE": BALANCE}
                    return MSG
                elif re.search(r"error-code-tracking", response):
                    strg = '<span class="error-code-tracking">(.*?)</span>'
                    ec = re.search(r'%(R)s' % {"R": strg}, response).group(1)
                    oldBALANCE = BALANCE
                    BALANCE, TOKEN = self.getBalanceToken(WURL)

                    if oldBALANCE == BALANCE:
                        MSG = {"STATUS": {"TYPE": "Error",
                            "MESSAGE": "Server response: %(ERR)s" % {"ERR": ec}
                            }, "BALANCE": BALANCE}
                        return MSG
                    elif oldBALANCE > BALANCE:
                        BALANCE, TOKEN = self.getBalanceToken(WURL)
                        MSG = {"STATUS": {"TYPE": "Success",
                            "MESSAGE": "Msg sent but w error %(E)s" % {"E": ec}
                            }, "BALANCE": BALANCE}
                        return MSG
                    else:
                        MSG = {"STATUS": {"TYPE": "Error",
                            "MESSAGE": "Server response: %(ERR)s" % {"ERR": ec}
                            }, "BALANCE": BALANCE}
                        return MSG
                else:
                    BALANCE, TOKEN = self.getBalanceToken(WURL)
                    MSG = {"STATUS": {"TYPE": "Error",
                        "MESSAGE": "Unknown Error"
                        }, "BALANCE": BALANCE}
                    return MSG

def plugin_instance():
    return provider()

#
# Uncomment the following and run the plugin directly for testing purpose:
#
#import ConfigParser
#import ast

#config = ConfigParser.ConfigParser()
#config.read('../../plugin_credentials.cfg')

## Which ID to load from the config
#ID = "vodafone_ireland"

#tLOGIN = config.get(ID, 'LOGIN')
#tPASSWORD = config.get(ID, 'PASSWORD')
## max 459 chars. 160/per SMS
#tSMSTEXT = config.get(ID, 'SMSTEXT')
## Max 5 recipients
#tNUMBERS = ast.literal_eval(config.get(ID, 'NUMBERS', 0))
## One week ahead scheduling with a hourly intervall:
## DATE: "0": "Today, 09/11/2011" to  "7": "Wednesday, 16/11/2011"
## TIME: "1": "01:00" to "24": "24:00"
#tPLUGINDICT = ast.literal_eval(config.get(ID, 'PLUGINDICT'))

#p = provider()
#r = p.sendSMS(tLOGIN, tPASSWORD, tNUMBERS, tSMSTEXT, tPLUGINDICT)
#print r
