import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import Web2SMS 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants

Page {
    id: editAccountPage

    property string setupType
    property Account account

    signal finished

    orientationLock: PageOrientation.LockPortrait

    Component.onCompleted: {
        // Setup the loader
        __setupLoader();
    }

    function __setupLoader() {
        var providerInfo = backend.plugins.providerById(account.providerId);
        console.debug("Edit providerId:", account.providerId);
        console.debug("Setup type:", setupType);

        var accountSetup = providerInfo.accountsetup.length ? providerInfo.accountsetup : "GenericAccountSettings.qml";
        console.debug("AccountSetup file:", accountSetup);

        var component = Qt.createComponent(accountSetup);

        if ( component.status != Component.Ready ) {
            console.log("Error loading ", accountSetup, ": ", component.errorString());
            return;
        }

        // Update the header info
        if ( setupType == "CreateNew" )
            //% "Add %1"
            pageTitle.title = qsTrId("qtn_web2sms_editaccount_title_new").arg(providerInfo.name);
        else
            //% "Edit %1"
            pageTitle.title = qsTrId("qtn_web2sms_editaccount_title_edit").arg(providerInfo.name);
        pageTitle.imageSource = providerInfo.icon;

        // Do we have to show the beta warning?
        // In AccountSetup the warning can't be disabled
        if ( providerInfo.state == ProviderInfo.Beta && (typeof(appSettings) == "undefined" || appSettings.showBetaWarning) )
            betaWarningTimer.start();

        loader.sourceComponent = component;
    }

    function __saveSettings() {
        var settings = loader.item;

        // Check if the account setup is ready
        if ( settings.isFinished() ) {
            account.sync();
        }
    }

    function __goBack() {
        if (pageStack.depth > 1)
            pageStack.pop()
        else
            Qt.quit()
    }

    Timer {
        id: betaWarningTimer
        interval: 50
        repeat: false
        onTriggered: betaWarning.open()
    }

    BetaWarning {
        id: betaWarning
    }

    Banner {
        id: banner
    }

    Connections {
        target: account
        onSynced: {
            //% "Settings saved"
            banner.showBanner(qsTrId("qtn_web2sms_editaccount_saved"));

            // Fire the signal that we're finished
            editAccountPage.finished();
        }
    }

    anchors.margins: UIConstants.DEFAULT_MARGIN

    Flickable {
        id: flickable

        anchors.fill: parent
        flickableDirection: Flickable.VerticalFlick
        contentWidth: column.width
        contentHeight: column.height

        Column {
            id: column

            width: editAccountPage.width

            spacing: 20

            Title {
                id: pageTitle

                //% "Configure your provider settings"
                subtitle: qsTrId("qtn_web2sms_editaccount_subtitle")
                width: parent.width
                separator: false
            }

            Component.onCompleted: {
                var provider = backend.plugins.providerById(account.providerId);

                // Show the balance
                if ( provider.supportsBalance ) {
                    balanceLabel.balanceFormat = provider.balanceFormat;
                    balanceLabel.balance = account.balance;
                    balanceColumn.visible = true;
                }
            }

            Column {
                id: balanceColumn

                visible: false
                width: parent.width
                spacing: parent.spacing

                SectionDelegate {
                    width: parent.width
                    //% "Balance"
                    sectionName: qsTrId("qtn_web2sms_editaccountpage_balance")
                }

                BalanceLabel {
                    id: balanceLabel
                    width: parent.width
                    wrapMode: Text.Wrap
                }
            }

            SectionDelegate {
                width: parent.width
                //% "Userdata"
                sectionName: qsTrId("qtn_web2sms_editaccountpage_userdata")
            }

            // TODO: Dynamically load plugin's settings
            Loader {
                id: loader
                width: parent.width
                visible: (account.status == Account.Ready)
            }
        }
    }

    BusyIndicator {
        anchors.centerIn: flickable

        Component.onCompleted: {
            platformStyle.size = "large";
        }

        running: (account.status == Account.Loading)
        visible: running
    }

    tools: ToolBarLayout {
        ToolIcon {
            id: backButton
            iconId: "toolbar-back"
            onClicked: __goBack();
        }

        ToolButtonRow {
            ToolButton {
                //% "Ready"
                text: qsTrId("qtn_web2sms_editaccount_save")
                onClicked: __saveSettings()
            }
        }

        ToolIcon {
            id: menuButton
            iconId: "toolbar-view-menu"
            onClicked: (myMenu.status == DialogStatus.Closed) ? myMenu.open() : myMenu.close()
            visible: (setupType != "CreateNew")
        }

        // Hackity ;)
        Item {
            width: backButton.width
            height: backButton.height
            visible: !menuButton.visible
        }
    }

    Menu {
        id: myMenu

        MenuLayout {
            MenuItem {
                //% "Delete"
                text: qsTrId("qtn_web2sms_editaccount_delete")
                onClicked: {
                    account.remove()
                    account.sync();
                }
            }
        }
    }
}
