import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Page {
    id: container

    property QtObject parentPage

    property bool quitWhenCreated: false
    property string setupType

    anchors.margins: UIConstants.DEFAULT_MARGIN
    orientationLock: PageOrientation.LockPortrait

    function createNewAccount(providerId) {
        var component = Qt.createComponent("EditAccountPage.qml");

        // Do some error checking
        if ( component.status != Component.Ready ) {
            console.log("Error creating EditAccountPage.qml: ", component.errorString());
            return;
        }

        var parameters = {
            "setupType":"CreateNew",
            "account":backend.createAccount(providerId)
        };

        var page = component.createObject(container, parameters);
        page.finished.connect(container.didCreateAccount);
        pageStack.push(page);
    }

    function didCreateAccount() {
        console.log("didCreateAccount");
        if ( quitWhenCreated )
            Qt.quit();
        else {
            // Don't show "this" page again
            pageStack.pop(parentPage);
        }
    }

    ListView {
        id: listView
        anchors.fill: parent
        flickableDirection: Flickable.VerticalFlick

        header: Column {
            spacing: 5
            width: listView.width
            height: 110

            Title {
                width: parent.width
                //% "Web2SMS setup"
                title: qsTrId("qtn_web2sms_providerselectionpage_title")
                //% "Do you miss your provider? Tap here."
                subtitle: qsTrId("qtn_web2sms_providerselectionpage_missing_provider")

                MouseArea {
                    anchors.fill: parent
                    onClicked: Qt.openUrlExternally("http://projects.developer.nokia.com/web2sms/wiki#Howtorequestanewprovider");
                }
            }

            Label {
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
                //% "No providers available"
                text: qsTrId("qtn_web2sms_providerselectionpage_empty")
                color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
                visible: listView.count == 0
            }
        }

        model: ProvidersModel {}

        delegate: ListDelegate {
            MoreIndicator {
                anchors {
                    right: parent.right
                    verticalCenter: parent.verticalCenter
                }
            }

            onClicked: createNewAccount(model.providerId)
        }

        ScrollDecorator {
            flickableItem: parent
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-back"
            onClicked: (pageStack.depth > 1) ? pageStack.pop() : Qt.quit();
        }
    }
}
