import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import Web2SMS 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants

Page {
    id: standalonePage

    anchors.margins: UIConstants.DEFAULT_MARGIN
    orientationLock: PageOrientation.LockPortrait

    property bool hasBackButton: true

    function createAccount() {
        // Create a new setup page and push it to the stack
        var component = Qt.createComponent("ProviderSelectionPage.qml");

        var page = component.createObject(standalonePage);
        page.parentPage = standalonePage;
        pageStack.push(page);
    }

    function editAccount(accountId) {
        var component = Qt.createComponent("EditAccountPage.qml");

        if ( component.status != Component.Ready )
            console.log(component.errorString());

        var parameters = {
            "setupType":"EditExisting",
            "account":backend.accountFromId(accountId)
        };

        // Create the page and fill in information
        var page = component.createObject(standalonePage, parameters);
        page.finished.connect(pageStack.pop); // Pop back when user finishes
        pageStack.push(page);
    }

    tools: ToolBarLayout {
        ToolIcon {
            id: backButton
            iconId: "toolbar-back"
            onClicked: standalonePage.pageStack.pop()
            visible: standalonePage.hasBackButton
        }

        ToolButton {
            //% "Add account"
            text: qsTrId("qtn_web2sms_standalonepage_new")
            onClicked: createAccount()
            width: 250
        }

        // Dummy item, without size the create button is not centered
        Item {
            width: backButton.width
            height: backButton.height
        }
    }

    ListView {
        anchors.fill: parent

        header: Column {
            id: column
            width: parent.width

            spacing: 30 // TODO: Find a UiConstants value

            Title {
                anchors {
                    left: parent.left
                    right: parent.right
                }

                //% "Your accounts"
                title: qsTrId("qtn_web2sms_standalonepage_title")
                //% "Configure your Web2SMS accounts"
                //subtitle: qsTrId("qtn_web2sms_standalonepage_subtitle")
                subtitle: ""
            }
        }

        model: AccountsModel {
            id: accountsModel
            providerFilter: "web2sms"
        }

        delegate: AccountDelegate {
            MoreIndicator {
                anchors {
                    right: parent.right
                    verticalCenter: parent.verticalCenter
                }
            }

            showSelection: false
            showImage: false
            onClicked: editAccount(model.id)
        }

        Label {
            anchors.centerIn: parent
            //% "You don't have any accounts"
            text: qsTrId("qtn_web2sms_no_accounts")
            color: UIConstants.COLOR_SECONDARY_FOREGROUND
            visible: (accountsModel.count == 0)
        }

        ScrollDecorator {
            flickableItem: parent
        }
    }

    Menu {
        id: myMenu

        MenuLayout {
            MenuItem {
                //% "Quit"
                text: qsTrId("qtn_web2sms_standalonepage_quit")
                onClicked: Qt.quit()
            }
        }
    }
}
