// Global includes
#include <AccountSetup/ProviderPluginProcess>
#include <QtDeclarative>
#include <QDebug>
#include <QSettings>
#include <QTextStream>
#include <QFile>
#include <MDeclarativeCache>
#include <MLocale>

#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

// Translation strings for .service file
#if 0
//% "Web2SMS"
qtTrId("qtn_web2sms_provider_name");
//% "Send SMS via Internet"
qtTrId("qtn_web2sms_provider_desc");
#endif

using namespace Accounts;

void setWindowChainedProperty( const Window &parentWinId, const Window &childWinId )
{
    Atom atomMInvokedBy = XInternAtom(QX11Info::display(), "_MEEGOTOUCH_WM_INVOKED_BY", False);

    Display *display = QX11Info::display();

    // for compositor page transition
    XChangeProperty(display, childWinId, atomMInvokedBy, XA_WINDOW, 32, PropModeReplace, (unsigned char *)&parentWinId, 1);

    // for task switcher view stacking
    XSetTransientForHint(display, childWinId, parentWinId);
}

void appendMApplicationWindowTypeProperty( const Window &childWinId )
{
    Atom atomWindowType = XInternAtom(QX11Info::display(), "_MEEGOTOUCH_NET_WM_WINDOW_TYPE_MAPPLICATION", False);
    XChangeProperty(QX11Info::display(), childWinId,
                    XInternAtom(QX11Info::display(), "_NET_WM_WINDOW_TYPE", False),
                    XA_ATOM, 32, PropModeAppend, (unsigned char*) &atomWindowType, 1);
}

Q_DECL_EXPORT int main(int argc, char **argv)
{
    QApplication* app = MDeclarativeCache::qApplication(argc, argv);

    // Load translations
    MLocale locale;
    locale.installTrCatalog("web2sms");

    MLocale::setDefault(locale);

    AccountSetup::ProviderPluginProcess* plugin = new AccountSetup::ProviderPluginProcess;

    // Make sure no one was faster than us
    if ( plugin != AccountSetup::ProviderPluginProcess::instance() )
        qFatal("Instance not unique\n");

    // Get the setup type which will be accessible from qml
    QString setupType;
    switch(plugin->setupType()) {
    case AccountSetup::CreateNew:
        setupType = "CreateNew";
        plugin->setReturnToAccountsList(true);
        break;

    case AccountSetup::EditExisting:
        setupType = "EditExisting";
        break;

    case AccountSetup::Unset:
        setupType = "Unset";
        break;
    }

    QDeclarativeView* view = MDeclarativeCache::qDeclarativeView();
    view->rootContext()->setContextProperty("setupType", setupType);

    // Adjust the input path list so that the application can find our qml plugin
    QDeclarativeEngine* engine = view->engine();
    QStringList importPathList = engine->importPathList();
    importPathList << "/opt/web2sms/lib/imports";
    engine->setImportPathList(importPathList);

    QObject::connect(view->engine(), SIGNAL(quit()), app, SLOT(quit()));

    // TODO: On N950 beta image the program startup is a bit slower than pure meegotouch. This should be checked
    //       when the final release is out.
    setWindowChainedProperty(plugin->parentWindowId(), view->winId());
    appendMApplicationWindowTypeProperty(view->winId());

    view->setSource( QUrl::fromLocalFile("/opt/web2sms/accounts/main.qml") );

#ifdef __arm__
    // We adjust the screen rotation from here as we have no chance in qml to detect whether we run in scratchbox or on the device
    QObject* screenObject = qvariant_cast<QObject*>(view->rootContext()->contextProperty("screen"));
    if ( screenObject ) {
        screenObject->setProperty("allowedOrientations", 1); // 1 is Portrait only
    }
#endif // __arm__

    // TODO: The current git version of qt-components has a bug which won't let us change
    //       the theme inversion state from the beginning inside of qml.
    QObject* themeObject = qvariant_cast<QObject*>(view->rootContext()->contextProperty("theme"));
    if ( themeObject )
        themeObject->setProperty("inverted", true);

    view->showFullScreen();

    int ret = app->exec();

    delete view;
    delete plugin;
    delete app;

    return ret;
}
