#ifndef ACCOUNTSMODEL_H
#define ACCOUNTSMODEL_H

// Local includes
#include "global.h"

// Global includes
#include <QAbstractListModel>
#include <QDeclarativeParserStatus>

namespace Accounts {
    class Account;
}

class AccountsModelPrivate;

class WEB2SMS_API AccountsModel : public QAbstractListModel
                                , public QDeclarativeParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QDeclarativeParserStatus)
    Q_PROPERTY(QString providerFilter READ providerFilter WRITE setProviderFilter NOTIFY providerFilterChanged)
    Q_PROPERTY(int count READ count NOTIFY countChanged)

public:
    explicit AccountsModel(QObject *parent = 0);
    ~AccountsModel();

    void setProviderFilter(const QString& providerFilter);
    QString providerFilter() const;

    // Public functions
    Accounts::Account* account(int row) const;

    /// Find the row where the given account id is
    Q_INVOKABLE int indexFromAccountId(quint32 accountId) const;

    enum Roles
    {
        DisplayNameRole = Qt::UserRole + 1,
        IdRole,
        EnabledRole,
        ProviderIdRole,
        UsernameRole,
        AccountRole
    };

    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    int count() const;

    // QDeclarativeParserStatus overrides
    void classBegin();
    void componentComplete();

signals:
    void providerFilterChanged();
    void countChanged();

protected:
    friend class AccountsModelPrivate;
    AccountsModelPrivate* d;
};

#endif // ACCOUNTSMODEL_H
