#ifndef ACCOUNTWRAPPER_H
#define ACCOUNTWRAPPER_H

// Local includes
#include "global.h"

// Global includes
#include <QObject>
#include <QVariant>

namespace Accounts {
    class Account;
}

// Forward declarations
class AccountWrapperPrivate;

class WEB2SMS_API AccountWrapper : public QObject
{
    Q_OBJECT
    Q_ENUMS(Status)
    Q_PROPERTY(int status READ status NOTIFY statusChanged)
    Q_PROPERTY(QString displayName READ displayName WRITE setDisplayName NOTIFY displayNameChanged)
    Q_PROPERTY(bool enabled READ enabled WRITE setEnabled NOTIFY enabledChanged)
    Q_PROPERTY(QString username READ username WRITE setUsername NOTIFY usernameChanged)
    Q_PROPERTY(QString password READ password WRITE setPassword NOTIFY passwordChanged)
    Q_PROPERTY(QString providerId READ providerId WRITE setProviderId NOTIFY providerIdChanged)
    Q_PROPERTY(QVariant pluginCredentials READ pluginCredentials WRITE setPluginCredentials NOTIFY pluginCredentialsChanged)
    Q_PROPERTY(QString balance READ balance WRITE setBalance NOTIFY balanceChanged)

public:
    explicit AccountWrapper(QObject *parent = 0);
    ~AccountWrapper();

    enum Status
    {
        Unknown,
        Error,
        Ready,
        Loading
    };

    Status status() const;

    void setAccount(Accounts::Account* account);
    Accounts::Account* account() const;

    void setDisplayName(const QString& displayName);
    QString displayName() const;

    void setEnabled(bool enabled);
    bool enabled() const;

    void setUsername(const QString& username);
    QString username() const;

    void setPassword(const QString& password);
    QString password() const;

    void setProviderId(const QString& providerId);
    QString providerId() const;

    void setPluginCredentials(const QVariant& pluginCredentials);
    QVariant pluginCredentials() const;

    /// The balance as string
    void setBalance(const QString& balance);
    QString balance() const;

    /// Get/Set properties
    Q_INVOKABLE void setValue(const QString& key, const QVariant& value);
    Q_INVOKABLE QString valueAsString(const QString& key) const;

    Q_INVOKABLE void remove();
    Q_INVOKABLE void sync();

signals:
    void statusChanged();
    void displayNameChanged();
    void enabledChanged();
    void usernameChanged();
    void passwordChanged();
    void providerIdChanged();
    void pluginCredentialsChanged();
    void balanceChanged();
    void synced();

private slots:
    void updateCachedEntries();

protected:
    AccountWrapperPrivate* d;
    friend class AccountWrapperPrivate;
};

#endif // ACCOUNTWRAPPER_H
