#ifndef BACKEND_H
#define BACKEND_H

// Local includes
#include "pluginresolver.h"
#include "messagehandler.h"

// Forward declarations
class AccountWrapper;
class BackendPrivate;

class WEB2SMS_API Backend : public QObject
{
    Q_OBJECT
    Q_PROPERTY(AccountWrapper* account READ account NOTIFY accountChanged)
    Q_PROPERTY(PluginResolver* plugins READ plugins CONSTANT)
    Q_PROPERTY(MessageHandler* messageHandler READ messageHandler CONSTANT)
    Q_PROPERTY(bool debugMode READ debugMode NOTIFY debugModeChanged)

public:
    explicit Backend(QObject *parent = 0);
    ~Backend();

    /// Determine whether we are in debug mode or not
    void setDebugMode(bool debugMode);
    bool debugMode() const;

    // Raise the window (to bring it to front)
    Q_INVOKABLE void raiseWindow();

    /// Create a new account (only needed in debug mode!). Sets the current account.
    Q_INVOKABLE AccountWrapper* createAccount(const QString& providerId);

    /// Get an account by it's id
    /// TODO: Check parameter types
    Q_INVOKABLE AccountWrapper* accountFromId(const QString& accountId);

    /// Get the current account
    AccountWrapper* account() const;

    /// Get the plugins
    PluginResolver* plugins() const;

    /// Get the message handler
    MessageHandler* messageHandler() const;

    /// Check if a number is international. This means it should begin with +xx or 00xx
    Q_INVOKABLE bool isInternationalNumber(const QString& number) const;

signals:
    void accountChanged();
    void debugModeChanged();

protected:
    friend class BackendPrivate;
    BackendPrivate* d;
};

#endif // BACKEND_H
