#ifndef CONTACTMODEL_H
#define CONTACTMODEL_H

// Local includes
#include "global.h"

// Global includes
#include <QAbstractListModel>
#include <QDeclarativeParserStatus>

class ContactModelPrivate;

class WEB2SMS_API ContactModel : public QAbstractListModel
                               , public QDeclarativeParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QDeclarativeParserStatus)
    Q_PROPERTY(QVariantList contacts READ contacts NOTIFY contactsChanged)
    Q_PROPERTY(bool autoUpdate READ autoUpdate WRITE setAutoUpdate NOTIFY autoUpdateChanged)
    Q_PROPERTY(QString nameFilter READ nameFilter WRITE setNameFilter NOTIFY nameFilterChanged)

public:
    explicit ContactModel(QObject *parent = 0);
    ~ContactModel();

    enum Roles
    {
        ContactRole = Qt::UserRole + 1,
        DisplayLabelRole,
        SectionRole
    };

    void setAutoUpdate(bool autoUpdate);
    bool autoUpdate() const;

    void setNameFilter(const QString& nameFilter);
    QString nameFilter() const;

    QVariantList contacts() const;

    Q_INVOKABLE void update();

    // QDeclarativeParserStatus overrides
    void classBegin();
    void componentComplete();

    // QAbstractListModel overrides
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;

signals:
    void autoUpdateChanged();
    void nameFilterChanged();
    void contactsChanged();

protected:
    ContactModelPrivate* d;
    friend class ContactModelPrivate;
};

#endif // CONTACTMODEL_H
