#ifndef CONVERSATIONMODEL_H
#define CONVERSATIONMODEL_H

// Local includes
#include "global.h"

// Global includes
#include <QDeclarativeParserStatus>
#include <QAbstractListModel>

#ifdef MEEGO_EDITION_HARMATTAN
#include <CommHistory/ConversationModel>
#include <ContextProvider>
#endif // MEEGO_EDITION_HARMATTAN

class ConversationModelPrivate;

class WEB2SMS_API ConversationModel : public QAbstractListModel
                                    , public QDeclarativeParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QDeclarativeParserStatus)
    Q_ENUMS(Status)
    Q_PROPERTY(int groupId READ groupId WRITE setGroupId NOTIFY groupIdChanged)
    Q_PROPERTY(QString observedConversation READ observedConversation WRITE setObservedConversation NOTIFY observedConversationChanged)
    Q_PROPERTY(int status READ status NOTIFY statusChanged)
    Q_ENUMS(EventStatus)

public:
    explicit ConversationModel(QObject *parent = 0);
    ~ConversationModel();

    enum Status
    {
        Unknown,
        Loading,
        Ready,
        Error
    };

    enum Roles
    {
        TextRole = Qt::UserRole + 1,
        StatusRole,
        StatusTextRole,
        DirectionRole,
        EndTimeRole,
        MessageTokenRole,
        EventIdRole,
        IsReadRole
    };

    enum EventStatus {
        UnknownStatus = 0,
        SendingStatus,
        SentStatus,
        DeliveredStatus,
        TemporarilyFailedStatus,
        PermanentlyFailedStatus
    };

    void setGroupId(int groupId);
    int groupId() const;

    /// Define if the model should handle the observation of messages.
    /// When this feature is turned on, the system will not play the
    /// ringtone and vibrate for incoming messages of this conversation.
    void setObservedConversation(const QString& observedConversation);
    QString observedConversation() const;

    Status status() const;

    Q_INVOKABLE bool deleteEvent(int eventId);

    // QAbstractListModel overrides
    int rowCount(const QModelIndex&) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);

    // QDeclarativeParserStatus overrides
    void classBegin();
    void componentComplete();

signals:
    void groupIdChanged();
    void observedConversationChanged();
    void statusChanged();

    void incomingMessage(); // This signal is triggered when a sound has to be played

private slots:
    void onModelReady(bool success);
    void onModelReset();
    void onRowsInserted(const QModelIndex&, int start, int end);
    void onRowsRemoved(const QModelIndex&, int start, int end);
    void onDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);

protected:
    void setStatus(Status status);
    void markReadMessages();

protected:
    ConversationModelPrivate* d;
    friend class ConversationModelPrivate;

    Status m_status;
    bool m_initialized;
    int m_groupId;

#ifdef MEEGO_EDITION_HARMATTAN
    QList<CommHistory::Event> m_events;
    CommHistory::ConversationModel m_model;

    ContextProvider::Service m_service;
    ContextProvider::Property* m_observedConversation;
#elif defined(Q_WS_MAEMO_5)
#endif
};

#endif // CONVERSATIONMODEL_H
