#ifndef GROUPMODEL_H
#define GROUPMODEL_H

// Local includes
#include "global.h"

// Global includes
#include <QDeclarativeParserStatus>
#include <QContactManager>

#if defined(MEEGO_EDITION_HARMATTAN)
#include <CommHistory/GroupModel>
typedef CommHistory::GroupModel GroupModelBase;
#elif defined(Q_WS_MAEMO_5) || defined(Q_OS_ANDROID)
#include <QAbstractListModel>
typedef QAbstractListModel GroupModelBase;
#else
#error "Unknown build configuration"
#endif

QTM_USE_NAMESPACE

class GroupModelPrivate;

class WEB2SMS_API GroupModel : public GroupModelBase
                             , public QDeclarativeParserStatus
{
    Q_OBJECT
    Q_ENUMS(Roles)
    Q_INTERFACES(QDeclarativeParserStatus)
    Q_PROPERTY(QString remoteUid READ remoteUid WRITE setRemoteUid NOTIFY remoteUidChanged)
    Q_PROPERTY(int count READ count NOTIFY countChanged)

public:
    explicit GroupModel(QObject *parent = 0);
    ~GroupModel();

    enum Roles
    {
        GroupIdRole = Qt::UserRole + 1,
        ContactNameRole,
        ContactIdRole,
        LastMessageTextRole,
        EndTimeRole,
        RemoteUidRole, // TODO: Return multiple remote uids?
        SectionRole,
        UnreadMessagesRole,
        LastEventStatusRole
    };

    void setRemoteUid(const QString& remoteUid);
    QString remoteUid() const;

    /// Get the data for a given row and role
    Q_INVOKABLE QVariant data(int row, int role) const;

    /// Get the groupId for a given remote uid
    Q_INVOKABLE int groupIdForRemoteUid(const QString& remoteUid);

    /// Get the row for a given remote uid
    Q_INVOKABLE int rowForRemoteUid(const QString& remoteUid);

    // CommHistory::GroupModel overrides
    QVariant data(const QModelIndex &index, int role) const;

    // QDeclarativeParserStatus overrides
    void classBegin();
    void componentComplete();

    // Get function for the FastScroller
    Q_INVOKABLE QVariant get(int row) const;
    int count() const;
    Q_SIGNAL void countChanged();

#ifndef MEEGO_EDITION_HARMATTAN
    // QAbstractListModel overrides
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
#endif // !MEEGO_EDITION_HARMATTAN

signals:
    void remoteUidChanged();

protected:
    GroupModelPrivate* d;
    friend class GroupModelPrivate;

    QContactManager m_contactManager;
    QString m_remoteUid;
};

#endif // GROUPMODEL_H
