// Local includes
#include "groupmodel.h"
#include "conversationmodel.h"

// Global includes
#include <QDebug>
#include <QDateTime>
#include <QContact>
#include <QContactDisplayLabel>
//#include <CommHistory/commonutils.h>

struct Group
{
    Group()
    : groupId(-1)
    , contactId(-1)
    , unreadMessages(0)
    , lastEventStatus(ConversationModel::DeliveredStatus)
    {
    }

    int groupId;
    QString contactName;
    int contactId;
    QString lastMessageText;
    QDateTime endTime;
    QString remoteUid;
    int unreadMessages;
    ConversationModel::EventStatus lastEventStatus;
};

class GroupModelPrivate
{
public:
    GroupModelPrivate()
    {
    }

    GroupModel* self;

    // Properties
    QList<Group> groups;

    // Functions
    void getGroups();
};

void GroupModelPrivate::getGroups()
{
    self->beginResetModel();
    groups.clear();

    // TODO: Implementation

    self->endResetModel();
}

GroupModel::GroupModel(QObject *parent)
: QAbstractListModel(parent)
, d(new GroupModelPrivate)
{
    d->self = this;

    // Hackity
    connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SIGNAL(countChanged()));
    connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SIGNAL(countChanged()));
    connect(this, SIGNAL(modelReset()), this, SIGNAL(countChanged()));

    QHash<int, QByteArray> roleNames;
    roleNames.insert(GroupIdRole, "groupId");
    roleNames.insert(ContactNameRole, "contactName");
    roleNames.insert(ContactIdRole, "contactId");
    roleNames.insert(LastMessageTextRole, "lastMessageText");
    roleNames.insert(EndTimeRole, "endTime");
    roleNames.insert(RemoteUidRole, "remoteUid");
    roleNames.insert(SectionRole, "section");
    roleNames.insert(UnreadMessagesRole, "unreadMessages");
    roleNames.insert(LastEventStatusRole, "lastEventStatus");

    setRoleNames(roleNames);
}

GroupModel::~GroupModel()
{
    delete d;
}

void GroupModel::classBegin()
{
}

void GroupModel::componentComplete()
{
    // Update the model
    d->getGroups();
}

QVariant GroupModel::get(int row) const
{
    // Sanity check
    QModelIndex indexOfRow = index(row, 0);
    if ( indexOfRow.isValid() == false )
        return QVariant();

    // Create a small map that returns the section
    QVariantMap map;
    map.insert("section", indexOfRow.data(SectionRole));

    return QVariant::fromValue(map);
}

int GroupModel::count() const
{
    return rowCount();
}

void GroupModel::setRemoteUid(const QString& remoteUid)
{
    if ( m_remoteUid == remoteUid )
        return;

    m_remoteUid = remoteUid;
    emit remoteUidChanged();
}

QString GroupModel::remoteUid() const
{
    return m_remoteUid;
}

QVariant GroupModel::data(int row, int role) const
{
    return data(index(row, 0), role);
}

int GroupModel::rowForRemoteUid(const QString &remoteUid)
{
    /*int rows = rowCount();
    for(int i=0; i < rows; ++i) {
        QModelIndex index = this->index(i, 0);
        CommHistory::Group group = this->group(index);

        foreach(const QString& tmp, group.remoteUids()) {
            if ( CommHistory::remoteAddressMatch(remoteUid, tmp) )
                return i;
        }
    }*/

    return -1;
}

int GroupModel::groupIdForRemoteUid(const QString &remoteUid)
{
    /*int rows = rowCount();
    for(int i=0; i < rows; ++i) {
        QModelIndex index = this->index(i, 0);
        CommHistory::Group group = this->group(index);

        foreach(const QString& tmp, group.remoteUids()) {
            if ( CommHistory::remoteAddressMatch(remoteUid, tmp) )
                return group.id();
        }
    }*/

    return -1;
}

QVariant GroupModel::data(const QModelIndex &index, int role) const
{
    // Get the group at the given position
    Group group = d->groups.at(index.row());

    switch(role) {
    case GroupIdRole:
        return QVariant::fromValue(group.groupId);

    case RemoteUidRole:
        return group.remoteUid;

    case ContactIdRole:
        return group.contactId;

    case ContactNameRole:
        return group.contactName;

    case LastMessageTextRole:
        return group.lastMessageText;

    case UnreadMessagesRole:
        return group.unreadMessages;

    case LastEventStatusRole:
        return group.lastEventStatus;

    case EndTimeRole:
        {
            QDateTime now = QDateTime::currentDateTime();
            QDateTime dt = group.endTime.toLocalTime();

            QString time = dt.time().toString(Qt::SystemLocaleShortDate);
            if ( dt.date() == now.date() ) {
                return time;
            } else {
                QString date = dt.date().toString(Qt::SystemLocaleShortDate);
                //% "%1 | %2"
                //: "DateTime format in GroupModel. %1 is the Date, %2 is the Time. Normal format is date | time if not the same day"
                return qtTrId("qtn_web2sms_datetime_format").arg(date).arg(time);
            }
        }

    case SectionRole:
        {
            // TODO: Calculate the section time
            QDateTime now = QDateTime::currentDateTime();
            QDateTime dt = group.endTime.toLocalTime();

            QDate groupDate = dt.date();
            QDate nowDate = now.date();

            if ( groupDate == nowDate ) {
                //% "Today"
                return qtTrId("qtn_web2sms_section_today");
            } else {
                QDate yesterday = nowDate.addDays(-1);
                if ( groupDate == yesterday ) {
                    //% "Yesterday"
                    return qtTrId("qtn_web2sms_section_yesterday");
                }

                // Calculate difference in weeks
                if ( dt.date().year() == now.date().year() ) {
                    int weeks = nowDate.weekNumber() - groupDate.weekNumber();

                    // Return the name of the day
                    if ( weeks == 0 || weeks == -1 ) {
                        return dt.toString("dddd");
                    } else if ( weeks == 1 ) {
                        //% "Last week"
                        return qtTrId("qtn_web2sms_section_last_week");
                    } else if ( weeks > 0 && weeks < 4 ) {
                        //% "%n weeks ago"
                        return qtTrId("qtn_web2sms_section_weeks_ago", weeks);
                    } else {
                        return dt.toString("MMMM");
                    }
                } else {
                    //% "%1, %2"
                    //: "This is the date formatting for messages a long time ago. %1 is the month like January, %2 is the year."
                    return qtTrId("qtn_web2sms_section_month_year").arg(dt.toString("MMMM")).arg(dt.date().year());
                }
            }
        }

    default:
        return QVariant();
    }
}

int GroupModel::rowCount(const QModelIndex &parent) const
{
    if ( parent.isValid() )
        return 0;

    return d->groups.size();
}
