#ifndef MESSAGEHANDLER_H
#define MESSAGEHANDLER_H

// Local includes
#include "accountsmodel.h"
#include "accountwrapper.h"
#include "conversationmodel.h"

// Global includes
#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>

#ifdef MEEGO_EDITION_HARMATTAN
#include <CommHistory/GroupModel>
#include <CommHistory/EventModel>

#include <ovinotificationsession.h>
#include <ovinotificationstate.h>
#include <ovinotificationmessage.h>
#include <ovinotificationpayload.h>
#endif // MEEGO_EDITION_HARMATTAN

class WEB2SMS_API MessageHandler : public QObject
{
    Q_OBJECT
    Q_ENUMS(OnlineStatus)
    Q_PROPERTY(QString serverAddress READ serverAddress WRITE setServerAddress NOTIFY serverAddressChanged)
    Q_PROPERTY(int onlineStatus READ onlineStatus NOTIFY onlineStatusChanged)

public:
    explicit MessageHandler(QObject *parent = 0);
    ~MessageHandler();

    enum OnlineStatus
    {
        Offline,
        Online,
        Connecting,
        Error
    };

    void setServerAddress(const QString& serverAddress);
    QString serverAddress() const;

    OnlineStatus onlineStatus() const;

    /// TODO: Should create a message object and messagetype parameter
    /// @returns A the group id where the message was put in
    Q_INVOKABLE int sendMessage(AccountWrapper* account, const QVariantList& contacts, const QString& text);
    Q_INVOKABLE bool resendMessage(AccountWrapper* account, const QString& messageToken);

    Q_INVOKABLE int createGroupWithNumber(const QString& number);
    Q_INVOKABLE int findGroupForNumber(const QString& number) const;

signals:
    void serverAddressChanged();
    void onlineStatusChanged();
    void error(const QString& text);

protected:
    /// Send the message via remote server
    void remoteSendMessage(const QString& eventId, const AccountWrapper* account, const QVariantList &contacts, const QString& text);

    bool initializeNotifications();

    void processMessage(const QString& eventId, const QVariant& data);
    void processError(const QString& eventId, const QString& text);

    void changeMessageStatus(const QString& eventId, ConversationModel::EventStatus status);

private slots:
    void didFinishRequest();

    // For notifications
    void didChangeState(QObject* object);
    void didReceiveNotification(QObject* object);

protected:
    QString m_serverAddress;
    OnlineStatus m_onlineStatus;

#ifdef MEEGO_EDITION_HARMATTAN
    CommHistory::GroupModel m_groupModel;
    CommHistory::EventModel m_eventModel;

    OviNotificationSession* session;
#endif // MEEGO_EDITION_HARMATTAN

    AccountsModel accounts;
    QNetworkAccessManager network;

    QHash<QString, AccountWrapper*> m_eventIdToAccount;
};

#endif // MESSAGEHANDLER_H
