#ifndef MESSAGESTATISTICS_H
#define MESSAGESTATISTICS_H

// Local includes
#include "qgsmcodec.h"
#include "messagetype.hpp"

// Global includes
#include <QObject>
#include <QDeclarativeParserStatus>
#include <QVariantList>

class WEB2SMS_API MessageStatistics : public QObject
                                    , public QDeclarativeParserStatus
{
    Q_OBJECT
    Q_INTERFACES(QDeclarativeParserStatus)
    Q_PROPERTY(QString text READ text WRITE setText NOTIFY modified)
    Q_PROPERTY(MessageType* messageType READ messageType WRITE setMessageType NOTIFY modified)
    Q_PROPERTY(Charset charset READ charset NOTIFY modified)
    Q_PROPERTY(State state READ state NOTIFY modified)
    Q_PROPERTY(QString errorString READ errorString NOTIFY modified)
    Q_PROPERTY(QVariantList contacts READ contacts WRITE setContacts NOTIFY modified)
    Q_PROPERTY(int charactersLeft READ charactersLeft NOTIFY modified)
    Q_PROPERTY(int messageCharacterCount READ messageCharacterCount NOTIFY modified)
    Q_PROPERTY(int messageCount READ messageCount NOTIFY modified)
    Q_PROPERTY(int completeCharacterCount READ completeCharacterCount NOTIFY modified)
    Q_ENUMS(State)
    Q_ENUMS(Charset)

public:
    explicit MessageStatistics(QObject *parent = 0);

    enum State
    {
        Error = 0,
        Ok
    };

    enum Charset
    {
        GSM,
        Unicode
    };

    void setText(const QString& text);
    QString text() const;

    void setMessageType(MessageType* messageType);
    MessageType* messageType() const;

    Charset charset() const;
    State state() const;
    QString errorString() const;

    void setContacts(const QVariantList& contacts);
    QVariantList contacts() const;

    int charactersLeft() const;
    int messageCharacterCount() const;
    int messageCount() const;
    int completeCharacterCount() const;

    // QDeclarativeParserStatus overrides
    void classBegin();
    void componentComplete();

signals:
    void modified();

protected:
    void recalc();

protected:
    QGsmCodec gsmCodec;
    QString m_text;
    MessageType* m_messageType;
    QVariantList m_contacts;

    Charset m_charset;
    State m_state;
    QString m_errorString;

    int m_charactersLeft;
    int m_messageCharacterCount;
    int m_messageCount;
    int m_completeCharacterCount;
};

#endif // MESSAGESTATISTICS_H
