// Local includes
#include "messagetype.hpp"

// Global includes
#include <QtCore/QSharedData>

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

class MessageTypeData : public QSharedData
{
public:
  MessageTypeData()
  {}

  MessageTypeData(const MessageTypeData& cpy)
  : QSharedData(cpy)
  , visibleName(cpy.visibleName)
  , name(cpy.name)
  , icon(cpy.icon)
  , information(cpy.information)
  , singleMessageLength(cpy.singleMessageLength)
  , maxMessageLength(cpy.maxMessageLength)
  , maxReceivers(cpy.maxReceivers)
  {}

  QString visibleName;     // tr("Extra long SMS")
  QString name;            // extralongsms
  QIcon icon;
  QString information;     // tr("You can send 500 characters for 5$")
  int singleMessageLength; // Length of "one" message
  int maxMessageLength;    // Maximum of characters that can be sent
  int maxReceivers;        // Maximum number of recipients
};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

MessageType::MessageType(const QString& visibleName,
                         const QString& name,
                         const QIcon& icon,
                         const QString& information,
                         int singleMessageLength,
                         int maxMessageLength,
                         int maxReceivers)
: data(new MessageTypeData)
{
  data->visibleName = visibleName;
  data->name = name;
  data->icon = icon;
  data->information = information;
  data->singleMessageLength = singleMessageLength;
  data->maxMessageLength = maxMessageLength;
  data->maxReceivers = maxReceivers;
}

//////////////////////////////////////////////////////////////////////////

MessageType::~MessageType()
{
}

//////////////////////////////////////////////////////////////////////////

QString MessageType::visibleName() const
{
  return data->visibleName;
}

//////////////////////////////////////////////////////////////////////////

QString MessageType::name() const
{
  return data->name;
}

//////////////////////////////////////////////////////////////////////////

QIcon MessageType::icon() const
{
  return data->icon;
}

//////////////////////////////////////////////////////////////////////////

QString MessageType::information() const
{
  return data->information;
}

//////////////////////////////////////////////////////////////////////////

int MessageType::singleMessageLength() const
{
  return data->singleMessageLength;
}

//////////////////////////////////////////////////////////////////////////

int MessageType::maxMessageLength() const
{
  return data->maxMessageLength;
}

//////////////////////////////////////////////////////////////////////////

int MessageType::maxReceivers() const
{
  return data->maxReceivers;
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
