#ifndef PLUGINRESOLVER_H
#define PLUGINRESOLVER_H

// Local includes
#include "providerinfo.h"

// Global includes
#include <QDeclarativeListProperty>

// Forward declarations
class PluginResolverPrivate;

// TODO: Decide if PluginResolver is a stupid name
class WEB2SMS_API PluginResolver : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QDeclarativeListProperty<ProviderInfo> providers READ providersDeclarative)
    Q_PROPERTY(int pluginTimestamp READ pluginTimestamp NOTIFY pluginTimestampChanged)

    explicit PluginResolver(QObject *parent = 0);

public:
    ~PluginResolver();

    // Get the global instance of this object
    static PluginResolver* instance();

    // Read the provider files
    void resolvePlugins();

    // Unload the plugins
    void unload();

    /// Get the list of providers
    QList<ProviderInfo*> providers() const;

    /// Get the list of providers in declarative format
    QDeclarativeListProperty<ProviderInfo> providersDeclarative();

    /// Find a provider by its id
    Q_INVOKABLE ProviderInfo* providerById(const QString& id) const;

    /// Get the plugin timestamp
    int pluginTimestamp() const;

signals:
    /// The providers have been changed or reloaded
    void providersChanged();
    void pluginTimestampChanged();

protected:
    PluginResolverPrivate* d;
    friend class PluginResolverPrivate;
};

#endif // PLUGINRESOLVER_H
