#include "providerinfo.h"

ProviderInfo::ProviderInfo(QObject *parent)
: QObject(parent)
, m_state(Stable)
{
}

void ProviderInfo::setPath(const QString& path)
{
    m_path = path;
}

void ProviderInfo::setId(const QString& id)
{
    m_id = id;
}

void ProviderInfo::setName(const QString& name)
{
    m_name = name;
}

void ProviderInfo::setDescription(const QString& description)
{
    m_description = description;
}

void ProviderInfo::setIcon(const QString& icon)
{
    m_icon = icon;
}

void ProviderInfo::setPlugin(const QString& plugin)
{
    m_plugin = plugin;
}

void ProviderInfo::setAccountsetup(const QString& accountsetup)
{
    m_accountsetup = accountsetup;
}

void ProviderInfo::setBalanceFormat(const QString &balanceFormat)
{
    // Set the balance format
    m_balanceFormat = balanceFormat;

    // Remember that we allow flags
    m_flags |= SupportsBalance;
}

void ProviderInfo::setState(ProviderInfo::State state)
{
    m_state = state;
}

void ProviderInfo::setSupportsUnicode(bool supportsUnicode)
{
    if ( supportsUnicode )
        m_flags |= SupportsUnicode;
    else
        m_flags &= ~SupportsUnicode;
}

void ProviderInfo::setFlags(ProviderFlags flags)
{
    m_flags = flags;
}

QString ProviderInfo::path() const
{
    return m_path;
}

QString ProviderInfo::id() const
{
    return m_id;
}

QString ProviderInfo::name() const
{
    return m_name;
}

QString ProviderInfo::description() const
{
    return m_description;
}

QString ProviderInfo::icon() const
{
    return m_icon;
}

QString ProviderInfo::plugin() const
{
    return m_plugin;
}

QString ProviderInfo::accountsetup() const
{
    return m_accountsetup;
}

QString ProviderInfo::balanceFormat() const
{
    return m_balanceFormat;
}

ProviderInfo::State ProviderInfo::state() const
{
    return m_state;
}

ProviderInfo::ProviderFlags ProviderInfo::flags() const
{
    return m_flags;
}

bool ProviderInfo::supportsBalance() const
{
    return m_flags.testFlag(SupportsBalance);
}

bool ProviderInfo::supportsUnicode() const
{
    return m_flags.testFlag(SupportsUnicode);
}

bool ProviderInfo::supportsSender() const
{
    return m_flags.testFlag(SupportsSender);
}
