#ifndef PROVIDERINFO_H
#define PROVIDERINFO_H

// Local includes
#include "global.h"

// Global includes
#include <QObject>

class WEB2SMS_API ProviderInfo : public QObject
{
    Q_OBJECT
    Q_FLAGS(ProviderFlag ProviderFlags)
    Q_ENUMS(State)
    Q_PROPERTY(QString path READ path NOTIFY providerInfoChanged)
    Q_PROPERTY(QString id READ id NOTIFY providerInfoChanged)
    Q_PROPERTY(QString name READ name NOTIFY providerInfoChanged)
    Q_PROPERTY(QString description READ description NOTIFY providerInfoChanged)
    Q_PROPERTY(QString icon READ icon NOTIFY providerInfoChanged)
    Q_PROPERTY(QString plugin READ plugin NOTIFY providerInfoChanged)
    Q_PROPERTY(QString accountsetup READ accountsetup NOTIFY providerInfoChanged)
    Q_PROPERTY(QString balanceFormat READ balanceFormat NOTIFY providerInfoChanged)
    Q_PROPERTY(int state READ state NOTIFY providerInfoChanged)
    Q_PROPERTY(bool supportsBalance READ supportsBalance NOTIFY providerInfoChanged)
    Q_PROPERTY(bool supportsUnicode READ supportsUnicode NOTIFY providerInfoChanged)
    Q_PROPERTY(bool supportsSender READ supportsSender NOTIFY providerInfoChanged)

public:
    explicit ProviderInfo(QObject *parent = 0);

    enum ProviderFlag
    {
        None = 0,
        SupportsBalance = 1,
        SupportsUnicode = 2,
        SupportsSender  = 4  // Provider needs a "sender" number
    };
    Q_DECLARE_FLAGS(ProviderFlags, ProviderFlag)

    enum State
    {
        Unstable,
        Alpha,
        Beta,
        Stable
    };

    void setPath(const QString& path);
    void setId(const QString& id);
    void setName(const QString& name);
    void setDescription(const QString& description);
    void setIcon(const QString& icon);
    void setPlugin(const QString& plugin);
    void setAccountsetup(const QString& accountsetup);
    void setBalanceFormat(const QString& balanceFormat);
    void setState(State state);
    void setSupportsUnicode(bool supportsUnicode);
    void setFlags(ProviderFlags flags);

    QString path() const;
    QString id() const;
    QString name() const;
    QString description() const;
    QString icon() const;
    QString plugin() const;
    QString accountsetup() const;
    QString balanceFormat() const;
    State state() const;
    ProviderFlags flags() const;

    bool supportsBalance() const;
    bool supportsUnicode() const;
    bool supportsSender() const;

signals:
    void providerInfoChanged();

private:
    ProviderFlags m_flags;

    // Path to the .xml file
    QString m_path;
    QString m_id;

    QString m_name;
    QString m_description;
    QString m_icon;

    QString m_plugin;

    QString m_accountsetup;
    QString m_balanceFormat;

    State m_state;
};

#endif // PROVIDERINFO_H
