// Local includes
#include "providersmodel.h"
#include "pluginresolver.h"

class ProvidersModelPrivate : public QObject
{
    Q_OBJECT

public:
    ProvidersModel* self;

    // Properties
    PluginResolver* plugins;
    QList<ProviderInfo*> providers;

    // Functions
    void init();

public slots:
    void onProvidersChanged();
};

void ProvidersModelPrivate::init()
{
    // Get the plugin resolver instance
    plugins = PluginResolver::instance();
    connect(plugins, SIGNAL(providersChanged()), this, SLOT(onProvidersChanged()));

    // Load the plugins
    providers = plugins->providers();
}

void ProvidersModelPrivate::onProvidersChanged()
{
    self->beginResetModel();
    // Reassign providers
    providers = plugins->providers();
    self->endResetModel();
}

ProvidersModel::ProvidersModel(QObject *parent)
: QAbstractListModel(parent)
, d(new ProvidersModelPrivate)
{
    d->self = this;
    d->init();

    // Setup the role names
    QHash<int, QByteArray> roleNames;
    roleNames.insert(TitleRole, "title");
    roleNames.insert(SubtitleRole, "subtitle");
    roleNames.insert(IconSourceRole, "iconSource");
    roleNames.insert(ProviderIdRole, "providerId");
    roleNames.insert(AccountSetupRole, "accountSetup");
    roleNames.insert(BalanceFormatRole, "balanceFormat");

    setRoleNames(roleNames);
}

ProvidersModel::~ProvidersModel()
{
    delete d;
}

int ProvidersModel::rowCount(const QModelIndex &parent) const
{
    if ( parent.isValid() )
        return 0;

    return d->providers.size();
}

QVariant ProvidersModel::data(const QModelIndex &index, int role) const
{
    // Sanity check
    if ( index.row() < 0 || index.row() >= d->providers.size() )
        return QVariant();

    // Get the selected provider
    const ProviderInfo* info = d->providers.at(index.row());

    switch(role) {
    case TitleRole:
        return info->name();

    case SubtitleRole:
        {
            // FIXME: Evil hackity to hide untranslated content
            QString subtitle = info->description();
            if ( subtitle.startsWith("qtn_") )
                return QString();
            else
                return subtitle;
        }

    case IconSourceRole:
        //return info->icon();
        return QString();

    case ProviderIdRole:
        return info->id();

    case AccountSetupRole:
        return info->accountsetup();

    case BalanceFormatRole:
        return info->balanceFormat();

    default:
        break;
    }

    return QVariant();
}

void ProvidersModel::classBegin()
{
}

void ProvidersModel::componentComplete()
{
}

#include "providersmodel.moc"
