import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants

BorderImage {
    id: bubble

    property int maxWidth
    property string direction: "Outbound"
    property string text
    property string status
    property alias statusColor: statusLabel.color

    property int inboundBubbleColor: 8
    property int outboundBubbleColor: 1

    property real fontFactor: 1.0

    QtObject {
        id: d

        property int inboundBubbleNumber: parseInt( (bubble.inboundBubbleColor / 2) + 0.5 )
        property int outboundBubbleNumber: parseInt( (bubble.outboundBubbleColor /2) + 0.5 )

        property string inboundBubbleState: (bubble.inboundBubbleColor % 2) == 0 ? "pressed" : "normal"
        property string outboundBubbleState: (bubble.outboundBubbleColor % 2) == 0 ? "pressed" : "normal"
    }

    function calcTextWidth() {
        return Math.max(calcLabel.width, statusLabel.width)
    }

    function calcLabelWidth() {
        return Math.min(calcLabel.width, bubble.maxWidth);
    }

    function calcBubbleWidth() {
        return Math.min(calcTextWidth()+10, bubble.maxWidth+10);
    }

    TextFieldStyle {
        id: textFieldStyle
    }

    Label {
        id: calcLabel
        text: model.text
        visible: false
        font.family: textFieldStyle.textFont
        font.pixelSize: textFieldStyle.textFont.pixelSize * bubble.fontFactor
    }

    width: calcBubbleWidth()
    height: column.height + 22

    border {
        left: 22
        right: 22
        bottom: bubble.direction == "Outbound" ? 36 : 22
        top: bubble.direction == "Outbound" ? 22 : 36
    }

    source: bubble.direction == "Inbound" ?
                "image://theme/meegotouch-messaging-conversation-bubble-incoming" + d.inboundBubbleNumber + "-" + d.inboundBubbleState :
                "image://theme/meegotouch-messaging-conversation-bubble-outgoing" + d.outboundBubbleNumber + "-" + d.outboundBubbleState

    Column {
        id: column

        anchors {
            top: parent.top
            topMargin: bubble.direction == "Inbound" ? 15 : 5
            left: parent.left
            leftMargin: 5
            right: parent.right
            rightMargin: 5
        }

        Label {
            id: textLabel
            text: bubble.text
            width: calcLabelWidth()
            wrapMode: Text.Wrap
            font.pixelSize: 22 * bubble.fontFactor
            color: bubble.direction == "Inbound" ? UIConstants.COLOR_INVERTED_FOREGROUND : UIConstants.COLOR_FOREGROUND
            horizontalAlignment: bubble.direction == "Inbound" ? Text.AlignRight : Text.AlignLeft
            font.family: textFieldStyle.textFont
        }

        Label {
            id: statusLabel
            // Uah, hacky
            x: bubble.direction == "Outbound" ? 0 : (parent.width - width)
            text: bubble.status
            font.pixelSize: 18 * bubble.fontFactor
            horizontalAlignment: bubble.direction == "Inbound" ? Text.AlignRight : Text.AlignLeft
            color: UIConstants.COLOR_SECONDARY_FOREGROUND
            font.family: textFieldStyle.textFont
        }
    }
}
