import QtQuick 1.1
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Item {
    id: bubbleDelegate
    width: ListView.view.width
    height: bubble.height

    signal animationsStarted
    signal animationsFinished

    property variant possibleAnimations: [
        //% "Scale"
        { "name": qsTrId("qtn_web2sms_animation_scale"), "source":"ScaleAnimation.qml" },
        //{ "name": qsTrId("qtn_web2sms_animation_size"), "source":"SizeAnimation.qml" },
        //{ "name": qsTrId("qtn_web2sms_animation_tile"), "source":"TileAnimation.qml" },
        //% "Slide"
        { "name": qsTrId("qtn_web2sms_animation_slide"), "source":"SlideAnimation.qml" }
    ]

    property alias fontFactor: bubble.fontFactor
    property alias inboundBubbleColor: bubble.inboundBubbleColor
    property alias outboundBubbleColor: bubble.outboundBubbleColor
    property int animation: -1

    property QtObject bubbleItem: bubble

    Component.onCompleted: updateAnimation();
    onAnimationChanged: updateAnimation();

    QtObject {
        id: d

        property variant animation
    }

    function updateAnimation() {
        if ( d.animation )
            d.animation.destroy();

        // Do nothing
        if ( bubbleDelegate.animation == -1 )
            return;

        // Create the animation component
        var component = Qt.createComponent( bubbleDelegate.possibleAnimations[bubbleDelegate.animation].source );
        if ( component.status != Component.Ready ) {
            console.log("Error loading component:", component.errorString());
            return;
        }

        d.animation = component.createObject(bubbleDelegate, {"target":bubble});
    }

    BubbleItem {
        id: bubble

        property alias transformation: bubbleRotation

        direction: model.direction
        text: model.text
        status: {
            if ( direction == "Inbound" )
                return model.endTime;
            else {
                switch(model.status) {
                case ConversationModel.SendingStatus:
                case ConversationModel.TemporarilyFailedStatus:
                case ConversationModel.PermanentlyFailedStatus:
                    return model.statusText + " | " + model.endTime;

                    // Roles we don't show anything for
                case ConversationModel.DeliveredStatus:
                case ConversationModel.SentStatus:
                default:
                    return model.endTime;
                }
            }
        }

        statusColor: {
            switch(model.status) {
            case ConversationModel.TemporarilyFailedStatus:
            case ConversationModel.PermanentlyFailedStatus:
                return "red";

            default:
                return UIConstants.COLOR_SECONDARY_FOREGROUND
            }
        }

        maxWidth: parent.width*0.8

        // Move inbound messages to the right of the screen
        property int realX: direction == "Inbound" ? parent.width - width : 0
        x: realX

        transform: Rotation {
            id: bubbleRotation
            origin.x: direction == "Inbound" ? bubble.width : 0
            axis { x: 0; y: 1; z: 0 }
        }
    }

    ListView.onAdd: d.animation.start()

    ListView.onRemove: SequentialAnimation {
        ScriptAction { script: bubbleDelegate.animationsStarted(); }
        PropertyAction { target: bubble; property: "ListView.delayRemove"; value: true }
        NumberAnimation {
            target: bubble;
            property: "scale";
            to: 0;
            duration: 250;
            easing.type: Easing.InBack
        }
        PropertyAction { target: bubble; property: "ListView.delayRemove"; value: false }
        ScriptAction { script: bubbleDelegate.animationsFinished(); }
    }
}
