import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Page {
    id: bubbleSettingsPage

    orientationLock: PageOrientation.LockPortrait
    anchors.margins: UIConstants.DEFAULT_MARGIN

    ListView {
        id: listView

        anchors.fill: parent

        model: VisualItemModel {
            Title {
                width: listView.width
                //% "Bubbles"
                title: qsTrId("qtn_web2sms_settings_bubbles_title")
                subtitle: "" // qsTrId("qtn_web2sms_settings_bubbles_subtitle")
                imageSource: ""
                separator: false
            }

            SectionDelegate {
                width: listView.width
                //% "Configuration"
                sectionName: qsTrId("qtn_web2sms_bubblesettings_section_config")
            }

            Label {
                width: listView.width
                //% "Inbound bubble color"
                text: qsTrId("qtn_web2sms_bubblesettings_inbound")
                color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
            }

            Slider {
                id: inboundSlider
                width: listView.width
                minimumValue: 1
                maximumValue: 8
                stepSize: 1
                value: appSettings.inboundBubbleColor

                Binding {
                    target: appSettings
                    property: "inboundBubbleColor"
                    value: inboundSlider.value
                }
            }

            Label {
                width: listView.width
                //% "Outbound bubble color"
                text: qsTrId("qtn_web2sms_bubblesettings_outbound")
                color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
            }

            Slider {
                id: outboundSlider
                width: listView.width
                minimumValue: 1
                maximumValue: 6
                stepSize: 1
                value: appSettings.outboundBubbleColor

                Binding {
                    target: appSettings
                    property: "outboundBubbleColor"
                    value: outboundSlider.value
                }
            }

            Label {
                width: listView.width
                //% "Font size"
                text: qsTrId("qtn_web2sms_bubblesettings_fontsize")
                color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
            }

            Slider {
                id: fontFactorSlider
                width: listView.width
                minimumValue: 0.5
                maximumValue: 2.0
                stepSize: 0.025
                value: appSettings.bubbleFontFactor

                Binding {
                    target: appSettings
                    property: "bubbleFontFactor"
                    value: fontFactorSlider.value
                }
            }

            Label {
                width: listView.width
                //% "Animation"
                text: qsTrId("qtn_web2sms_bubblesettings_animation")
                color: UIConstants.COLOR_INVERTED_SECONDARY_FOREGROUND
            }

            ButtonRow {
                id: animationRow

                property int selectedAnimation
                property bool initialized: false

                Component {
                    id: buttonComponent

                    Button {
                    }
                }

                Component.onCompleted: {
                    var component = Qt.createComponent("BubbleItemDelegate.qml");
                    if ( component.status != Component.Ready ) {
                        console.log("Failed to load component:", component.errorString());
                        return;
                    }

                    // Create a instance of the delegate to get access to the possible animations
                    var delegate = component.createObject(bubbleSettingsPage, {"visible":false});
                    var possibleAnimations = delegate.possibleAnimations;

                    // Add the buttons
                    for(var i=0; i < possibleAnimations.length; ++i) {
                        buttonComponent.createObject(animationRow, {"text":possibleAnimations[i].name});
                    }

                    // We're finished now
                    initialized = true;

                    // Choose the default selection
                    checkedButton = children[appSettings.bubbleAnimation];

                    delegate.destroy();
                    component.destroy();
                }

                onCheckedButtonChanged: {
                    // WTF?
                    if ( initialized == false )
                        return;

                    var index = -1;

                    for(var i=0; i < children.length; ++i) {
                        if ( children[i] == checkedButton ) {
                            index = i;
                            break;
                        }
                    }

                    console.log("Changing to animation", index);
                    selectedAnimation = index;

                    // Update appSettings, too
                    appSettings.bubbleAnimation = selectedAnimation;

                    conversationModel.update();
                }
            }

            SectionDelegate {
                width: listView.width
                //% "Preview"
                sectionName: qsTrId("qtn_web2sms_bubblesettings_section_preview")
            }

            ListView {
                id: bubbleView
                width: listView.width
                interactive: false
                rotation: 180
                spacing: 10

                height: contentHeight

                Component.onCompleted: {
                    ///bubbleView.height = bubbleView.contentHeight;
                    //console.log("Changed height to", bubbleView.contentHeight);
                    //bubbleView.height = 420;
                }

                model: ListModel {
                    id: conversationModel

                    function update() {
                        conversationModel.clear();
                        conversationModel.insert(0, {
                                                     "direction":"Inbound",
                                                     "text":"Hello Dummy!",
                                                     "status":ConversationModel.DeliveredStatus,
                                                     "endTime":"30.11.2011 | 12:10"
                                                 });
                        conversationModel.insert(0, {
                                                     "direction":"Outbound",
                                                     "text":"What's up, mate?",
                                                     "status":ConversationModel.DeliveredStatus,
                                                     "endTime":"30.11.2011 | 12:13"
                                                 });
                        conversationModel.insert(0, {
                                                     "direction":"Inbound",
                                                     "text":"Nothing so far ...",
                                                     "status":ConversationModel.DeliveredStatus,
                                                     "endTime":"30.11.2011 | 12:15"
                                                 });
                        conversationModel.insert(0, {
                                                     "direction":"Outbound",
                                                     "text":"Want to go to the cinema this evening? There are some really cool films!",
                                                     "status":ConversationModel.DeliveredStatus,
                                                     "endTime":"30.11.2011 | 12:16"
                                                 });
                        conversationModel.insert(0, {
                                                     "direction":"Inbound",
                                                     "text":"Sure, why not :-)",
                                                     "status":ConversationModel.DeliveredStatus,
                                                     "endTime":"30.11.2011 | 12:18"
                                                 });

                        //bubbleView.height = bubbleView.contentHeight;
                    }
                }

                delegate: BubbleItemDelegate {
                    id: bubbleItemDelegate
                    rotation: 180

                    animation: bubbleSettingsPage.status == PageStatus.Active ? animationRow.selectedAnimation : -1

                    fontFactor: fontFactorSlider.value
                    inboundBubbleColor: inboundSlider.value
                    outboundBubbleColor: outboundSlider.value
                }
            }
        }

        ScrollDecorator {
            flickableItem: parent
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-back"
            onClicked: pageStack.pop();
        }
    }
}
