import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Sheet {
    id: contactChooser

    /// The selected contact. Valid after accepted()
    property variant contact: null

    /////////// END OF PUBLIC API /////////////

    //% "Done"
    acceptButtonText: qsTrId("qtn_web2sms_sheet_done")
    //% "Cancel"
    rejectButtonText: qsTrId("qtn_web2sms_sheet_cancel")

    property QtObject acceptButton: contactChooser.buttons[1]

    function updateAcceptButton() {
        acceptButton.enabled = (listView.currentIndex !== -1);
    }

    // Prepare initial settings
    Component.onCompleted: {
        listView.currentIndex = -1;
        updateAcceptButton();
    }

    content: Item {
        anchors.fill: parent

        ListView {
            id: listView

            anchors {
                fill: parent
                margins: UIConstants.DEFAULT_MARGIN
            }

            // Allow selections
            focus: true

            section {
                criteria: ViewSection.FirstCharacter
                property: "displayLabel"
                delegate: SectionDelegate {
                    width: parent.width
                    sectionName: section // contact.displayLabel[0]
                }
            }

            delegate: AccountDelegate {
                titleText: model.displayLabel

                AvatarImage {
                    anchors.verticalCenter: parent.verticalCenter
                    contactId: contact.contactId
                }

                onClicked: {
                    // Set the current contact
                    contactChooser.contact = model.contact;
                    // Adjust the current index
                    ListView.view.currentIndex = model.index;
                    // And enable the accept button
                    contactChooser.updateAcceptButton();
                }
            }

            model: ContactModel {
                id: contactModel
            }
        }

        /// This must be outside of the listview, otherwise it looks strange with the borders
        FastScroll {
            listView: listView
        }
    }
}
