import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/Utils.js" as Utils
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

Item {
    id: container

    property alias placeholderText: prompt.text
    property alias activeFocus: textEdit.activeFocus
    property alias contacts: formatter.contacts

    function forceActiveFocus() {
        textEdit.forceActiveFocus();
    }

    function tryAddContact(contact) {
        var contactId = contact.contactId;
        console.log("Showing contact", contactId);
        console.log(contact.displayLabel, "has", contact.phoneNumbers.length, "numbers");

        // Accept the suggested contact
        if ( contact.phoneNumbers.length === 1 ) {
            formatter.acceptSuggestedContact(contact.displayLabel, contact.phoneNumbers[0].number);
            textEdit.forceActiveFocus();
        } else {
            // Clear the number model
            numberModel.clear();

            // Insert our numbers since we can't assign a declarative list property as model
            for(var i=0; i < contact.phoneNumbers.length; ++i) {
                var row = {
                    "name":contact.phoneNumbers[i].number
                };

                console.log("Appending row", row);

                numberModel.append(row);
            }

            numberChooser.contact = contact;
            numberChooser.titleText = contact.displayLabel;
            numberChooser.selectedIndex = -1;
            numberChooser.open();
        }
    }

    function chooseContact(contacts) {
        console.log("Showing contact selection dialog with", contacts.length, "items");

        // Clear the contacts model
        contactsModel.clear();

        // Insert our contacts since we can't assign a declarative list property as model
        for(var i=0; i < contacts.length; ++i) {
            var row = {
                "name":contacts[i].displayLabel,
                "contact":contacts[i]
            };

            contactsModel.append(row);
        }

        contactChooser.selectedIndex = -1;
        contactChooser.open();
    }

    ///// END OF PULIC API /////

    width: 300
    height: textEdit.height

    property RecipientBubble recipientBubble: RecipientBubble {
        property bool implicitShow: false

        anchors {
            left: parent.left
            leftMargin: 16
        }

        parent: Utils.findRootItem(container)
        visible: implicitShow //&& textEdit.activeFocus || activeFocus

        y: 120 + textEdit.height

        onContactClicked: {
            tryAddContact(contact);

            // Hide ourself
            implicitShow = false;
        }

        onBubbleClicked: {
            chooseContact(contacts);

            implicitShow = false;
        }
    }

    SelectionDialog {
        id: numberChooser

        property variant contact

        model: ListModel {
            id: numberModel
        }

        onAccepted: {
            var contact = numberChooser.contact;
            console.log(contact.displayLabel, ": chosen number", selectedIndex);
            formatter.acceptSuggestedContact(contact.displayLabel, contact.phoneNumbers[selectedIndex].number);

            textEdit.forceActiveFocus();
        }
    }

    SelectionDialog {
        id: contactChooser

        //% "Choose contact"
        titleText: qsTrId("qtn_web2sms_contacttextedit_choose_contact");

        model: ListModel {
            id: contactsModel
        }

        onAccepted: {
            var contact = finder.contacts[selectedIndex];

            console.log("ContactChooser: selected", contact.displayLabel);

            tryAddContact(contact);
        }
    }

    Component.onDestruction: {
        recipientBubble.destroy();
    }

    PlainTextArea {
        id: textEdit
        inputMethodHints: Qt.ImhNoPredictiveText

        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
        }

        textFormat: TextEdit.RichText

        onTextChanged: {
            formatter.setInputData(textEdit.text, textEdit.cursorPosition)
        }

        Component.onCompleted: {
            if ( theme.inverted ) {
                style.textColor = UIConstants.COLOR_INVERTED_FOREGROUND;
            }
        }

        Text {
            id: prompt

            anchors.fill: parent
            anchors.leftMargin: UIConstants.PADDING_XLARGE
            anchors.rightMargin: UIConstants.PADDING_XLARGE
            anchors.topMargin: (UIConstants.FIELD_DEFAULT_HEIGHT - font.pixelSize) / 2
            anchors.bottomMargin: (UIConstants.FIELD_DEFAULT_HEIGHT - font.pixelSize) / 2

            // memory allocation optimization: cursorPosition is checked to minimize displayText evaluations
            visible: formatter.isEmpty
            font: textEdit.platformStyle.textFont
            color: textEdit.platformStyle.promptTextColor
            elide: Text.ElideRight
        }
    }

    Banner {
        id: banner
    }

    ContactTextEditFormatter {
        id: formatter
        defaultFont: textEdit.platformStyle.textFont

        property bool blockSignals: false

        onContactNameChanged: finder.findContactsByName(contactName)
        onAdjustTextEdit: {
            textEdit.text = text;
            textEdit.cursorPosition = position;
        }
        onError: banner.showBanner(errorMessage)
    }

    ContactModel {
        id: finder

        function findContactsByName(name) {
            // Prevent a search that matches everyone
            if ( name.length === 0 )
                return;

            finder.nameFilter = name;
            finder.update();
        }

        autoUpdate: false

        /*onRowsInserted*/
        onModelReset: {
            console.log("Found", finder.contacts.length, "contacts");

            var count = contacts.length;
            if ( count > 0 ) {
                recipientBubble.contact = contacts[0];
                recipientBubble.contacts = contacts;
                recipientBubble.name = contacts[0].displayLabel;
                recipientBubble.count = count;
                recipientBubble.implicitShow = true;
            }
            else
                recipientBubble.implicitShow = false;
        }
    }
}
