import QtQuick 1.1
import QtMultimediaKit 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import Web2SMS 1.0

ListView {
    id: conversationView

    property string remoteUid
    property int groupId

    // The message text is currently only required to detect on qml-side whether this is a unicode message or not
    signal resendMessage(string messageToken, string messageText)

    function forwardMessage(text) {
        console.log("Forwarding message");

        var component = Qt.createComponent("NewMessagePage.qml");
        if ( component.status != Component.Ready ) {
            console.log("Error forwarding message:", component.errorString());
            return;
        }

        var properties = {
            "text":text,
            "isForwarded":true
        };

        pageStack.push(component, properties);
    }

    function deleteMessage(eventId) {
        console.log("Deleting event ", eventId);
        conversationView.model.deleteEvent(eventId);
    }

    function copyToClipboard(text) {
        clipboard.text = text;
        console.log("Copied '" + clipboard.text + "' to clipboard");

        // The user wants visual feedback
        //% "Copied your text to the clipboard"
        banner.showBanner( qsTrId("qtn_web2sms_copied_clipboard") );
    }

    spacing: 10
    cacheBuffer: 100

    rotation: 180

    delegate: BubbleItemDelegate {
        rotation: 180

        outboundBubbleColor: appSettings.outboundBubbleColor
        inboundBubbleColor: appSettings.inboundBubbleColor
        animation: appSettings.bubbleAnimation
        fontFactor: appSettings.bubbleFontFactor

        MouseArea {
            anchors.fill: parent.bubbleItem
            onPressAndHold: {
                // Copy menu-relevant data
                contextMenu.messageText = model.text
                contextMenu.messageToken = model.messageToken;
                contextMenu.eventId = model.eventId;
                contextMenu.eventStatus = model.status;
                // And open the menu
                contextMenu.open()
            }
        }
    }

    model: ConversationModel {
        id: conversationModel

        observedConversation: conversationView.remoteUid
        groupId: conversationView.groupId

        onIncomingMessage: {
            if ( conversationView.remoteUid.length > 0 ) {
                console.log("Playing message beep for incoming message");
                incomingMessageBeep.play();
            } else {
                console.log("Incoming message, inactive window. No beep.");
            }
        }
    }

    SoundEffect {
         id: incomingMessageBeep
         source: "/usr/share/sounds/ui-tones/snd_message_fg.wav"
     }

    ScrollDecorator {
        id: scroller
        flickableItem: parent

        // Rotate the scroller around the y-axis that it looks normal to the user
        transform: Rotation {
            axis {
                x: 0
                y: 1
                z: 0
            }

            origin.x: scroller.width / 2
            angle: 180
        }
    }

    Label {
        anchors.centerIn: parent
        text: {
            if ( conversationModel.status == ConversationModel.Ready ) {
                //% "Empty conversation"
                return qsTrId("qtn_web2sms_conversationview_empty");
            } else {
                //% "Error loading conversation"
                return qsTrId("qtn_web2sms_conversationview_error");
            }
        }

        // Show red color on error
        color: conversationModel.status == ConversationModel.Error ? "red" : UIConstants.COLOR_SECONDARY_FOREGROUND
        visible: conversationModel.status == ConversationModel.Error || (conversationModel.status == ConversationModel.Ready && conversationView.count == 0)
        rotation: 180
    }

    BusyIndicator {
        anchors.centerIn: parent
        visible: (conversationModel.status == ConversationModel.Loading)
        running: visible

        Component.onCompleted: {
            platformStyle.size = "large";
        }
    }

    Banner {
        id: banner
    }

    Item {
        id: contextMenu

        property string messageText
        property string messageToken
        property int eventId
        property int eventStatus: -1

        function open() {
            var theMenu = (eventStatus != ConversationModel.PermanentlyFailedStatus) ? firstMenu : secondMenu;
            theMenu.open();
        }

        ContextMenu {
            id: firstMenu
            parent: conversationView

            MenuLayout {
                MenuItem {
                    //% "Forward"
                    text: qsTrId("qtn_web2sms_conversationview_forward")
                    onClicked: forwardMessage(contextMenu.messageText)
                }

                MenuItem {
                    //% "Delete"
                    text: qsTrId("qtn_web2sms_conversationview_delete")
                    onClicked: deleteMessage(contextMenu.eventId)
                }

                MenuItem {
                    //% "Copy"
                    text: qsTrId("qtn_web2sms_conversationview_copy")
                    onClicked: copyToClipboard(contextMenu.messageText)
                }
            }
        }

        ContextMenu {
            id: secondMenu
            parent: conversationView

            MenuLayout {
                MenuItem {
                    //% "Resend"
                    text: qsTrId("qtn_web2sms_conversationview_resend")
                    onClicked: resendMessage(contextMenu.messageToken, contextMenu.messageText)
                }

                MenuItem {
                    //% "Forward"
                    text: qsTrId("qtn_web2sms_conversationview_forward")
                    onClicked: forwardMessage(contextMenu.messageText)
                }

                MenuItem {
                    //% "Delete"
                    text: qsTrId("qtn_web2sms_conversationview_delete")
                    onClicked: deleteMessage(contextMenu.eventId)
                }

                MenuItem {
                    //% "Copy"
                    text: qsTrId("qtn_web2sms_conversationview_copy")
                    onClicked: copyToClipboard(contextMenu.messageText)
                }
            }
        }
    }
}
