import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/extras/constants.js" as UI
import Web2SMS 1.0

Item {
    id: listItem

    property string name
    property string messageText
    property string time
    property int contactId

    signal clicked
    property alias pressed: mouseArea.pressed

    property color titleColor: theme.inverted ? UI.LIST_TITLE_COLOR_INVERTED : UI.LIST_TITLE_COLOR
    property color subtitleColor: theme.inverted ? UI.LIST_SUBTITLE_COLOR_INVERTED : UI.LIST_SUBTITLE_COLOR

    height: contentItem.height // UI.LIST_ITEM_HEIGHT
    width: ListView.view.width

    BorderImage {
        anchors.fill: parent
        // Fill page porders
        anchors.leftMargin: -UI.MARGIN_XLARGE
        anchors.rightMargin: -UI.MARGIN_XLARGE
        visible: !mouseArea.pressed && model.unreadMessages !== 0
        border {
            left: 16
            top: 16
            right: 16
            bottom: 16
        }

        source: theme.inverted ? "image://theme/meegotouch-unread-inbox-panel-inverted-background" :
                                 "image://theme/meegotouch-unread-inbox-panel-background"
    }

    BorderImage {
        id: background
        anchors.fill: parent
        // Fill page porders
        anchors.leftMargin: -UI.MARGIN_XLARGE
        anchors.rightMargin: -UI.MARGIN_XLARGE
        visible: mouseArea.pressed
        border {
            left: 16
            top: 16
            right: 16
            bottom: 16
        }

        source: {
            if ( model.unreadMessages === 0 ) {
                return theme.inverted ? "image://theme/meegotouch-list-inverted-background-pressed-center" :
                                        "image://theme/meegotouch-list-background-pressed-center";
            } else {
                return theme.inverted ? "image://theme/meegotouch-unread-inbox-panel-inverted-background-selected" :
                                        "image://theme/meegotouch-unread-inbox-panel-background-selected"
            }
        }
    }

    TextFieldStyle {
        id: textFieldStyle
    }

    Item {
        id: contentItem

        anchors {
            left: parent.left
            leftMargin: 8
            right: parent.right
        }
        height: column.height + 32

        Column {
            id: column

            anchors {
                left: parent.left
                right: avatarImage.left
                rightMargin: 8
                verticalCenter: parent.verticalCenter
            }

            spacing: 8

            Label {
                id: mainText
                text: listItem.name
                //text: model.title
                font.weight: Font.Bold
                font.pixelSize: UI.LIST_TILE_SIZE
                color: listItem.titleColor
                width: parent.width
                elide: Text.ElideRight
            }

            Label {
                id: subText
                text: listItem.messageText
                //text: model.subtitle
                font.weight: Font.Light
                font.pixelSize: UI.LIST_SUBTILE_SIZE
                color: listItem.titleColor
                elide: Text.ElideRight
                maximumLineCount: 1
                width: parent.width
                font.family: textFieldStyle.textFont
            }

            Label {
                id: timestampText
                text: {
                    if ( model.lastEventStatus == ConversationModel.TemporarilyFailedStatus ||
                         model.lastEventStatus == ConversationModel.PermanentlyFailedStatus )
                        //% "Failed"
                        return qsTrId("qtn_web2sms_status_permanently_failed");
                    else
                        return listItem.time
                }
                font.weight: Font.Light
                font.pixelSize: UI.LIST_SUBTILE_SIZE - 5
                //color: listItem.subtitleColor
                color: {
                    if ( model.lastEventStatus == ConversationModel.TemporarilyFailedStatus ||
                         model.lastEventStatus == ConversationModel.PermanentlyFailedStatus )
                        return "red";
                    else if ( model.unreadMessages > 0 && !mouseArea.pressed )
                        return "#1080dd"; // Blue
                    else
                        return listItem.subtitleColor;
                }
                font.family: textFieldStyle.textFont
            }
        }

        AvatarImage {
            id: avatarImage
            anchors {
                right: parent.right
                rightMargin: 8
                verticalCenter: parent.verticalCenter
            }
            contactId: listItem.contactId
            //contactId: model.contactId
        }
    }
    MouseArea {
        id: mouseArea;
        anchors.fill: parent
        onClicked: {
            listItem.clicked();
        }
    }
}
