import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import "/usr/lib/qt4/imports/com/nokia/extras/constants.js" as UI
import Web2SMS 1.0

Page {
    id: groupPage

    //orientationLock: PageOrientation.LockPortrait
    anchors.margins: UIConstants.DEFAULT_MARGIN

    AccountsModel {
        id: accountsModel
        providerFilter: "web2sms"
        onCountChanged: updateToolBar()
    }

    // We have to use this method, otherwise it occurrs that the toolbar is invisible
    // after account creation.
    onStatusChanged: updateToolBar()

    EmptyPage {
        id: emptyPage
        anchors.fill: parent
        anchors.margins: -UIConstants.DEFAULT_MARGIN
        visible: (accountsModel.count == 0)
        z: 100 // This is above all layers!
    }

    function updateToolBar() {
        if ( status == PageStatus.Activating || status == PageStatus.Active )
            applicationWindow.showToolBar = (accountsModel.count > 0);
        else if ( status == PageStatus.Inactive )
            applicationWindow.showToolBar = true;
    }

    property ConversationPage conversationPage: ConversationPage {
    }

    function openConversationWithRemoteUid(remoteUid, push) {
        // Search for the row of the remote uid
        var row = groupModel.rowForRemoteUid(remoteUid);
        if ( row == -1 ) {
            console.log("Unable to find the row for remoteUid", remoteUid);
            return null;
        }

        // Find the details we need
        var contactName = groupModel.data(row, GroupModel.ContactNameRole);
        var contactId = groupModel.data(row, GroupModel.ContactIdRole);
        var groupId = groupModel.data(row, GroupModel.GroupIdRole);

        // Really open the conversation
        return openConversation(contactName, remoteUid, contactId, groupId, push);
    }

    function openConversation(contactName, remoteUid, contactId, groupId, push) {
        console.log("openConversation:",groupId);

        // Assign new values
        conversationPage.contactName = contactName;
        conversationPage.remoteUid = remoteUid;
        conversationPage.contactId = contactId;
        conversationPage.groupId = groupId;

        if ( push == undefined )
            pageStack.push(conversationPage);

        return conversationPage;
    }

    function newMessage() {
        var component = Qt.createComponent("NewMessagePage.qml");

        if ( component.status != Component.Ready ) {
            console.log("Error creating new message page:", component.errorString());
            return;
        }

        pageStack.push(component);
    }

    function openAccountConfiguration() {
        var component = Qt.createComponent("/opt/web2sms/accounts/StandalonePage.qml");

        if ( component.status != Component.Ready ) {
            console.log("Error creating component:", component.errorString());
            return;
        }

        pageStack.push(component);
    }

    function openSettings() {
        pageStack.push(Qt.resolvedUrl("SettingsPage.qml"));
    }

    Connections {
        target: appInterface
        onConversationRequested: {
            var row = groupModel.rowForRemoteUid(contactId);
            if ( row == -1 ) {
                console.log("Can't find the event in my list");
                return;
            }

            var contactName = groupModel.data(row, GroupModel.ContactNameRole);
            var cid = groupModel.data(row, GroupModel.ContactIdRole);
            var groupId = groupModel.data(row, GroupModel.GroupIdRole);

            console.log("contactName:", contactName, "remoteUid:", contactId, "cid:", cid, "groupId:", groupId);

            openConversation(contactName, contactId, cid, groupId);

            // Bring our window to front
            backend.raiseWindow();
        }
    }

    function openAboutPage() {
        var component = Qt.createComponent("AboutPage.qml");
        pageStack.push(component);
    }

    function openProviderUpdatePage() {
        var component = Qt.createComponent("ProviderUpdatePage.qml");
        pageStack.push(component);
    }

    Image {
        id: header

        anchors {
            left: parent.left
            right: parent.right
            top: parent.top
            margins: -UIConstants.DEFAULT_MARGIN
        }

        z: 1
        height: 72

        source: theme.inverted ? "image://theme/meegotouch-view-header-inverted" :
                                 "image://theme/color5-meegotouch-view-header-fixed"

        Label {
            anchors.verticalCenter: parent.verticalCenter
            x: 20
            id: titleLabel
            text: "Web2SMS"
            color: "white"
            font.pixelSize: 30
        }

        ProviderUpdateButton {
            anchors {
                verticalCenter: parent.verticalCenter
                right: parent.right
                rightMargin: 8
            }

            onClicked: openProviderUpdatePage()
        }
    }

    ListView {
        id: listView

        anchors {
            left: parent.left
            right: parent.right
            top: header.bottom
            topMargin: UIConstants.DEFAULT_MARGIN
            bottom: parent.bottom
        }

        spacing: UI.LIST_ITEM_SPACING / 2
        cacheBuffer: 100

        section.criteria: ViewSection.FullString
        section.property: "section"
        section.delegate: SectionDelegate {
            anchors {
                left: parent.left
                leftMargin: -UIConstants.DEFAULT_MARGIN
                right: parent.right
            }

            sectionName: section
        }

        delegate: GroupDelegate {
            // Properties
            name: model.contactName
            messageText: model.lastMessageText
            time: model.endTime
            contactId: model.contactId
            // Events
            onClicked: openConversation(model.contactName, model.remoteUid, model.contactId, model.groupId)
        }

        model: GroupModel {
            id: groupModel
        }

        Label {
            anchors.centerIn: parent
            //% "No conversations"
            text: qsTrId("qtn_web2sms_grouppage_empty")
            color: UIConstants.COLOR_SECONDARY_FOREGROUND
            visible: (parent.count == 0)
        }

        FastScroll {
            listView: parent
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-new-message"
            onClicked: newMessage()
        }

        ToolIcon {
            iconId: "toolbar-view-menu"
            onClicked: (menu.status == DialogStatus.Closed) ? menu.open() : menu.close()
        }
    }

    Menu {
        id: menu

        MenuLayout {
            MenuItem {
                //% "Configure accounts"
                text: qsTrId("qtn_web2sms_configure_accounts")
                onClicked: openAccountConfiguration()
            }

            MenuItem {
                //% "Settings"
                text: qsTrId("qtn_web2sms_settings_title")
                onClicked: openSettings()
            }

            MenuItem {
                //% "About"
                text: qsTrId("qtn_web2sms_about_title")
                onClicked: openAboutPage()
            }
        }
    }
}
