import QtQuick 1.1
import com.nokia.meego 1.0
//import com.nokia.extras 1.0
import Web2SMS 1.0

TopToolbar {
    id: panel

    property Account account: null
    property QtObject __sheet: null

    function showProviderSelection() {
        var component = Qt.createComponent("ProviderSelectionSheet.qml");

        if ( component.status != Component.Ready ) {
            console.log("Failed to create component:", component.errorString());
            return;
        }

        var properties = {
            "selectedAccountId":appSettings.activeAccountId
        };

        __sheet = component.createObject(panel, properties);
        __sheet.accepted.connect(panel.didSelectAccount);
        __sheet.open();
    }

    function didSelectAccount() {
        console.log("didSelectAccount:", __sheet.selectedAccountId);
        appSettings.activeAccountId = __sheet.selectedAccountId;
    }

    Component.onCompleted: {
        // Load the last settings from AppSettings
        account = backend.accountFromId(appSettings.activeAccountId);
        console.log("Initialized with", account);
    }

    Connections {
        target: appSettings
        onActiveAccountIdChanged: {
            console.log("onActiveAccountIdChanged");
            account = backend.accountFromId(appSettings.activeAccountId);
            console.log("Account changed to", account);
        }
    }

    width: 200
    height: 60

    // For beauty reasons we rotate the background image for 180° and then rotate the content 180° back
    rotation: 180

    Item {
        anchors.fill: parent
        rotation: 180

        MouseArea {
            anchors.fill: parent
            onClicked: showProviderSelection()
        }

        Label {
            anchors {
                verticalCenter: parent.verticalCenter
                left: parent.left
                leftMargin: 8
                right: moreIndicator.left
                rightMargin: 8
            }

            elide: Text.ElideLeft

            text: {
                if ( account ) {
                    // Find the provider
                    var provider = backend.plugins.providerById(account.providerId);

                    if ( account.balance.length ) {
                        // Format the balance
                        var formattedBalance = qsTrId(provider.balanceFormat).arg(account.balance);

                        return qsTr("%1 | %2").arg(provider.name).arg(formattedBalance);
                    } else
                        return provider.name;
                } else {
                    //% "You have to select an account"
                    return qsTrId("qtn_web2sms_providerinfopanel_no_account");
                }
            }
        }

        Image {
            id: moreIndicator

            anchors {
                verticalCenter: parent.verticalCenter
                right: parent.right
            }

            source: theme.inverted ? "image://theme/icon-m-common-drilldown-arrow-inverse" : // WHY is this thing called inverse?
                                     "image://theme/icon-m-common-drilldown-arrow"
        }
    }
}
