import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import "/usr/lib/qt4/imports/com/nokia/extras/constants.js" as UI
import Web2SMS 1.0

Page {
    id: providerUpdatePage

    orientationLock: PageOrientation.LockPortrait
    anchors.margins: UIConstants.DEFAULT_MARGIN

    ListView {
        id: listView

        anchors.fill: parent
        spacing: UI.LIST_ITEM_SPACING

        model: VisualItemModel {
            Title {
                width: listView.width

                //% "Provider updates"
                title: qsTrId("qtn_web2sms_providerupdatepage_title")
                //% "New providers can be added and fixed without a complete update of the application. The updates are small and consume about 50 kb."
                subtitle: qsTrId("qtn_web2sms_providerupdatepage_subtitle")
                imageSource: ""
            }

            SwitchWithText {
                id: dailyUpdateCheckBox

                width: listView.width
                //% "Daily check for updates"
                text: qsTrId("qtn_web2sms_providerupdatepage_daily_check")
                checked: appSettings.dailyUpdateCheck

                Binding {
                    target: appSettings
                    property: "dailyUpdateCheck"
                    value: dailyUpdateCheckBox.checked
                }
            }

            SwitchWithText {
                id: autoinstallUpdateCheckBox

                width: listView.width
                height: dailyUpdateCheckBox.checked ? children[0].height : 0
                opacity: dailyUpdateCheckBox.checked ? 1.0 : 0.0
                //% "Automatically install updates"
                text: qsTrId("qtn_web2sms_providerupdatepage_auto_install")
                checked: appSettings.autoInstallUpdate

                Binding {
                    target: appSettings
                    property: "autoInstallUpdate"
                    value: autoinstallUpdateCheckBox.checked
                }

                Behavior on opacity {
                    NumberAnimation { duration: 200 }
                }

                Behavior on height {
                    NumberAnimation { duration: 150; easing.type: Easing.InOutBack }
                }
            }

            Separator {
                width: listView.width
            }

            Label {
                width: listView.width
                //% "Last checked: %1"
                text: qsTrId("qtn_web2sms_providerupdatepage_last_update").arg(appSettings.lastUpdateCheckString)
            }

            Label {
                id: updateCheckErrorLabel
                width: listView.width
                text: updateChecker.errorString
                color: "red" // Yeah, great hardcoded stuff
                wrapMode: Text.Wrap

                onHeightChanged: console.log("height changed", height)
                opacity: 0.0
                height: 0

                states: State {
                    when: updateChecker.state == ProviderUpdateChecker.Error
                    PropertyChanges {
                        target: updateCheckErrorLabel
                        height: undefined
                        opacity: 1.0
                    }
                }

                transitions: Transition {
                    ParallelAnimation {
                        NumberAnimation { property: "opacity"; duration: 200; }
                        NumberAnimation { property: "height"; duration: 150; easing.type: Easing.InOutBack }
                    }
                }
            }

            Item {
                width: listView.width
                height: children[1].height

                BusyIndicator {
                    anchors {
                        horizontalCenter: parent.horizontalCenter
                        left: parent.left
                        leftMargin: UIConstants.DEFAULT_MARGIN
                    }

                    running: visible
                    visible: updateChecker.state == ProviderUpdateChecker.Checking
                }

                Button {
                    anchors.horizontalCenter: parent.horizontalCenter

                    text: {
                        if ( updateChecker.state == ProviderUpdateChecker.Checking ) {
                            //% "Checking ..."
                            return qsTrId("qtn_web2sms_providerupdatepage_checking");
                        } else {
                            //% "Check now"
                            return qsTrId("qtn_web2sms_providerupdatepage_checkbutton");
                        }
                    }
                    enabled: updateChecker.state != ProviderUpdateChecker.Checking

                    onClicked: {
                        updateChecker.checkForUpdate()

                        // Update the time
                        appSettings.lastUpdateCheck = new Date();
                    }
                }
            }

            Column {
                id: updateColumn
                width: listView.width
                spacing: UI.LIST_ITEM_SPACING
                opacity: (updateChecker.state == ProviderUpdateChecker.UpdateAvailable) ? 1.0 : 0.0
                height: (updateChecker.state == ProviderUpdateChecker.UpdateAvailable) ? childrenRect.height : 0

                Behavior on opacity {
                    NumberAnimation { duration: 200 }
                }

                Behavior on height {
                    NumberAnimation { duration: 150; easing.type: Easing.InOutBack }
                }

                Separator {
                    width: parent.width
                }

                Label {
                    text: {
                        if ( updateInstaller.state == ProviderUpdateInstaller.Done )
                            //% "Update installed"
                            return qsTrId("qtn_web2sms_providerupdatepage_done");
                        else
                            //% "Update available!"
                            return qsTrId("qtn_web2sms_providerupdatepage_update_available");
                    }
                }

                Button {
                    anchors.horizontalCenter: parent.horizontalCenter
                    //% "Install"
                    text: qsTrId("qtn_web2sms_providerupdatepage_download")
                    onClicked: updateInstaller.installUpdate()
                }

                Item {
                    width: 1
                    height: 30
                }

                ProgressBar {
                    width: parent.width
                    indeterminate: (value == 0.0)
                    value: updateInstaller.progress
                    visible: (updateInstaller.state == ProviderUpdateInstaller.Installing)
                }

                Label {
                    id: updateInstallErrorLabel
                    width: listView.width
                    text: updateInstaller.errorString
                    color: "red" // Yeah, great hardcoded stuff
                    wrapMode: Text.Wrap

                    onHeightChanged: console.log("height changed", height)
                    opacity: 0.0
                    height: 0

                    states: State {
                        when: updateInstaller.state == ProviderUpdateInstaller.Error
                        PropertyChanges {
                            target: updateInstallErrorLabel
                            height: undefined
                            opacity: 1.0
                        }
                    }

                    transitions: Transition {
                        ParallelAnimation {
                            NumberAnimation { property: "opacity"; duration: 200; }
                            NumberAnimation { property: "height"; duration: 150; easing.type: Easing.InOutBack }
                        }
                    }
                }
            }
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            id: backIcon
            iconId: 'toolbar-back'
            onClicked: pageStack.pop()
        }
    }
}
