import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0
import "/usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UIConstants
import "/usr/lib/qt4/imports/com/nokia/extras/constants.js" as UI
import Web2SMS 1.0

Page {
    id: settingsPage

    orientationLock: PageOrientation.LockPortrait
    anchors.margins: UIConstants.DEFAULT_MARGIN

    function showBubbleSettings() {
        var component = Qt.createComponent("BubbleSettingsPage.qml");
        if ( component.status != Component.Ready ) {
            console.log("Error loading component:", component.errorString());
            return;
        }

        pageStack.push(component);
    }

    function showProviderUpdates() {
        var component = Qt.createComponent("ProviderUpdatePage.qml");
        if ( component.status != Component.Ready ) {
            console.log("Error loading component:", component.errorString());
            return;
        }

        pageStack.push(component);
    }

    ListView {
        id: listView
        anchors.fill: parent
        spacing: UI.LIST_ITEM_SPACING

        model: VisualItemModel {
            Title {
                width: listView.width

                //% "Settings"
                title: qsTrId("qtn_web2sms_settings_title")
                subtitle: ""
            }

            SwitchWithText {
                id: showBetaWarningCheckbox
                width: listView.width
                //% "Show Beta warning"
                text: qsTrId("qtn_web2sms_settings_show_betawarning")
                checked: appSettings.showBetaWarning

                Binding {
                    target: appSettings
                    property: "showBetaWarning"
                    value: showBetaWarningCheckbox.checked
                }
            }

            SwitchWithText {
                id: useForMessagingCheckbox
                width: listView.width
                //% "Open SMS notifications with Web2SMS"
                text: qsTrId("qtn_web2sms_settings_make_default_program")
                checked: appSettings.handleNotifications

                Binding {
                    target: appSettings
                    property: "handleNotifications"
                    value: useForMessagingCheckbox.checked
                }
            }

            SwitchWithText {
                id: nightModeCheckbox
                width: listView.width
                //% "Night mode"
                text: qsTrId("qtn_web2sms_settings_night_mode")
                checked: appSettings.nightMode

                Binding {
                    target: appSettings
                    property: "nightMode"
                    value: nightModeCheckbox.checked
                }
            }

            SwitchWithText {
                id: customServerCheckbox
                //% "Use custom message server"
                text: qsTrId("qtn_web2sms_settings_use_custom_server")
                checked: appSettings.useCustomServer
                width: listView.width

                Binding {
                    target: appSettings
                    property: "useCustomServer"
                    value: customServerCheckbox.checked
                }
            }

            Column {
                id: customServerColumn

                opacity: customServerCheckbox.checked ? 1.0 : 0.0
                width: listView.width
                height: customServerCheckbox.checked ? childrenRect.height : 0

                Behavior on opacity {
                    NumberAnimation { duration: 200 }
                }

                Behavior on height {
                    NumberAnimation { duration: 150; easing.type: Easing.InOutBack }
                }

                Label {
                    //% "Custom server address"
                    text: qsTrId("qtn_web2sms_settings_custom_server_address")
                }

                TextField {
                    id: customServerAddressField

                    anchors {
                        left: parent.left
                        right: parent.right
                    }

                    //% "http://someserver"
                    placeholderText: qsTrId("qtn_web2sms_settings_custom_server_address_placeholder")
                    text: appSettings.customServerAddress

                    Binding {
                        target: appSettings
                        property: "customServerAddress"
                        value: customServerAddressField.text
                    }
                }
            }

            Separator {
                width: listView.width
            }

            AccountDelegate {
                width: listView.width
                showImage: false
                showSelection: false
                //% "Bubbles"
                titleText: qsTrId("qtn_web2sms_settings_bubbles_title")
                //% "Change bubble colors and animation"
                subtitleText: qsTrId("qtn_web2sms_settings_bubbles_subtitle")
                onClicked: showBubbleSettings()


                MoreIndicator {
                    anchors {
                        right: parent.right
                        verticalCenter: parent.verticalCenter
                    }
                }
            }

            AccountDelegate {
                width: listView.width
                showImage: false
                showSelection: false
                titleText: qsTrId("qtn_web2sms_providerupdatepage_title")
                //% "Configure automatic updates"
                subtitleText: qsTrId("qtn_web2sms_providerupdatepage_shortdesc")
                onClicked: showProviderUpdates()

                MoreIndicator {
                    anchors {
                        right: parent.right
                        verticalCenter: parent.verticalCenter
                    }
                }
            }
        }

        ScrollDecorator {
            flickableItem: parent
        }
    }

    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-back"
            onClicked: pageStack.pop()
        }
    }
}
