#ifndef CONTACTTEXTEDITFORMATTER_H
#define CONTACTTEXTEDITFORMATTER_H

#include <QObject>
#include <QDeclarativeItem>
#include <QDeclarativeListProperty>

class Contact;
class ContactTextEditFormatterPrivate;

class ContactTextEditFormatter : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QFont defaultFont READ defaultFont WRITE setDefaultFont NOTIFY defaultFontChanged)
    Q_PROPERTY(QString outputText READ outputText NOTIFY outputTextChanged)
    Q_PROPERTY(QString contactName READ contactName NOTIFY contactNameChanged)
    Q_PROPERTY(QVariantList contacts READ contacts NOTIFY contactsChanged)
    Q_PROPERTY(bool isEmpty READ isEmpty NOTIFY isEmptyChanged)

public:
    explicit ContactTextEditFormatter(QObject *parent = 0);
    ~ContactTextEditFormatter();

    void setDefaultFont(const QFont& font);
    QFont defaultFont() const;

    QString outputText() const;
    QString contactName() const;

    Q_INVOKABLE void setInputData(const QString& inputText, int cursorPosition);
    Q_INVOKABLE void acceptSuggestedContact(const QString& name, const QString& phoneNumber);

    QVariantList contacts() const;

    bool isEmpty() const;

signals:
    void defaultFontChanged();
    void outputTextChanged();
    void contactNameChanged();
    void contactsChanged();
    void isEmptyChanged();

    void adjustTextEdit(const QString& text, int position);
    void error(const QString& errorMessage);

protected:
    friend class ContactTextEditFormatterPrivate;
    ContactTextEditFormatterPrivate* d;
};

#endif // CONTACTTEXTEDITFORMATTER_H
