#include "settings.h"
#ifdef MEEGO_EDITION_HARMATTAN
#include "notificationhandler.h"
#endif // MEEGO_EDITION_HARMATTAN

#ifndef Q_OS_ANDROID
#include "appinterface.h"
#include "appinterface_adaptor.h"
#endif // Q_OS_ANDROID

#include "contacttexteditformatter.h"
#include "providerupdatechecker.h"
#include "providerupdateinstaller.h"

// Global includes
#include <QApplication>
#ifndef Q_OS_ANDROID
#include <QDBusConnection>
#endif
#include <QtDeclarative>
#include <QLocale>
#ifdef MEEGO_EDITION_HARMATTAN
#include <QDeclarativeView>
#include <MDeclarativeCache>
#include <MLocale>
#endif // MEEGO_EDITION_HARMATTAN

Q_DECL_EXPORT int main(int argc, char **argv)
{
#ifdef MEEGO_EDITION_HARMATTAN
    // Get the application from cache
    QApplication* app = MDeclarativeCache::qApplication(argc, argv);

    // Load translations
    MLocale locale;
    locale.installTrCatalog("web2sms");

    MLocale::setDefault(locale);
#else // MEEGO_EDITION_HARMATTAN
    QApplication* app = new QApplication(argc, argv);

#ifdef Q_OS_ANDROID
    QString translationPath = "/l10n";
#else
    QString translationPath = "/usr/share/l10n/meegotouch";
#endif // Q_OS_ANDROID
    QString fileName = QString("web2sms_%1").arg( QLocale::system().name() );

    QTranslator* translator = new QTranslator(app);
    // Load the default translation and if it fails, try the english one
    if ( translator->load(fileName, translationPath) || translator->load("web2sms_en", translationPath) ) {
        qDebug() << "Installed translations";
        app->installTranslator(translator);
    }

#endif // MEEGO_EDITION_HARMATTAN

#ifndef Q_OS_ANDROID
    // Register us on DBus
    QDBusConnection connection = QDBusConnection::sessionBus();
    connection.registerService("net.muhkuh.web2sms");
    connection.registerService("net.muhkuh.web2sms.Messaging"); // This is needed for the NotificationHandler
#endif // Q_OS_ANDROID

    Settings settings;
    ProviderUpdateChecker updateChecker;
    ProviderUpdateInstaller updateInstaller;

#ifndef Q_OS_ANDROID
    AppInterface appInterface;
    MessagingIfAdaptor adaptor(&appInterface);
    connection.registerObject("/", &appInterface);
#endif // Q_OS_ANDROID
#ifdef MEEGO_EDITION_HARMATTAN
    NotificationHandler notificationHandler(&settings);
#endif // MEEGO_EDITION_HARMATTAN

    qmlRegisterUncreatableType<Settings>("Web2SMS", 1, 0, "Settings", "Settings is an internal type");
    qmlRegisterType<ContactTextEditFormatter>("Web2SMS", 1, 0, "ContactTextEditFormatter");
    qmlRegisterUncreatableType<ProviderUpdateChecker>("Web2SMS", 1, 0, "ProviderUpdateChecker", "ProviderUpdateChecker is an internal type");
    qmlRegisterUncreatableType<ProviderUpdateInstaller>("Web2SMS", 1, 0, "ProviderUpdateInstaller", "ProviderUpdateInstaller is an internal type");

    // Create the view
#ifdef MEEGO_EDITION_HARMATTAN
    QDeclarativeView* view = MDeclarativeCache::qDeclarativeView();
#else // MEEGO_EDITION_HARMATTAN
    QDeclarativeView* view = new QDeclarativeView;
#endif // MEEGO_EDITION_HARMATTAN
    view->rootContext()->setContextProperty("appSettings", &settings);
#ifndef Q_OS_ANDROID
    view->rootContext()->setContextProperty("appInterface", &appInterface);
#endif // Q_OS_ANDROID
    view->rootContext()->setContextProperty("appVersion", PACKAGEVERSION);
    view->rootContext()->setContextProperty("updateChecker", &updateChecker);
    view->rootContext()->setContextProperty("updateInstaller", &updateInstaller);

    // Update the network settings
    QNetworkAccessManager* networkAccessManager = view->rootContext()->engine()->networkAccessManager();
    updateChecker.setNetworkAccessManager(networkAccessManager);
    updateInstaller.setNetworkAccessManager(networkAccessManager);

#ifdef Q_OS_ANDROID
#define PATH ""
#else
#define PATH "/opt/web2sms/"
#endif // Q_OS_ANDROID

    // Adjust the input path list so that the application can find our qml plugin
    QDeclarativeEngine* engine = view->engine();
    QStringList importPathList = engine->importPathList();
    importPathList << PATH "/lib/imports" << PATH "/imports";
    engine->setImportPathList(importPathList);

#ifdef Q_OS_ANDROID
    engine->setBaseUrl( QUrl::fromLocalFile("/") );
    engine->addPluginPath(QApplication::applicationDirPath()+"/../lib");
#endif // Q_OS_ANDROID

    // Run the code
    view->setSource( QUrl::fromLocalFile(PATH "/messaging-ui/main.qml") );
#ifdef Q_OS_ANDROID
    view->show();
#else // Q_OS_ANDROID
    view->showFullScreen();
#endif // Q_OS_ANDROID

    // Run the event loop
    int ret = app->exec();

    // Clean up memory
#ifndef MEEGO_EDITION_HARMATTAN
    delete view; // Crashes on harmattan for some reason
#endif // MEEGO_EDITION_HARMATTAN
    delete app;

    return ret;
}
