import QtQuick 1.1
import QtMobility.systeminfo 1.2
import com.nokia.meego 1.0
import Web2SMS 1.0

PageStackWindow {
    id: applicationWindow

    function updateServerAddress() {
        if ( appSettings.useCustomServer == false )
            backend.messageHandler.serverAddress = "https://web2sms.olausson.de:1078";
        else
            backend.messageHandler.serverAddress = appSettings.customServerAddress;
    }

    function updateNightMode() {
        // Set theme inverted to the same value as the night mode
        if ( typeof(theme) != "undefined" )
            theme.inverted = appSettings.nightMode;
    }

    function updateUpdateTimer() {
        var now = new Date();
        var diff = Math.floor((now.getTime() - appSettings.lastUpdateCheck.getTime()) / 1000);
        if ( diff > 86400 || isNaN(diff) ) {
            if ( appSettings.dailyUpdateCheck ) {
                console.log("24h since the last check, checking now for updates");
                updateTimer.stop();
                connectionAwareUpdateChecker.checkForUpdate();
            }
        } else {
            // Can be 2 hours before or two hours after
            updateTimer.minimumInterval = 86400 - diff - 3600;
            updateTimer.maximumInterval = 86400 - diff + 3600;
            updateTimer.start();

            console.log("Update timer waits for about", 86400-diff, "seconds");
        }
    }

    Connections {
        target: appSettings
        onUseCustomServerChanged: updateServerAddress()
        onCustomServerAddressChanged: updateServerAddress()
        onNightModeChanged: updateNightMode()
        onDailyUpdateCheckChanged: updateUpdateTimer()
    }

    Connections {
        target: backend.messageHandler
        onError: banner.showBanner(text)
    }

    Connections {
        target: updateChecker
        onStateChanged: {
            // Only handle update available state
            if ( updateChecker.state != ProviderUpdateChecker.UpdateAvailable )
                return;

            // Install the update if the user has enabled automatic installing
            if ( appSettings.autoInstallUpdate ) {
                console.log("Found update, installing it");
                updateInstaller.installUpdate();
            }
        }
    }

    AlignedTimer {
        id: updateTimer
        onTimeout: {
            console.log("Update timer woke up");

            // TODO: Check online connection
            if ( appSettings.dailyUpdateCheck )
                connectionAwareUpdateChecker.checkForUpdate();
        }
    }

    Item {
        id: connectionAwareUpdateChecker

        property bool wantsUpdateCheck: false

        function checkForUpdate() {
            if ( backend.messageHandler.onlineStatus != MessageHandler.Online ) {
                console.log("Update check delayed until the phone has internet connectivity");
                wantsUpdateCheck = true;
            }
            else {
                console.log("Phone is already online, continueing with update check");
                appSettings.lastUpdateCheck = new Date();
                updateChecker.checkForUpdate();

                updateTimer.minimumInterval = 86400 - 3600;
                updateTimer.maximumInterval = 86400 + 3600;
                updateTimer.start();
            }
        }

        Connections {
            target: backend.messageHandler
            onOnlineStatusChanged: {
                if ( connectionAwareUpdateChecker.wantsUpdateCheck &&
                     backend.messageHandler.onlineStatus == MessageHandler.Online ) {
                    console.log("Internet connection established, checking for updates");
                    // Reset the update checking
                    appSettings.lastUpdateCheck = new Date();
                    connectionAwareUpdateChecker.wantsUpdateCheck = false;
                    updateChecker.checkForUpdate();

                    updateTimer.minimumInterval = 86400 - 3600;
                    updateTimer.maximumInterval = 86400 + 3600;
                    updateTimer.start();
                }
            }
        }
    }

    Banner {
        id: banner
    }

    // TODO: Check this as it may slow down startup
    Component.onCompleted: {
        updateNightMode();
        updateServerAddress();

        // Start the update check timer
        if ( appSettings.dailyUpdateCheck ) {
            updateUpdateTimer();
        }
    }

    // Watch orientation changes
    onOrientationChangeStarted: {
        showStatusBar = (screen.currentOrientation != Screen.Landscape);
    }

    initialPage: GroupPage {}
}
