#include "notificationhandler.h"

#include <QDBusConnection>
#include <QDebug>
#include <notificationsystem/notificationsinkadaptor.h>
#include <MRemoteAction>

NotificationHandler::NotificationHandler(Settings* settings, QObject *parent)
: NotificationSink(parent)
, settings(settings)
{
    qDBusRegisterMetaType<Notification>();
    qDBusRegisterMetaType<QList<Notification> >();
    qDBusRegisterMetaType<NotificationGroup>();
    qDBusRegisterMetaType<NotificationParameters>();
    qDBusRegisterMetaType<QList<NotificationGroup> >();

    new NotificationSinkAdaptor(this);

    QDBusConnection session = QDBusConnection::sessionBus();
    session.registerObject("/notificationsink", this);

    QDBusInterface manager("com.meego.core.MNotificationManager", "/notificationsinkmanager", "com.meego.core.MNotificationManager");
    QDBusMessage msg = manager.call("registerSink", "net.muhkuh.web2sms", "/notificationsink");

    qDebug() << "RegisterSink:" << msg.errorMessage();
}

void NotificationHandler::addNotification(const Notification &notification)
{
    // Get the notification parameters
    /*const NotificationParameters& parameters = notification.parameters();

    // Check if it's an sms
    QString eventType = parameters.value("eventType").toString();
    if ( eventType != "x-nokia.messaging.sms" )
        return;

    qDebug() << "Received SMS notification, checking remote action.";

    // Check the remote action
    QString action = parameters.value("action").toString();
    if ( action.startsWith("com.nokia.Messaging") == false ) {
        qDebug() << "Notification already changed. action=" << action;
        return;
    }

    MRemoteAction theAction(action);

    QString newAction = "net.muhkuh.web2sms";

    QDBusInterface manager("com.meego.core.MNotificationManager", "/notificationmanager", "com.meego.core.MNotificationManager");
    QDBusMessage msg = manager.call("updateNotification", notification.userId(), notification.notificationId(), eventType, parameters.value("summary"), parameters.value("body"), newAction, QString(), parameters.value("count"));

    qDebug() << "Change call:" << msg.errorMessage();*/
}

void NotificationHandler::removeNotification(uint notificationId)
{
    //qDebug() << __PRETTY_FUNCTION__ << ":" << notificationId;
}

void NotificationHandler::addGroup(uint groupId, const NotificationParameters& parameters)
{
    // Check if the user wants us to handle notifications
    if ( settings->handleNotifications() == false )
        return;

    // Check if it's an sms
    QString eventType = parameters.value("eventType").toString();
    if ( eventType != "x-nokia.messaging.sms" )
        return;

    // Check the remote action
    QString action = parameters.value("action").toString();
    if ( action.startsWith("com.nokia.Messaging") == false ) {
        qDebug() << "Group" << groupId << "already changed. action=" << action;
        return;
    }

    // Change the action to web2sms
    QString newAction = action.replace("com.nokia", "net.muhkuh.web2sms");

    QVariantList args;
    args << (uint)2 // TODO: Get this key somewhere
         << groupId
         << eventType
         << parameters.value("summary")
         << parameters.value("body")
         << newAction
         << QString()
         << parameters.value("count");

    QDBusInterface manager("com.meego.core.MNotificationManager", "/notificationmanager", "com.meego.core.MNotificationManager");
    QDBusMessage msg = manager.callWithArgumentList(QDBus::Block, "updateGroup", args);

    qDebug() << "Changed group " << groupId << "to Web2SMS remote. Errors:" << msg.errorMessage();
}

void NotificationHandler::removeGroup(uint groupId)
{
    Q_UNUSED(groupId);
    qDebug() << __PRETTY_FUNCTION__ << ":" << groupId;
}
