// Local includes
#include "providerupdatechecker.h"
#include <pluginresolver.h>

// Global includes
#include <QDeclarativeEngine>
#include <QDeclarativeContext>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QPointer>
#include <QUrl>
#include <QDebug>

#ifdef Q_OS_ANDROID
#include <QJson/Parser>
#else // Q_OS_ANDROID
#include <qjson/parser.h>
#endif // Q_OS_ANDROID

class ProviderUpdateCheckerPrivate : public QObject
{
    Q_OBJECT

public:
    ProviderUpdateCheckerPrivate()
    : state(ProviderUpdateChecker::Unknown)
    {
    }

    // Properties
    ProviderUpdateChecker* self;

    // Functions
    void setState(ProviderUpdateChecker::State state);
    void setErrorString(const QString& errorString);

public slots:
    void didFinishRequest();

public:
    QPointer<QNetworkAccessManager> network;

    ProviderUpdateChecker::State state;
    QString errorString;
};

void ProviderUpdateCheckerPrivate::setState(ProviderUpdateChecker::State state)
{
    if ( this->state == state )
        return;

    this->state = state;
    emit self->stateChanged();
}

void ProviderUpdateCheckerPrivate::setErrorString(const QString &errorString)
{
    if ( this->errorString == errorString )
        return;

    this->errorString = errorString;
    emit self->errorStringChanged();
}

void ProviderUpdateCheckerPrivate::didFinishRequest()
{
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());

    if ( reply->error() ) {
        setErrorString(reply->errorString());
        setState(ProviderUpdateChecker::Error);
    } else {
        QByteArray data = reply->readAll();

        bool ok=false;
        QJson::Parser parser;
        QVariant object = parser.parse(data, &ok);

        if ( ok ) {
            QVariantMap hash = object.toMap();
            int timestamp = (int)hash.value("DATE").toDouble();

            qDebug() << object;

            qDebug() << "Available plugin timestamp:" << timestamp;

            if ( PluginResolver::instance()->pluginTimestamp() < timestamp ) {
                qDebug() << "The online version is newer";
                setState(ProviderUpdateChecker::UpdateAvailable);
            } else {
                qDebug() << "The local version is newer";
                setState(ProviderUpdateChecker::NoUpdateAvailable);
            }

            setErrorString(QString());
        } else {
            setErrorString(parser.errorString());
            setState(ProviderUpdateChecker::Error);
        }
    }

    reply->deleteLater();
}

ProviderUpdateChecker::ProviderUpdateChecker(QObject *parent)
: QObject(parent)
, d(new ProviderUpdateCheckerPrivate)
{
    d->self = this;
}

ProviderUpdateChecker::~ProviderUpdateChecker()
{
    delete d;
}

ProviderUpdateChecker::State ProviderUpdateChecker::state() const
{
    return d->state;
}

QString ProviderUpdateChecker::errorString() const
{
    return d->errorString;
}

void ProviderUpdateChecker::setNetworkAccessManager(QNetworkAccessManager *networkAccessManager)
{
    if ( d->network == networkAccessManager )
        return;

    d->network = networkAccessManager;
}

QNetworkAccessManager *ProviderUpdateChecker::networkAccessManager() const
{
    return d->network;
}

void ProviderUpdateChecker::checkForUpdate()
{
    d->setState(Checking);
    d->setErrorString(QString());

    /// Get the network access manager
    /*QDeclarativeContext* context = QDeclarativeEngine::contextForObject(this);
    QDeclarativeEngine* engine = context->engine();
    QNetworkAccessManager* network = engine->networkAccessManager();*/

    /// Send the request
    /// TODO: Don't use a hardcoded url
    QNetworkRequest request( QUrl("https://web2sms.olausson.de:1078/update") );
    QNetworkReply* reply = d->network->post(request, QByteArray());
    connect(reply, SIGNAL(finished()), d, SLOT(didFinishRequest()));
}

#include "providerupdatechecker.moc"
