#ifndef PROVIDERUPDATECHECKER_H
#define PROVIDERUPDATECHECKER_H

#include <QObject>

class QNetworkAccessManager;
class ProviderUpdateCheckerPrivate;

class ProviderUpdateChecker : public QObject
{
    Q_OBJECT
    Q_ENUMS(State)
    Q_PROPERTY(int state READ state NOTIFY stateChanged)
    Q_PROPERTY(QString errorString READ errorString NOTIFY errorStringChanged)

public:
    explicit ProviderUpdateChecker(QObject *parent = 0);
    ~ProviderUpdateChecker();

    enum State
    {
        Unknown,
        Error,
        Checking,
        UpdateAvailable,
        NoUpdateAvailable
    };

    State state() const;

    QString errorString() const;

    /// The network access manager. Required for proxy settings etc.
    void setNetworkAccessManager(QNetworkAccessManager* networkAccessManager);
    QNetworkAccessManager* networkAccessManager() const;

    /// Run the update checker
    Q_INVOKABLE void checkForUpdate();

signals:
    void stateChanged();
    void errorStringChanged();

protected:
    friend class ProviderUpdateCheckerPrivate;
    ProviderUpdateCheckerPrivate* d;
};

#endif // PROVIDERUPDATECHECKER_H
