#ifndef PROVIDERUPDATEINSTALLER_H
#define PROVIDERUPDATEINSTALLER_H

// Global includes
#include <QObject>

class QNetworkAccessManager;
class ProviderUpdateInstallerPrivate;

class ProviderUpdateInstaller : public QObject
{
    Q_OBJECT
    Q_ENUMS(State)
    Q_PROPERTY(int state READ state NOTIFY stateChanged)
    Q_PROPERTY(QString errorString READ errorString NOTIFY errorStringChanged)
    Q_PROPERTY(qreal progress READ progress NOTIFY progressChanged)

public:
    explicit ProviderUpdateInstaller(QObject *parent = 0);
    ~ProviderUpdateInstaller();
    
    enum State
    {
        Unknown,
        Error,
        Installing,
        Done
    };

    State state() const;
    QString errorString() const;
    qreal progress() const;

    /// The network access manager. Required for proxy settings etc.
    void setNetworkAccessManager(QNetworkAccessManager* networkAccessManager);
    QNetworkAccessManager* networkAccessManager() const;

    /// Install updates
    Q_INVOKABLE void installUpdate();

signals:
    void stateChanged();
    void errorStringChanged();
    void progressChanged();

protected:
    ProviderUpdateInstallerPrivate* d;
    friend class ProviderUpdateInstallerPrivate;
};

#endif // PROVIDERUPDATEINSTALLER_H
