#include "settings.h"

Settings::Settings(QObject *parent)
: QObject(parent)
, m_settings(QSettings::UserScope, "net.muhkuh", "Web2SMS")
{
}

void Settings::setActiveAccountId(const QString& activeAccountId)
{
    m_settings.setValue("activeAccountId", activeAccountId);
    emit activeAccountIdChanged();
}

QString Settings::activeAccountId() const
{
    return m_settings.value("activeAccountId").toString();
}

void Settings::setUseCustomServer(bool useCustomServer)
{
    m_settings.setValue("useCustomServer", QVariant::fromValue(useCustomServer));
    emit useCustomServerChanged();
}

bool Settings::useCustomServer() const
{
    return m_settings.value("useCustomServer", false).toBool();
}

void Settings::setCustomServerAddress(const QString& customServerAddress)
{
    m_settings.setValue("customServerAddress", customServerAddress);
    emit customServerAddressChanged();
}

QString Settings::customServerAddress() const
{
    return m_settings.value("customServerAddress").toString();
}

void Settings::setHandleNotifications(bool handleNotifications)
{
    m_settings.setValue("handleNotifications", handleNotifications);
    emit handleNotificationsChanged();
}

bool Settings::handleNotifications() const
{
    return m_settings.value("handleNotifications", false).toBool();
}

void Settings::setShowBetaWarning(bool showBetaWarning)
{
    m_settings.setValue("showBetaWarning", showBetaWarning);
    emit showBetaWarningChanged();
}

bool Settings::showBetaWarning() const
{
    return m_settings.value("showBetaWarning", true).toBool();
}

void Settings::setNightMode(bool nightMode)
{
    m_settings.setValue("nightMode", QVariant::fromValue(nightMode));
    emit nightModeChanged();
}

bool Settings::nightMode() const
{
    return m_settings.value("nightMode", false).toBool();
}

void Settings::setInboundBubbleColor(int inboundBubbleColor)
{
    m_settings.setValue("inboundBubbleColor", inboundBubbleColor);
    emit inboundBubbleColorChanged();
}

int Settings::inboundBubbleColor() const
{
    return m_settings.value("inboundBubbleColor", 2).toInt();
}

void Settings::setOutboundBubbleColor(int outboundBubbleColor)
{
    m_settings.setValue("outboundBubbleColor", outboundBubbleColor);
    emit outboundBubbleColorChanged();
}

int Settings::outboundBubbleColor() const
{
    return m_settings.value("outboundBubbleColor", 1).toInt();
}

void Settings::setBubbleFontFactor(qreal bubbleFontFactor)
{
    m_settings.setValue("bubbleFontFactor", QVariant::fromValue(bubbleFontFactor));
    emit bubbleFontFactorChanged();
}

qreal Settings::bubbleFontFactor() const
{
    return m_settings.value("bubbleFontFactor", 1.0).toReal();
}

void Settings::setBubbleAnimation(int bubbleAnimation)
{
    if ( this->bubbleAnimation() == bubbleAnimation )
        return;

    m_settings.setValue("bubbleAnimation", bubbleAnimation);
    emit bubbleAnimationChanged();
}

int Settings::bubbleAnimation() const
{
    return m_settings.value("bubbleAnimation", 1).toInt();
}

void Settings::setDailyUpdateCheck(bool dailyUpdateCheck)
{
    if ( this->dailyUpdateCheck() == dailyUpdateCheck )
        return;

    m_settings.setValue("dailyUpdateCheck", dailyUpdateCheck);
    emit dailyUpdateCheckChanged();
}

bool Settings::dailyUpdateCheck() const
{
    return m_settings.value("dailyUpdateCheck", true).toBool();
}

void Settings::setAutoInstallUpdate(bool autoInstallUpdate)
{
    if ( this->autoInstallUpdate() == autoInstallUpdate )
        return;

    m_settings.setValue("autoInstallUpdate", autoInstallUpdate);
    emit autoInstallUpdateChanged();
}

bool Settings::autoInstallUpdate() const
{
    return m_settings.value("autoInstallUpdate", true).toBool();
}

void Settings::setLastUpdateCheck(const QDateTime &lastUpdateCheck)
{
    if ( this->lastUpdateCheck() == lastUpdateCheck )
        return;

    m_settings.setValue("lastUpdateCheck", lastUpdateCheck);
    emit lastUpdateCheckChanged();
}

QDateTime Settings::lastUpdateCheck() const
{
    return m_settings.value("lastUpdateCheck", true).toDateTime();
}

QString Settings::lastUpdateCheckString() const
{
    QDateTime dt = lastUpdateCheck();
    if ( dt.isNull() ) {
        //% "Never"
        return qtTrId("qtn_web2sms_settings_lastupdate_never");
    }

    return dt.toString(Qt::SystemLocaleShortDate);
}
