#ifndef SETTINGS_H
#define SETTINGS_H

#include <QObject>
#include <QSettings>
#include <QDateTime>

class Settings : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString activeAccountId READ activeAccountId WRITE setActiveAccountId NOTIFY activeAccountIdChanged)
    Q_PROPERTY(bool useCustomServer READ useCustomServer WRITE setUseCustomServer NOTIFY useCustomServerChanged)
    Q_PROPERTY(QString customServerAddress READ customServerAddress WRITE setCustomServerAddress NOTIFY customServerAddressChanged)
    Q_PROPERTY(bool handleNotifications READ handleNotifications WRITE setHandleNotifications NOTIFY handleNotificationsChanged)
    Q_PROPERTY(bool showBetaWarning READ showBetaWarning WRITE setShowBetaWarning NOTIFY showBetaWarningChanged)
    Q_PROPERTY(bool nightMode READ nightMode WRITE setNightMode NOTIFY nightModeChanged)

    Q_PROPERTY(int inboundBubbleColor READ inboundBubbleColor WRITE setInboundBubbleColor NOTIFY inboundBubbleColorChanged)
    Q_PROPERTY(int outboundBubbleColor READ outboundBubbleColor WRITE setOutboundBubbleColor NOTIFY outboundBubbleColorChanged)
    Q_PROPERTY(qreal bubbleFontFactor READ bubbleFontFactor WRITE setBubbleFontFactor NOTIFY bubbleFontFactorChanged)
    Q_PROPERTY(int bubbleAnimation READ bubbleAnimation WRITE setBubbleAnimation NOTIFY bubbleAnimationChanged)

    Q_PROPERTY(bool dailyUpdateCheck READ dailyUpdateCheck WRITE setDailyUpdateCheck NOTIFY dailyUpdateCheckChanged)
    Q_PROPERTY(bool autoInstallUpdate READ autoInstallUpdate WRITE setAutoInstallUpdate NOTIFY autoInstallUpdateChanged)
    Q_PROPERTY(QDateTime lastUpdateCheck READ lastUpdateCheck WRITE setLastUpdateCheck NOTIFY lastUpdateCheckChanged)
    Q_PROPERTY(QString lastUpdateCheckString READ lastUpdateCheckString NOTIFY lastUpdateCheckChanged)

public:
    explicit Settings(QObject *parent = 0);

    void setActiveAccountId(const QString& activeAccountId);
    QString activeAccountId() const;

    void setUseCustomServer(bool useCustomServer);
    bool useCustomServer() const;

    void setCustomServerAddress(const QString& customServerAddress);
    QString customServerAddress() const;

    void setHandleNotifications(bool handleNotifications);
    bool handleNotifications() const;

    void setShowBetaWarning(bool showBetaWarning);
    bool showBetaWarning() const;

    void setNightMode(bool nightMode);
    bool nightMode() const;
    Q_SIGNAL void nightModeChanged();

    /// Bubble settings
    void setInboundBubbleColor(int inboundBubbleColor);
    int inboundBubbleColor() const;

    void setOutboundBubbleColor(int outboundBubbleColor);
    int outboundBubbleColor() const;

    void setBubbleFontFactor(qreal bubbleFontFactor);
    qreal bubbleFontFactor() const;

    void setBubbleAnimation(int bubbleAnimation);
    int bubbleAnimation() const;

    /// Provider update settings
    void setDailyUpdateCheck(bool dailyUpdateCheck);
    bool dailyUpdateCheck() const;

    void setAutoInstallUpdate(bool autoInstallUpdate);
    bool autoInstallUpdate() const;

    void setLastUpdateCheck(const QDateTime& lastUpdateCheck);
    QDateTime lastUpdateCheck() const;
    QString lastUpdateCheckString() const;

signals:
    void activeAccountIdChanged();
    void useCustomServerChanged();
    void customServerAddressChanged();
    void handleNotificationsChanged();
    void showBetaWarningChanged();

    void inboundBubbleColorChanged();
    void outboundBubbleColorChanged();
    void bubbleFontFactorChanged();
    void bubbleAnimationChanged();

    void dailyUpdateCheckChanged();
    void autoInstallUpdateChanged();
    void lastUpdateCheckChanged();

protected:
    QSettings m_settings;
};

#endif // SETTINGS_H
