import QtQuick 1.1
import com.nokia.meego 1.0
import "/usr/lib/qt4/imports/com/nokia/extras/constants.js" as UI
import Web2SMS 1.0

Item {
    id: listItem

    signal clicked
    property alias pressed: mouseArea.pressed

    property alias iconSource: image.source
    property alias titleText: mainText.text
    property alias subtitleText: subText.text

    property int titleSize: UI.LIST_TILE_SIZE
    property int titleWeight: Font.Bold
    property color titleColor: theme.inverted ? UI.LIST_TITLE_COLOR_INVERTED : UI.LIST_TITLE_COLOR
    property color titleColorSelected: UI.LIST_TITLE_COLOR_INVERTED // Seems to look always the best

    property int subtitleSize: UI.LIST_SUBTILE_SIZE
    property int subtitleWeight: Font.Light
    property color subtitleColor: theme.inverted ? UI.LIST_SUBTITLE_COLOR_INVERTED : UI.LIST_SUBTITLE_COLOR
    property color subtitleColorSelected: UI.LIST_SUBTITLE_COLOR_INVERTED

    property bool showImage: true
    property bool showSelection: true

    property QtObject provider

    Component.onCompleted: {
        // Sanity check
        if ( typeof(model) == "undefined" || typeof(model.providerId) == "undefined" )
            return;

        // Get the provider
        provider = backend.plugins.providerById(model.providerId);

        // Set the provider name
        listItem.titleText = provider.name;
        listItem.iconSource = provider.icon;
        listItem.subtitleText = model.username;

        // Add the beta information
        if ( provider.state == ProviderInfo.Beta )
            listItem.titleText += " (beta)";
    }

    height: UI.LIST_ITEM_HEIGHT
    width: parent.width

    BorderImage {
        id: background
        anchors.fill: parent
        // Fill page porders
        anchors.leftMargin: -UI.MARGIN_XLARGE
        anchors.rightMargin: -UI.MARGIN_XLARGE
        visible: mouseArea.pressed
        source: theme.inverted ? "image://theme/meegotouch-list-inverted-background-pressed-center" :
                                 "image://theme/meegotouch-list-background-pressed-center"
    }

    BorderImage {
        id: selectedBackground
        anchors.fill: parent
        // Fill page porders
        anchors.leftMargin: -UI.MARGIN_XLARGE
        anchors.rightMargin: -UI.MARGIN_XLARGE
        visible: showSelection && listItem.ListView.isCurrentItem
        source: theme.inverted ? "image://theme/meegotouch-list-inverted-background-selected-center" :
                                 "image://theme/meegotouch-list-background-selected-center"
    }

    Row {
        anchors.fill: parent
        spacing: UI.LIST_ITEM_SPACING

        Image {
            id: image
            anchors.verticalCenter: parent.verticalCenter
            //visible: model.iconSource ? true : false
            visible: showImage
            width: UI.LIST_ICON_SIZE
            height: UI.LIST_ICON_SIZE
            //source: model.iconSource ? model.iconSource : ""
            asynchronous: true
        }

        Column {
            anchors.verticalCenter: parent.verticalCenter

            Label {
                id: mainText
                font.weight: listItem.titleWeight
                font.pixelSize: listItem.titleSize
                color: (showSelection && listItem.ListView.isCurrentItem) ? listItem.titleColorSelected :
                                                                            listItem.titleColor
            }

            Label {
                id: subText
                font.weight: listItem.subtitleWeight
                font.pixelSize: listItem.subtitleSize
                color: (showSelection && listItem.ListView.isCurrentItem) ? listItem.subtitleColorSelected :
                                                                            listItem.subtitleColor

                visible: text != ""
            }

            /*Label {
                id: disabledText
                //% "Disabled"
                text: qsTrId("qtn_web2sms_disabled");
                visible: model.enabled
            }*/
        }
    }
    MouseArea {
        id: mouseArea;
        anchors.fill: parent
        onClicked: {
            listItem.clicked();
        }
    }
}
