#include "plugin.h"

// Type includes
#include "groupmodel.h"
#include "conversationmodel.h"
#include "contactpicker.h"
#include "clipboard.h"
#include "avatarimage.h"

#include "accountsmodel.h"
#include "providersmodel.h"
#include "accountwrapper.h"
#include "backend.h"
#include "contactmodel.h"

#include "messagetype.hpp"
#include "messagestatistics.h"
#include "pluginresolver.h"

// Global includes
#include <QtDeclarative/qdeclarative.h>
#include <QDeclarativeEngine>
#include <QDeclarativeContext>
#include <QApplication>
#include <QSslSocket>
#include <QFile>
#include <QDebug>
#include <QSslConfiguration>

Web2SMSDeclarativePlugin::Web2SMSDeclarativePlugin()
{
    // Install the CAcert root certificate
    QFile certFile("/opt/web2sms/share/test.crt");
    if ( certFile.open(QIODevice::ReadOnly) ) {
        QSslCertificate cert(&certFile, QSsl::Pem);
        if ( cert.isValid() ) {
            QSslSocket::addDefaultCaCertificate(cert);


            QSslConfiguration configuration = QSslConfiguration::defaultConfiguration();
            // Adjust the protocol
            configuration.setProtocol(QSsl::TlsV1);
            configuration.setPeerVerifyMode(QSslSocket::VerifyNone);

            // Adjust the certificates
            QList<QSslCertificate> certificates = configuration.caCertificates();
            certificates.append(cert);
            configuration.setCaCertificates(certificates);

            QSslConfiguration::setDefaultConfiguration(configuration);

            qDebug() << "Registered CAcert root certificate";
        } else {
            qDebug() << "CAcert root certificate is invalid";
        }
    } else {
        qDebug() << "Error reading CAcert root certificate";
    }
}

Web2SMSDeclarativePlugin::~Web2SMSDeclarativePlugin()
{
}

void Web2SMSDeclarativePlugin::registerTypes(const char *uri)
{
    // Register our qml objects
    qmlRegisterType<AvatarImage>(uri, 1, 0, "AvatarImage");
    qmlRegisterType<GroupModel>(uri, 1, 0, "GroupModel");
    qmlRegisterType<ConversationModel>(uri, 1, 0, "ConversationModel");
    qmlRegisterType<ContactHelper>(uri, 1, 0, "ContactPicker");
    qmlRegisterType<ContactModel>(uri, 1, 0, "ContactModel");

    qmlRegisterUncreatableType<MessageType>(uri, 1, 0, "MessageType", "MessageType is an internal type");
    qmlRegisterUncreatableType<MessageHandler>(uri, 1, 0, "MessageHandler", "MessageHandler is an internal type");
    qmlRegisterType<MessageStatistics>(uri, 1, 0, "MessageStatistics");

    // AccountSetup
    qmlRegisterType<AccountsModel>(uri, 1, 0, "AccountsModel");
    qmlRegisterType<ProvidersModel>(uri, 1, 0, "ProvidersModel");
    qmlRegisterType<AccountWrapper>(uri, 1, 0, "Account");
    qmlRegisterType<ProviderInfo>(uri, 1, 0, "ProviderInfo");
    qmlRegisterUncreatableType<PluginResolver>(uri, 1, 0, "PluginManager", "PluginManager is an internal type");
    qmlRegisterUncreatableType<Backend>(uri, 1, 0, "Backend", "Backend is an internal type");
}

void Web2SMSDeclarativePlugin::initializeEngine(QDeclarativeEngine *engine, const char *uri)
{
    Q_UNUSED(uri);

    // Initialize the plugins
    PluginResolver::instance()->resolvePlugins();

    // Get the root context
    QDeclarativeContext* context = engine->rootContext();

    // Register the clipboard
    context->setContextProperty("backend", new Backend);
    context->setContextProperty("clipboard", new Clipboard);
}

Q_EXPORT_PLUGIN2(web2sms, Web2SMSDeclarativePlugin)
