# Copyright 1999-2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
# $Header: $

EAPI="2"

inherit eutils

DESCRIPTION="web2sms is a daemon which sends SMS for the web2sms client"
HOMEPAGE="https://projects.developer.nokia.com/web2sms"
SRC_URI="http://olausson.de/downloads/finish/7-web2sms/28-web2sms -> ${PN}-${PV}.tar.bz2"

LICENSE="GPL-3"
SLOT="0"
KEYWORDS="~x86 ~amd64"
IUSE="ssl"

RDEPEND=">=dev-lang/python-2.7[ssl,xml]
    >=dev-python/twisted-11.0.0
    >=dev-python/twisted-web-11.0.0
    >=dev-python/mechanize-0.2.5
    >=dev-python/requests-0.9.0
    >=dev-python/pyopenssl-0.12
    >=dev-python/configobj-4.72
    >=dev-python/simplejson-2.2.1
    >=dev-python/lxml-2.3.1
    >=dev-python/beautifulsoup-3.2.0
    >=www-servers/apache-2.2.21-r1[vhost]"

DEPEND="${RDEPEND}"

pkg_setup() {
    enewgroup web2sms || die
    enewuser web2sms -1 -1 /home/web2sms web2sms || die
}

src_unpack() {
    unpack "${A}"
}

src_prepare() {
    cfg="${S}/webservice/web2sms.cfg"
    if ! use ssl ; then
        sed -e 's/USE_SSL = True/USE_SSL = False/' -i "${cfg}" || die
    fi
}

src_install() {
    diropts -m0755
    dodir /opt/web2sms || die
    dodir /var/log/web2sms || die
    dodir /home/web2sms/www || die
    dodir /opt/web2sms/plugins || die

    sd="/opt/web2sms/webservice"
    dodir "${sd}" || die
    insinto "${sd}"
    doins webservice/*.py || die

    sd="/etc/web2sms"
    dodir "${sd}" || die
    dodir "${sd}"/keys || die
    insinto "${sd}"
    doins webservice/web2sms.cfg || die

    insinto /etc/apache2/vhosts.d
    doins webservice/web2sms_vhost.include || die

    insinto /etc/init.d
    newexe webservice/web2sms.init web2sms || die

    if use logrotate ; then
        insinto /etc/logrotate.d
        doins webservice/web2sms.logrotate || die
    fi

    fowners web2sms:web2sms /home/web2sms/www || die
    fowners web2sms:web2sms /var/log/web2sms || die
}

pkg_postinst() {
    einfo "You need to download the latest plugins from Gitorius:"
    einfo "git clone git://gitorious.org/web2sms/plugins.git /opt/web2sms/plugins"
    einfo ""

    if use ssl ; then
        einfo "You have chosen to use SSL communication between server and client."
        einfo "To make this work you have to generate a server key and put it in /etc/web2sms/keys/"
        einfo "Here's an example how to create a key"
        einfo ""
        einfo "# Generate a server key"
        einfo "openssl genrsa -des3 -out server.key 4096"
        einfo ""
        einfo "# Generate a request for signing"
        einfo "openssl req -new -key server.key -out server.csr"
        einfo ""
        einfo "# Send \"server.csr\" to e.g. http://cacert.org/ and sign the key with the resutling \"ca.crt\""
        einfo "openssl x509 -req -days 3650 -in server.csr -CA ca.crt -CAkey server.key -set_serial 01 -out server.crt"
        einfo ""
        einfo "# Remove password from server.key"
        einfo "openssl rsa -in server.key -out server.key.insecure"
    fi

}