#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#------------------------------------------------------------------------------
#web2sms_server.py, Copyright Bjoern Olausson
#------------------------------------------------------------------------------
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#To view the license visit
#http://www.gnu.org/licenses/gpl-3.0.html
#or write to
#Free Software Foundation, Inc.
#51 Franklin St, Fifth Floor
#Boston, MA
#02110-1301 USA
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#
# Python server to accept JSON payload from www2sms
#
# twistd -y web2sms_server.py
# twistd --pidfile twistd2.pid --logfile twistd2.log -y web2sms_server.py
#
__version__ = '2.1.1'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import os
import sys
import glob
import time
import urllib2
import hashlib
import simplejson
import ConfigParser
from pprint import pprint
from OpenSSL import SSL
from urllib import urlencode
from datetime import datetime
from optparse import OptionParser
from twisted.web.resource import Resource
from twisted.application.service import Application
from twisted.web.server import Site, NOT_DONE_YET
from twisted.internet import ssl, reactor, threads, protocol
# use a t.application.internet.TCPServer (or SSLServer) instead of reactor.listenTCP/reactor.listenSSL, and return that from makeService. And write a simple usage.Options subclass that lets you specify that config file. Shouldn't be too hard.
# https://bitbucket.org/jerub/twisted-plugin-example

#usage = '''twistd -ny web2sms_server.py'''

#opa = OptionParser(usage=usage, version=__version__)

#opa.add_option("-c", action="store", dest="configfile",
    #metavar="path", default="/home/blub/Desktop/DEV/web2sms/web2sms.cfg",
    #help="Use alternate config file.")
#opa.add_option("-i", action="store", dest="useid",
    #metavar="str", default="web2sms",
    #help="Load alternate section from config")

## instruct optparse to parse the program's command line
#(options, args) = opa.parse_args()

# Which config file to use
#configfile = options.configfile
configfile = "/etc/web2sms/web2sms.cfg"

# Which ID to load from the config
#ID = options.useid
ID = "web2sms"

config = ConfigParser.RawConfigParser()
config.read(configfile)

USE_SSL = config.getboolean('global', 'USE_SSL')
KEY_PATH = config.get('global', 'KEY_PATH')
KEY = config.get('global', 'KEY')
CRT = config.get('global', 'CRT')
PLUGIN_DIR = config.get('global', 'PLUGIN_DIR')
UPDATE_ZIP = config.get('global', 'UPDATE_ZIP')
UPDATE_ZIP_URL = config.get('global', 'UPDATE_ZIP_URL')
WEB2SMS_API = config.get(ID, 'WEB2SMS_API')
USE_PUSH = config.getboolean('global', 'USE_PUSH')


# Check if we are supposed to work with PUSH
if USE_PUSH:
    PORT = config.getint('global', 'PORT_PUSH')
else:
    PORT = config.getint('global', 'PORT_NOPUSH')

# Append the directories in PLUGIN_DIR to the PYTHONPATH
PLUGIN_DIR_LIST = glob.glob(PLUGIN_DIR + "/*")
for DIR in PLUGIN_DIR_LIST:
    if os.path.isdir(DIR):
        sys.path.append(DIR)


def push(result, PUSHID, EVENTID):
    print "Sending PUSH"
    SERVICE_ID = config.get(ID, 'SERVICE_ID')
    APPLICATION_ID = config.get(ID, 'APPLICATION_ID')
    SERVICE_SECRET = config.get(ID, 'SERVICE_SECRET')

    NOKIA_PUSH_API = config.get('global', 'NOKIA_PUSH_API')
    #Uses Nokia-Account ID for pushing the notification
    URL = "%(A)s/%(P)s@ovi.com" % {"A": NOKIA_PUSH_API, "P": PUSHID}

    HTTPS_HANDLER = urllib2.HTTPSHandler()
    AUTH_HANDLER = urllib2.HTTPDigestAuthHandler()
    AUTH_HANDLER.add_password("null", URL, SERVICE_ID, SERVICE_SECRET)

    OPENER = urllib2.build_opener(HTTPS_HANDLER, AUTH_HANDLER)
    urllib2.install_opener(OPENER)

    try:
        result["EVENTID"] = EVENTID
    except Exception, e:
        result["EVENTID"] = {"0-0-0-0-0"}
        MSG = {"STATUS": {
            "TYPE": "error",
            "MESSAGE": e
                                            }
                    }
        print MSG
        return MSG

    PAYLOAD = simplejson.dumps(result)
    DATA = {"toapp": APPLICATION_ID,
        "payload": PAYLOAD,
        "wakeup": "true"
        }

    print DATA

    try:
        F = urllib2.urlopen(URL, urlencode(DATA))
    except urllib2.HTTPError, e:
        MSG = {"STATUS": {
            "TYPE": "error",
            "MESSAGE": str(e.code)
                                            }
                    }
        print MSG
        return MSG
    except urllib2.URLError, e:
        MSG = {"STATUS": {
            "TYPE": "error",
            "MESSAGE": str(e.reason)
                                            }
                    }
        print MSG
        return MSG
    else:
        response = F.read()
        # If response is empty the HTTP result code is 204 which means that the
        # recipient application is unreachable
        if response == "":
            F.close()
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Application not reachable"
                                                }
                        }
            print MSG
            return MSG
        else:
            F.close()
            MSG = {"STATUS": {
                "TYPE": "success",
                "MESSAGE": response
                                                }
                        }
            print MSG
            return MSG


class RunPlugin():
    # This is a blocking method
    def load(self, PROVIDERID, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        print "Running ", PROVIDERID
        try:
            P = __import__(PROVIDERID)
        except Exception, e:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Failed to load Plugin"
                    }
                }
            print MSG
            return MSG

        try:
            PMSG = P.provider().sendSMS(
                LOGIN,
                PASSWORD,
                NUMBERS,
                SMSTEXT,
                PLUGINDICT
                )
        except Exception, e:
            what = "- %(p)s - Faild to execute!\n" % {"p": PROVIDERID}
            reason = "Exception:\n%(err)s\n\n" % {"err": e}
            try:
                preturned = "Plugin return:\n%(ret)s\n\n" % {"ret": PMSG}
            except Exception, e:
                preturned = "Plugin returned nothing!\n\n"
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": what + reason + preturned
                    }
                }
            print MSG
            return MSG
        else:
            print PMSG
            return PMSG

    # Lets run the blocking method using Deferred...
    def run(self, DATA):
        p = threads.deferToThread(self.load,
            DATA["PROVIDERID"].encode('utf8'),
            DATA["LOGIN"].encode('utf8'),
            DATA["PASSWORD"].encode('utf8'),
            DATA["NUMBERS"],
            DATA["SMSTEXT"].encode('utf8'),
            DATA["PLUGINDICT"]
            )
        PUSHID = DATA["PUSHID"].encode('utf8')
        EVENTID = DATA["EVENTID"].encode('utf8')
        p.addCallback(self.suc, PUSHID, EVENTID)
        p.addErrback(self.err, PUSHID, EVENTID)
        return p

    def suc(self, CALLBACK, PUSHID, EVENTID):
        if USE_PUSH:
            push(CALLBACK, PUSHID, EVENTID)
        else:
            return CALLBACK

    def err(self, CALLBACK, PUSHID, EVENTID):
        if USE_PUSH:
            push(CALLBACK, PUSHID, EVENTID)
        else:
            return CALLBACK


# Setup the server
class SendPage(Resource):
    def checkParameters(self, data):
        # Ensure numbers is a list type
        if not isinstance(data["NUMBERS"], list):
            return False

    def render_POST(self, request):
        #pprint(request.__dict__)
        DATA = simplejson.loads(request.content.getvalue())

        if self.checkParameters(DATA) == False:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Malformed parameters"
                                                }
                        }
            return simplejson.dumps(MSG)

        # Check for a Nokia account
        nokia_acc = DATA["PUSHID"]
        if not nokia_acc:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Missing Nokia Account, please create on first!"
                    }
                }
            print MSG
            return simplejson.dumps(MSG)

        # Check if the password is empty. If it is, this indicates that
        # the client is using the buggy 1.0.0 Client
        pwtest = DATA["PASSWORD"]
        if not pwtest:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "You are using a broken client version"
                    }
                }
            print MSG
            return simplejson.dumps(MSG)

        # Check for valid API ID
        try:
            API_ID = DATA["WEB2SMS_API"]
        except Exception, e:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Connection refused"
                                                }
                        }
            print "Missing API ID"
            return simplejson.dumps(MSG)
        else:
            if API_ID != WEB2SMS_API:
                MSG = {"STATUS": {
                    "TYPE": "error",
                    "MESSAGE": "Connection refused"
                                                    }
                            }
                print WEB2SMS_API
                print API_ID
                print "Invalid API ID"
                return simplejson.dumps(MSG)

        PLUGIN = RunPlugin()
        S = PLUGIN.run(DATA)

        # Method called when the client needs feedback
        # Only used when _NOT_ using PUSH
        def send(result):
            STATUS = simplejson.dumps(result)
            request.write(STATUS)
            request.finish()

        if USE_PUSH:
            # If PUSH is working, we disconnect the client ASAP
            # and send the response via PUSH
            MSG = {"STATUS": {"TYPE": "delayed"}}
            STATUS = simplejson.dumps(MSG)
            return STATUS
        else:
            # As long as PUSH is not working,
            # we keep the client waiting for response
            S.addCallback(send)
            return NOT_DONE_YET


class NotifyPage(Resource):
    def render_POST(self, request):
        #pprint(request.__dict__)
        MSG = "This is the notification API"
        return MSG


class RegisterPage(Resource):
    def render_POST(self, request):
        return "This is the registration API"


class PluginsPage(Resource):
    def render_POST(self, request):
        return "This is the plugin lister API"


class UpdatePage(Resource):
    def get_file_attributes(self):
        t_start = time.time()
        md5 = hashlib.md5()
        try:
            f = open(UPDATE_ZIP, 'rb')
        except IOError:
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "%(f)s not found" % {"f": UPDATE_ZIP}
                                                },
                "FILE": False,
                "DATE": False,
                "MD5": False,
                "PTIME:": False
                        }
        else:
            for line in f:
                md5.update(line)

            md5sum = md5.hexdigest()
            ts = os.path.getctime(UPDATE_ZIP)
            t_stop = time.time() - t_start

            MSG = {"STATUS": {
                    "TYPE": "success",
                    "MESSAGE": "%(f)s found" % {"f": UPDATE_ZIP}
                                                },
                "FILE": UPDATE_ZIP_URL,
                "DATE": ts,
                "MD5": md5sum,
                "PTIME:": t_stop
                        }

        return simplejson.dumps(MSG)

    def render_POST(self, request):
        MSG = self.get_file_attributes()
        return MSG


#Initiate and start the server
root = Resource()
root.putChild("send", SendPage())
root.putChild("notify", NotifyPage())
root.putChild("register", RegisterPage())
root.putChild("plugins", PluginsPage())
root.putChild("update", UpdatePage())

application = Application("web2sms")

if USE_SSL:
    myContextFactory = ssl.DefaultOpenSSLContextFactory(
        '%(P)s/%(K)s' % {"P": KEY_PATH, "K": KEY},
        '%(P)s/%(K)s' % {"P": KEY_PATH, "K": CRT}
        )

    reactor.listenSSL(PORT,
        Site(root),
        myContextFactory
        )
else:
    reactor.listenTCP(PORT, Site(root))

#reactor.run()
