#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#------------------------------------------------------------------------------
#web2sms_server.py, Copyright Bjoern Olausson (maemo@olausson.de)
#------------------------------------------------------------------------------
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#To view the license visit
#http://www.gnu.org/licenses/gpl-3.0.html
#or write to
#Free Software Foundation, Inc.
#51 Franklin St, Fifth Floor
#Boston, MA
#02110-1301 USA
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#
# Python server to accept JSON payload from web2sms
#
# twistd -y web2sms_server.py
# twistd --pidfile twistd2.pid --logfile twistd2.log -y web2sms_server.py
#
__version__ = '1.1.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])

import os
import sys
import glob
import time
import pprint
import urllib2
import hashlib
import smtplib
import simplejson
import collections
import ConfigParser

from OpenSSL import SSL
from urllib import urlencode
from datetime import datetime
from optparse import OptionParser

from twisted.web.resource import Resource
from twisted.application.service import Application
from twisted.web.server import Site, NOT_DONE_YET
from twisted.internet import ssl, reactor, threads, protocol

from email import Encoders
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart


pp = pprint.PrettyPrinter(indent=4)

#usage = '''twistd -ny web2sms_server.py'''

#opa = OptionParser(usage=usage, version=__version__)

#opa.add_option("-c", action="store", dest="configfile",
    #metavar="path", default="/home/blub/Desktop/DEV/web2sms/web2sms.cfg",
    #help="Use alternate config file.")
#opa.add_option("-i", action="store", dest="useid",
    #metavar="str", default="web2sms",
    #help="Load alternate section from config")

## instruct optparse to parse the program's command line
#(options, args) = opa.parse_args()

# Which config file to use
#configfile = options.configfile
try:
    cfg = open("../web2sms.cfg", "r")
except IOError, e:
    configfile = "/etc/web2sms/web2sms.cfg"
else:
    configfile = "../web2sms.cfg"
    cfg.close()

# Which ID to load from the config
#ID = options.useid
ID = "web2sms"

config = ConfigParser.RawConfigParser()
config.read(configfile)

USE_SSL = config.getboolean('global', 'USE_SSL')
KEY_PATH = config.get('global', 'KEY_PATH')
KEY = config.get('global', 'KEY')
CRT = config.get('global', 'CRT')
PLUGIN_DIR = config.get('global', 'PLUGIN_DIR')
SERVER_DIR = config.get('global', 'SERVER_DIR')
UPDATE_ZIP = config.get('global', 'UPDATE_ZIP')
UPDATE_ZIP_URL = config.get('global', 'UPDATE_ZIP_URL')
WEB2SMS_API = config.get(ID, 'WEB2SMS_API')
USE_PUSH = config.getboolean('global', 'USE_PUSH')
GMAIL_USER = config.get('global', 'GMAIL_USER')
GMAIL_PASSWD = config.get('global', 'GMAIL_PASSWD')
ERROR_MAIL_TO = config.get('global', 'ERROR_MAIL_TO')

# Check if we are supposed to work with PUSH
if USE_PUSH:
    PORT = config.getint('global', 'PORT_PUSH')
else:
    PORT = config.getint('global', 'PORT_NOPUSH')

# Append the directories in PLUGIN_DIR to the PYTHONPATH
# and load the broken_clients list
PLUGIN_DIR_LIST = glob.glob(PLUGIN_DIR + "/*")

for DIR in PLUGIN_DIR_LIST:
    if os.path.isdir(DIR):
        sys.path.append(DIR)

# Append the server dir to load additional modules
sys.path.append(SERVER_DIR)
b_c = __import__("broken_clients")
broken_clients = b_c.broken_clients
r_r = __import__("nokia_rest_api")
rest_repsonse = r_r.rest_repsonse
rest_error = r_r.rest_error


def convert(data):
    if isinstance(data, unicode):
        return unicode(data, "utf-8")
    elif isinstance(data, collections.Mapping):
        return dict(map(convert, data.iteritems()))
    elif isinstance(data, collections.Iterable):
        return type(data)(map(convert, data))
    else:
        return unicode(data, "utf-8")


def email(result, MAIL_TO, SUBJECT="error log", attach="false"):
    '''Code taken from http://goo.gl/Q1hPl'''
    print "Sending mail to: " + MAIL_TO
    text = simplejson.dumps(result)

    msg = MIMEMultipart()

    msg['From'] = GMAIL_USER
    msg['To'] = ERROR_MAIL_TO
    msg['Subject'] = "[web2sms-error] %(subj)s" % {"subj": SUBJECT}

    msg.attach(MIMEText(text))

    if attach != "false":
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(open(attach, 'rb').read())
        Encoders.encode_base64(part)
        part.add_header('Content-Disposition',
            'attachment; filename="%s"' % os.path.basename(attach))
        msg.attach(part)

    mailServer = smtplib.SMTP("smtp.gmail.com", 587)
    mailServer.ehlo()
    mailServer.starttls()
    mailServer.ehlo()
    mailServer.login(GMAIL_USER, GMAIL_PASSWD)
    mailServer.sendmail(GMAIL_USER, MAIL_TO, msg.as_string())
    # Should be mailServer.quit(), but that crashes...
    mailServer.close()


def push(result, PUSHID, EVENTID, PROVIDER):
    print "Sending PUSH"
    SERVICE_ID = config.get(ID, 'SERVICE_ID')
    APPLICATION_ID = config.get(ID, 'APPLICATION_ID')
    SERVICE_SECRET = config.get(ID, 'SERVICE_SECRET')

    NOKIA_PUSH_API = config.get('global', 'NOKIA_PUSH_API')
    #Uses Nokia-Account ID for pushing the notification
    IDL = PUSHID.split("@")
    if len(IDL) > 1:
        PID = PUSHID
    else:
        PID = str(PUSHID) + "@ovi.com"

    URL = "%(A)s/%(P)s" % {"A": NOKIA_PUSH_API, "P": PID}

    HTTPS_HANDLER = urllib2.HTTPSHandler()
    AUTH_HANDLER = urllib2.HTTPDigestAuthHandler()
    AUTH_HANDLER.add_password("null", URL, SERVICE_ID, SERVICE_SECRET)

    OPENER = urllib2.build_opener(HTTPS_HANDLER, AUTH_HANDLER)
    urllib2.install_opener(OPENER)

    try:
        result["EVENTID"] = EVENTID
    except Exception, e:
        result["EVENTID"] = "0-0-0-0-0"
        MSG = {"STATUS": {
            "TYPE": "Error",
            "MESSAGE": "Error setting EVENTID"
                }
            }
        print MSG
        pp.pprint(MSG)
        email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
        return MSG

    PAYLOAD = simplejson.dumps(result)
    DATA = {"toapp": APPLICATION_ID,
        "payload": PAYLOAD,
        "wakeup": "true",
        "expires-at": "86400",
        }

    try:
        F = urllib2.urlopen(URL, urlencode(DATA))
    except urllib2.HTTPError, e:
        MSG = {"STATUS": {
            "TYPE": "Error",
            "MESSAGE": "HTTP error " + str(e.code)
                        }
                }
        email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
    except urllib2.URLError, e:
        MSG = {"STATUS": {
            "TYPE": "Error",
            "MESSAGE": "URL error " + str(e.reason)
                        }
            }
        email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
    else:
        response = F.read()
        # If response is empty the HTTP result code is 204 which means that the
        # recipient application is unreachable
        try:
            resp = int(response)
        except ValueError, e:
            resp = response

        if resp in rest_repsonse.keys():
            reason = rest_repsonse[resp]
            MSG = {"STATUS": {
                "TYPE": "Success",
                "MESSAGE": "%(c)s %(r)s" % {"c": resp, "r": reason},
                            }
                    }
        elif resp in rest_error.keys():
            reason = rest_error[resp]
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "%(c)s %(r)s" % {"c": resp, "r": reason},
                            }
                    }
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
        else:
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "Unkown REST response: %(r)s" % {"r": resp}
                            }
                    }
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
        F.close()

    print MSG
    pp.pprint(MSG)
    return MSG


class RunPlugin():
    # This is a blocking method
    def load(self, PROVIDERID, LOGIN, PASSWORD, NUMBERS, SMSTEXT, PLUGINDICT):
        print "Running ", PROVIDERID
        try:
            P = __import__(PROVIDERID)
        except Exception, e:
            MSG = {"STATUS": {
                "TYPE": "error",
                "MESSAGE": "Failed to load plugin %(p)s" % {"p": PROVIDERID}
                    }
                }
            print MSG
            pp.pprint(MSG)
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG
        PV = P.__version__
        print "%(p)s plugin version: %(pv)s" % {"p": PROVIDERID, "pv": PV}
        try:
            PMSG = P.provider().sendSMS(
                LOGIN,
                PASSWORD,
                NUMBERS,
                SMSTEXT,
                PLUGINDICT
                )
        except Exception, e:
            what = "- %(p)s - Faild to execute!\n" % {"p": PROVIDERID}
            reason = "Exception:\n%(err)s\n\n" % {"err": e}
            try:
                preturned = "Plugin return:\n%(ret)s\n\n" % {"ret": PMSG}
            except Exception, e:
                preturned = "Plugin returned nothing!\n\n"
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": what + reason + preturned
                    }
                }
            print MSG
            pp.pprint(MSG)
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG
        else:
            print PMSG
            return PMSG

    # Lets run the blocking method using Deferred...
    def run(self, DATA):
        PROVIDER = DATA["PROVIDERID"]
        p = threads.deferToThread(self.load,
            PROVIDER,
            DATA["LOGIN"],
            DATA["PASSWORD"],
            DATA["NUMBERS"],
            DATA["SMSTEXT"],
            DATA["PLUGINDICT"]
            )

        PUSHID = DATA["PUSHID"]
        EVENTID = DATA["EVENTID"]

        p.addCallback(self.suc, PUSHID, EVENTID, PROVIDER)
        p.addErrback(self.err, PUSHID, EVENTID, PROVIDER)

        return p

    def suc(self, CALLBACK, PUSHID, EVENTID, PROVIDER):
        if USE_PUSH:
            push(CALLBACK, PUSHID, EVENTID, PROVIDER)
            try:
                STAT = CALLBACK["STATUS"]["TYPE"]
            except Exception, e:
                email(CALLBACK, ERROR_MAIL_TO)
            else:
                if STAT.lower() == "error":
                    subj = PROVIDER
                    email(CALLBACK, ERROR_MAIL_TO, subj)
        else:
            return CALLBACK

    def err(self, CALLBACK, PUSHID, EVENTID, PROVIDER):
        if USE_PUSH:
            push(CALLBACK, PUSHID, EVENTID, PROVIDER)
        else:
            return CALLBACK


# Setup the server
class SendPage(Resource):
    def checkParameters(self, data):
        if data["LOGIN"] == "DEBUG_USER":
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "Using DEBUG_USER"
                            }
                        }
            print "Using DEBUG_USER"
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG

        # Check the client version
        try:
            cv = data["CLIENT_VERSION"]
        except KeyError:
            # Check if the password is empty. If it is, this indicates that
            # the client is using the buggy 1.0.0 client version
            pwtest = data["PASSWORD"]
            if not pwtest:
                cv = "1.0.0"
            else:
                # Client version 1.0.1 does not yet have the CLIENT_VERSION set
                cv = "1.0.1"

        cvi = int(cv.replace(".", ""))
        print "Client version: %(cv)s (%(cvi)s)" % {"cv": cv, "cvi": cvi}

        if cvi in broken_clients.keys():
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": broken_clients[cvi]
                    }
                }
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG

        # Check for valid API ID
        try:
            API_ID = data["WEB2SMS_API"]
        except KeyError, e:
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "Connection refused"
                            }
                        }
            print "Missing API ID"
            return MSG
        else:
            if API_ID != WEB2SMS_API:
                MSG = {"STATUS": {
                    "TYPE": "Error",
                    "MESSAGE": "Connection refused"
                                }
                            }
                print "Valid API ID:"
                print WEB2SMS_API
                print "Invalid API ID:"
                print API_ID
                return MSG

        # Ensure numbers is a list type
        if not isinstance(data["NUMBERS"], list):
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "NUMBERS is not a list!"
                    }
                }
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG

        # Check for a Nokia account
        nokia_acc = data["PUSHID"]
        if not nokia_acc:
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "Missing Nokia Account, please create on first!"
                    }
                }
            email(MSG, ERROR_MAIL_TO, MSG["STATUS"]["MESSAGE"])
            return MSG

        # If everything is O.K. we return an empty MSG
        MSG = {}

    def render_POST(self, request):
        DATA = simplejson.loads(request.content.getvalue(), encoding="utf-8")
        # Check various parameters
        MSG = self.checkParameters(DATA)
        if MSG:
            print MSG
            pp.pprint(MSG)
            return simplejson.dumps(MSG)
        else:
            PLUGIN = RunPlugin()
            S = PLUGIN.run(DATA)

        # Method called when the client needs feedback
        # Only used when _NOT_ using PUSH
        def send(result):
            STATUS = simplejson.dumps(result)
            request.write(STATUS)
            request.finish()

        if USE_PUSH:
            # If PUSH is working, we disconnect the client ASAP
            # and send the response via PUSH
            MSG = {"STATUS": {"TYPE": "delayed"}}
            STATUS = simplejson.dumps(MSG)
            return STATUS
        else:
            # As long as PUSH is not working,
            # we keep the client waiting for response
            S.addCallback(send)
            return NOT_DONE_YET


class NotifyPage(Resource):
    def render_POST(self, request):
        MSG = "This is the notification API"
        return MSG


class RegisterPage(Resource):
    def render_POST(self, request):
        return "This is the registration API"


class PluginsPage(Resource):
    def render_POST(self, request):
        return "This is the plugin lister API"


class UpdatePage(Resource):
    def get_file_attributes(self):
        t_start = time.time()
        md5 = hashlib.md5()
        try:
            f = open(UPDATE_ZIP, 'rb')
        except IOError:
            MSG = {"STATUS": {
                "TYPE": "Error",
                "MESSAGE": "%(f)s not found" % {"f": UPDATE_ZIP}
                                                },
                "FILE": False,
                "DATE": False,
                "MD5": False,
                "PTIME:": False
                        }
        else:
            for line in f:
                md5.update(line)

            md5sum = md5.hexdigest()
            ts = os.path.getctime(UPDATE_ZIP)
            t_stop = time.time() - t_start

            MSG = {"STATUS": {
                    "TYPE": "success",
                    "MESSAGE": "%(f)s found" % {"f": UPDATE_ZIP}
                                                },
                "FILE": UPDATE_ZIP_URL,
                "DATE": ts,
                "MD5": md5sum,
                "PTIME:": t_stop
                        }

        return simplejson.dumps(MSG)

    def render_POST(self, request):
        MSG = self.get_file_attributes()
        return MSG


#Initiate and start the server
root = Resource()
root.putChild("send", SendPage())
root.putChild("notify", NotifyPage())
root.putChild("register", RegisterPage())
root.putChild("plugins", PluginsPage())
root.putChild("update", UpdatePage())

application = Application("web2sms")

if USE_SSL:
    myContextFactory = ssl.DefaultOpenSSLContextFactory(
        '%(P)s/%(K)s' % {"P": KEY_PATH, "K": KEY},
        '%(P)s/%(K)s' % {"P": KEY_PATH, "K": CRT}
        )

    reactor.listenSSL(PORT,
        Site(root),
        myContextFactory
        )
else:
    reactor.listenTCP(PORT, Site(root))

#reactor.run()
